@@ -21,15 +21,14 @@
 #endif
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdarg.h>
+#include <stdlib.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <unistd.h>
-#ifdef HAVE_SYS_STAT_H
+#include <dlfcn.h>
 #include <sys/stat.h>
-#endif
 #ifdef HAVE_SECURITY_SECURITY_H
 #include <Security/Security.h>
 #endif
@@ -43,11 +42,8 @@
 #include "winbase.h"
 #include "winternl.h"
 #include "wincrypt.h"
-#include "snmp.h"
 #include "crypt32_private.h"
-
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(crypt);
 
@@ -84,7 +80,7 @@ static void gnutls_log( int level, const char *msg )
     TRACE( "<%d> %s", level, msg );
 }
 
-static BOOL gnutls_initialize(void)
+static NTSTATUS process_attach( void *args )
 {
     const char *env_str;
     int ret;
@@ -102,7 +98,7 @@ static BOOL gnutls_initialize(void)
     if (!(libgnutls_handle = dlopen( SONAME_LIBGNUTLS, RTLD_NOW )))
     {
         ERR_(winediag)( "failed to load libgnutls, no support for pfx import/export\n" );
-        return FALSE;
+        return STATUS_DLL_NOT_FOUND;
     }
 
 #define LOAD_FUNCPTR(f) \
@@ -143,52 +139,54 @@ static BOOL gnutls_initialize(void)
 fail:
     dlclose( libgnutls_handle );
     libgnutls_handle = NULL;
-    return FALSE;
+    return STATUS_DLL_INIT_FAILED;
 }
 
-static void gnutls_uninitialize(void)
+static NTSTATUS process_detach( void *args )
 {
     pgnutls_global_deinit();
     dlclose( libgnutls_handle );
     libgnutls_handle = NULL;
+    return STATUS_SUCCESS;
 }
 #define RSA_MAGIC_KEY  ('R' | ('S' << 8) | ('A' << 16) | ('2' << 24))
 #define RSA_PUBEXP     65537
 
-static DWORD import_key( gnutls_x509_privkey_t key, DWORD flags, void **data_ret )
+struct cert_store_data
 {
+    gnutls_pkcs12_t p12;
+    gnutls_x509_privkey_t key;
+    gnutls_x509_crt_t *chain;
+    unsigned int key_bitlen;
+    unsigned int chain_len;
+};
+
+static NTSTATUS import_store_key( void *args )
+{
+    struct import_store_key_params *params = args;
+    struct cert_store_data *data = params->data;
     int i, ret;
-    unsigned int bitlen;
+    unsigned int bitlen = data->key_bitlen;
     gnutls_datum_t m, e, d, p, q, u, e1, e2;
     BLOBHEADER *hdr;
     RSAPUBKEY *rsakey;
-    BYTE *buf, *src, *dst;
+    BYTE *src, *dst;
     DWORD size;
 
-    *data_ret = NULL;
-
-    if ((ret = pgnutls_x509_privkey_get_pk_algorithm2( key, &bitlen )) < 0)
-    {
-        pgnutls_perror( ret );
-        return 0;
-    }
-
-    if (ret != GNUTLS_PK_RSA)
-    {
-        FIXME( "key algorithm %u not supported\n", ret );
-        return 0;
-    }
-
-    if ((ret = pgnutls_x509_privkey_export_rsa_raw2( key, &m, &e, &d, &p, &q, &u, &e1, &e2 )) < 0)
-    {
-        pgnutls_perror( ret );
-        return 0;
-    }
-
     size = sizeof(*hdr) + sizeof(*rsakey) + (bitlen * 9 / 16);
-    if (!(buf = RtlAllocateHeap( GetProcessHeap(), 0, size ))) goto done;
+    if (!params->buf || *params->buf_size < size)
+    {
+        *params->buf_size = size;
+        return STATUS_BUFFER_TOO_SMALL;
+    }
 
-    hdr = (BLOBHEADER *)buf;
+    if ((ret = pgnutls_x509_privkey_export_rsa_raw2( data->key, &m, &e, &d, &p, &q, &u, &e1, &e2 )) < 0)
+    {
+        pgnutls_perror( ret );
+        return STATUS_INVALID_PARAMETER;
+    }
+
+    hdr = params->buf;
     hdr->bType    = PRIVATEKEYBLOB;
     hdr->bVersion = CUR_BLOB_VERSION;
     hdr->reserved = 0;
@@ -235,8 +233,6 @@ static DWORD import_key( gnutls_x509_privkey_t key, DWORD flags, void **data_ret
     else src = d.data;
     for (i = bitlen / 8 - 1; i >= 0; i--) *dst++ = src[i];
 
-    *data_ret = buf;
-
 done:
     free( m.data );
     free( e.data );
@@ -246,8 +242,7 @@ done:
     free( u.data );
     free( e1.data );
     free( e2.data );
-    if (!*data_ret) RtlFreeHeap( GetProcessHeap(), 0, buf );
-    return size;
+    return STATUS_SUCCESS;
 }
 
 static char *password_to_ascii( const WCHAR *str )
@@ -265,58 +260,103 @@ static char *password_to_ascii( const WCHAR *str )
     return ret;
 }
 
-static BOOL WINAPI import_cert_store( CRYPT_DATA_BLOB *pfx, const WCHAR *password, DWORD flags,
-                                      void **key_ret, void ***chain_ret, DWORD *count_ret )
+static NTSTATUS open_cert_store( void *args )
 {
+    struct open_cert_store_params *params = args;
     gnutls_pkcs12_t p12;
     gnutls_datum_t pfx_data;
     gnutls_x509_privkey_t key;
     gnutls_x509_crt_t *chain;
-    unsigned int chain_len, i;
+    unsigned int chain_len;
+    unsigned int bitlen;
     char *pwd = NULL;
     int ret;
+    struct cert_store_data *store_data;
 
-    if (password && !(pwd = password_to_ascii( password ))) return FALSE;
+    if (!libgnutls_handle) return STATUS_DLL_NOT_FOUND;
+    if (params->password && !(pwd = password_to_ascii( params->password ))) return STATUS_NO_MEMORY;
 
     if ((ret = pgnutls_pkcs12_init( &p12 )) < 0) goto error;
 
-    pfx_data.data = pfx->pbData;
-    pfx_data.size = pfx->cbData;
+    pfx_data.data = params->pfx->pbData;
+    pfx_data.size = params->pfx->cbData;
     if ((ret = pgnutls_pkcs12_import( p12, &pfx_data, GNUTLS_X509_FMT_DER, 0 )) < 0) goto error;
 
     if ((ret = pgnutls_pkcs12_simple_parse( p12, pwd ? pwd : "", &key, &chain, &chain_len, NULL, NULL, NULL, 0 )) < 0)
         goto error;
 
-    if (!import_key( key, flags, key_ret )) goto error;
+    if ((ret = pgnutls_x509_privkey_get_pk_algorithm2( key, &bitlen )) < 0)
+        goto error;
 
-    *chain_ret = RtlAllocateHeap( GetProcessHeap(), 0, chain_len * sizeof(*chain_ret) );
-    *count_ret = chain_len;
-    for (i = 0; i < chain_len; i++)
+    free( pwd );
+
+    if (ret != GNUTLS_PK_RSA)
     {
-        size_t size = 0;
-
-        if ((ret = pgnutls_x509_crt_export( chain[i], GNUTLS_X509_FMT_DER, NULL, &size )) != GNUTLS_E_SHORT_MEMORY_BUFFER)
-            goto error;
-
-        (*chain_ret)[i] = RtlAllocateHeap( GetProcessHeap(), 0, size );
-        if ((ret = pgnutls_x509_crt_export( chain[i], GNUTLS_X509_FMT_DER, (*chain_ret)[i], &size )) < 0)
-        {
-            i++;
-            while (i) RtlFreeHeap( GetProcessHeap(), 0, (*chain_ret)[--i] );
-            RtlFreeHeap( GetProcessHeap(), 0, *chain_ret );
-            goto error;
-        }
+        FIXME( "key algorithm %u not supported\n", ret );
+        pgnutls_pkcs12_deinit( p12 );
+        return STATUS_INVALID_PARAMETER;
     }
-    pgnutls_pkcs12_deinit( p12 );
-    return TRUE;
+
+    store_data = malloc( sizeof(*store_data) );
+    store_data->p12 = p12;
+    store_data->key = key;
+    store_data->chain = chain;
+    store_data->key_bitlen = bitlen;
+    store_data->chain_len = chain_len;
+    *params->data_ret = store_data;
+    return STATUS_SUCCESS;
 
 error:
     pgnutls_perror( ret );
     pgnutls_pkcs12_deinit( p12 );
     free( pwd );
-    return FALSE;
+    return STATUS_INVALID_PARAMETER;
 }
 
+static NTSTATUS import_store_cert( void *args )
+{
+    struct import_store_cert_params *params = args;
+    struct cert_store_data *data = params->data;
+    size_t size = 0;
+    int ret;
+
+    if (params->index >= data->chain_len) return STATUS_NO_MORE_ENTRIES;
+
+    if ((ret = pgnutls_x509_crt_export( data->chain[params->index], GNUTLS_X509_FMT_DER, NULL, &size )) != GNUTLS_E_SHORT_MEMORY_BUFFER)
+        return STATUS_INVALID_PARAMETER;
+
+    if (!params->buf || *params->buf_size < size)
+    {
+        *params->buf_size = size;
+        return STATUS_BUFFER_TOO_SMALL;
+    }
+    if ((ret = pgnutls_x509_crt_export( data->chain[params->index], GNUTLS_X509_FMT_DER, params->buf, &size )) < 0)
+        return STATUS_INVALID_PARAMETER;
+
+    return STATUS_SUCCESS;
+}
+
+static NTSTATUS close_cert_store( void *args )
+{
+    struct close_cert_store_params *params = args;
+
+    if (params->data)
+    {
+        pgnutls_pkcs12_deinit( params->data->p12 );
+        free( params->data );
+    }
+    return STATUS_SUCCESS;
+}
+
+#else /* SONAME_LIBGNUTLS */
+
+static NTSTATUS process_attach( void *args ) { return STATUS_SUCCESS; }
+static NTSTATUS process_detach( void *args ) { return STATUS_SUCCESS; }
+static NTSTATUS open_cert_store( void *args ) { return STATUS_DLL_NOT_FOUND; }
+static NTSTATUS import_store_key( void *args ) { return STATUS_DLL_NOT_FOUND; }
+static NTSTATUS import_store_cert( void *args ) { return STATUS_DLL_NOT_FOUND; }
+static NTSTATUS close_cert_store( void *args ) { return STATUS_DLL_NOT_FOUND; }
+
 #endif /* SONAME_LIBGNUTLS */
 
 struct root_cert
@@ -598,7 +638,7 @@ static void load_root_certs(void)
             {
                 SecCertificateRef cert = (SecCertificateRef)CFArrayGetValueAtIndex(certs, i);
                 CFDataRef certData;
-                if ((status = SecKeychainItemExport(cert, kSecFormatX509Cert, 0, NULL, &certData)) == noErr)
+                if ((status = SecItemExport(cert, kSecFormatX509Cert, 0, NULL, &certData)) == noErr)
                 {
                     BYTE *data = add_cert( CFDataGetLength(certData) );
                     if (data) memcpy( data, CFDataGetBytePtr(certData), CFDataGetLength(certData) );
@@ -616,8 +656,9 @@ static void load_root_certs(void)
         import_certs_from_path( CRYPT_knownLocations[i], TRUE );
 }
 
-static BOOL WINAPI enum_root_certs( void *buffer, SIZE_T size, SIZE_T *needed )
+static NTSTATUS enum_root_certs( void *args )
 {
+    struct enum_root_certs_params *params = args;
     static BOOL loaded;
     struct list *ptr;
     struct root_cert *cert;
@@ -625,39 +666,25 @@ static BOOL WINAPI enum_root_certs( void *buffer, SIZE_T size, SIZE_T *needed )
     if (!loaded) load_root_certs();
     loaded = TRUE;
 
-    if (!(ptr = list_head( &root_cert_list ))) return FALSE;
+    if (!(ptr = list_head( &root_cert_list ))) return STATUS_NO_MORE_ENTRIES;
     cert = LIST_ENTRY( ptr, struct root_cert, entry );
-    *needed = cert->size;
-    if (cert->size <= size)
+    *params->needed = cert->size;
+    if (cert->size <= params->size)
     {
-        memcpy( buffer, cert->data, cert->size );
+        memcpy( params->buffer, cert->data, cert->size );
         list_remove( &cert->entry );
         free( cert );
     }
-    return TRUE;
-}
-
-static struct unix_funcs funcs =
-{
-    enum_root_certs,
-    NULL
-};
-
-NTSTATUS CDECL __wine_init_unix_lib( HMODULE module, DWORD reason, const void *ptr_in, void *ptr_out )
-{
-    switch (reason)
-    {
-    case DLL_PROCESS_ATTACH:
-#ifdef SONAME_LIBGNUTLS
-        if (gnutls_initialize()) funcs.import_cert_store = import_cert_store;
-#endif
-        *(const struct unix_funcs **)ptr_out = &funcs;
-        break;
-    case DLL_PROCESS_DETACH:
-#ifdef SONAME_LIBGNUTLS
-        if (libgnutls_handle) gnutls_uninitialize();
-#endif
-        break;
-    }
     return STATUS_SUCCESS;
 }
+
+unixlib_entry_t __wine_unix_call_funcs[] =
+{
+    process_attach,
+    process_detach,
+    open_cert_store,
+    import_store_key,
+    import_store_cert,
+    close_cert_store,
+    enum_root_certs,
+};
