@@ -55,6 +55,8 @@ struct expect_queue
     struct hid_expect *end;
     struct hid_expect spurious;
     struct hid_expect *buffer;
+    IRP *pending_wait;
+    char context[64];
 };
 
 static void expect_queue_init( struct expect_queue *queue )
@@ -68,12 +70,31 @@ static void expect_queue_init( struct expect_queue *queue )
 
 static void expect_queue_cleanup( struct expect_queue *queue )
 {
+    KIRQL irql;
+    IRP *irp;
+
+    KeAcquireSpinLock( &queue->lock, &irql );
+    if ((irp = queue->pending_wait))
+    {
+        queue->pending_wait = NULL;
+        if (!IoSetCancelRoutine( irp, NULL )) irp = NULL;
+    }
+    KeReleaseSpinLock( &queue->lock, irql );
+
+    if (irp)
+    {
+        irp->IoStatus.Information = 0;
+        irp->IoStatus.Status = STATUS_DELETE_PENDING;
+        IoCompleteRequest( irp, IO_NO_INCREMENT );
+    }
+
     ExFreePool( queue->buffer );
 }
 
 static void expect_queue_reset( struct expect_queue *queue, void *buffer, unsigned int size )
 {
     struct hid_expect *missing, *missing_end, *tmp;
+    char context[64];
     KIRQL irql;
 
     missing = ExAllocatePool( PagedPool, EXPECT_QUEUE_BUFFER_SIZE );
@@ -89,12 +110,13 @@ static void expect_queue_reset( struct expect_queue *queue, void *buffer, unsign
 
     if (size) memcpy( queue->end, buffer, size );
     queue->end = queue->end + size / sizeof(struct hid_expect);
+    memcpy( context, queue->context, sizeof(context) );
     KeReleaseSpinLock( &queue->lock, irql );
 
     tmp = missing;
     while (tmp != missing_end)
     {
-        winetest_push_context( "%s expect[%d]", tmp->context, tmp - missing );
+        winetest_push_context( "%s expect[%d]", context, tmp - missing );
         if (tmp->broken)
         {
             todo_wine_if( tmp->todo )
@@ -103,7 +125,7 @@ static void expect_queue_reset( struct expect_queue *queue, void *buffer, unsign
         else
         {
             todo_wine_if( tmp->todo )
-            ok( 0, "missing (code %#x id %u len %u)\n", tmp->code, tmp->report_id, tmp->report_len );
+            ok( tmp->wine_only, "missing (code %#x id %u len %u)\n", tmp->code, tmp->report_id, tmp->report_len );
         }
         winetest_pop_context();
         tmp++;
@@ -112,13 +134,63 @@ static void expect_queue_reset( struct expect_queue *queue, void *buffer, unsign
     ExFreePool( missing );
 }
 
+static void WINAPI wait_cancel_routine( DEVICE_OBJECT *device, IRP *irp )
+{
+    struct expect_queue *queue = irp->Tail.Overlay.DriverContext[0];
+    KIRQL irql;
+
+    IoReleaseCancelSpinLock( irp->CancelIrql );
+
+    KeAcquireSpinLock( &queue->lock, &irql );
+    queue->pending_wait = NULL;
+    KeReleaseSpinLock( &queue->lock, irql );
+
+    irp->IoStatus.Information = 0;
+    irp->IoStatus.Status = STATUS_CANCELLED;
+    IoCompleteRequest( irp, IO_NO_INCREMENT );
+}
+
+static NTSTATUS expect_queue_wait( struct expect_queue *queue, IRP *irp )
+{
+    NTSTATUS status;
+    KIRQL irql;
+
+    KeAcquireSpinLock( &queue->lock, &irql );
+    if (queue->pos == queue->end)
+        status = STATUS_SUCCESS;
+    else
+    {
+        IoSetCancelRoutine( irp, wait_cancel_routine );
+        if (irp->Cancel && !IoSetCancelRoutine( irp, NULL ))
+            status = STATUS_CANCELLED;
+        else
+        {
+            irp->Tail.Overlay.DriverContext[0] = queue;
+            IoMarkIrpPending( irp );
+            queue->pending_wait = irp;
+            status = STATUS_PENDING;
+        }
+    }
+    KeReleaseSpinLock( &queue->lock, irql );
+
+    if (status == STATUS_SUCCESS)
+    {
+        irp->IoStatus.Status = STATUS_SUCCESS;
+        IoCompleteRequest( irp, IO_NO_INCREMENT );
+    }
+
+    return status;
+}
+
 static void expect_queue_next( struct expect_queue *queue, ULONG code, HID_XFER_PACKET *packet,
-                               LONG *index, struct hid_expect *expect, BOOL compare_buf )
+                               LONG *index, struct hid_expect *expect, BOOL compare_buf,
+                               char *context, ULONG context_size )
 {
     struct hid_expect *missing, *missing_end, *tmp;
     ULONG len = packet->reportBufferLen;
     BYTE *buf = packet->reportBuffer;
     BYTE id = packet->reportId;
+    IRP *irp = NULL;
     KIRQL irql;
 
     missing = ExAllocatePool( PagedPool, EXPECT_QUEUE_BUFFER_SIZE );
@@ -130,7 +202,7 @@ static void expect_queue_next( struct expect_queue *queue, ULONG code, HID_XFER_
     while (tmp < queue->end)
     {
         if (running_under_wine && !tmp->todo) break;
-        if (!running_under_wine && !tmp->broken) break;
+        if (!running_under_wine && !tmp->broken && !tmp->wine_only) break;
         if (tmp->code == code && tmp->report_id == id && tmp->report_len == len &&
             (!compare_buf || RtlCompareMemory( tmp->report_buf, buf, len ) == len))
             break;
@@ -140,19 +212,38 @@ static void expect_queue_next( struct expect_queue *queue, ULONG code, HID_XFER_
     if (tmp < queue->end) queue->pos = tmp + 1;
     else tmp = &queue->spurious;
     *expect = *tmp;
+
+    while (queue->pos < queue->end)
+    {
+        if (running_under_wine || !queue->pos->wine_only) break;
+        queue->pos++;
+    }
+    if (queue->pos == queue->end && (irp = queue->pending_wait))
+    {
+        queue->pending_wait = NULL;
+        if (!IoSetCancelRoutine( irp, NULL )) irp = NULL;
+    }
+    memcpy( context, queue->context, context_size );
     KeReleaseSpinLock( &queue->lock, irql );
 
-    ok( tmp != &queue->spurious, "got spurious packet\n" );
+    if (irp)
+    {
+        irp->IoStatus.Information = 0;
+        irp->IoStatus.Status = STATUS_SUCCESS;
+        IoCompleteRequest( irp, IO_NO_INCREMENT );
+    }
 
-    winetest_push_context( "%s expect[%d]", tmp->context, tmp - queue->buffer );
+    ok( tmp != &queue->spurious, "%s got spurious packet\n", context );
+
+    winetest_push_context( "%s expect[%d]", context, tmp - queue->buffer );
     todo_wine_if( tmp->todo )
-    ok( 1, "found code %#x id %u len %u\n", tmp->code, tmp->report_id, tmp->report_len );
+    ok( !tmp->wine_only, "found code %#x id %u len %u\n", tmp->code, tmp->report_id, tmp->report_len );
     winetest_pop_context();
 
     tmp = missing;
     while (tmp != missing_end)
     {
-        winetest_push_context( "%s expect[%d]", tmp->context, tmp - missing );
+        winetest_push_context( "%s expect[%d]", context, tmp - missing );
         if (tmp->broken)
         {
             todo_wine_if( tmp->todo )
@@ -161,7 +252,7 @@ static void expect_queue_next( struct expect_queue *queue, ULONG code, HID_XFER_
         else
         {
             todo_wine_if( tmp->todo )
-            ok( 0, "missing (code %#x id %u len %u)\n", tmp->code, tmp->report_id, tmp->report_len );
+            ok( tmp->wine_only, "missing (code %#x id %u len %u)\n", tmp->code, tmp->report_id, tmp->report_len );
         }
         winetest_pop_context();
         tmp++;
@@ -376,6 +467,25 @@ static NTSTATUS WINAPI driver_power( DEVICE_OBJECT *device, IRP *irp )
     return PoCallDriver( ext->NextDeviceObject, irp );
 }
 
+#define check_buffer( a, b ) check_buffer_( __LINE__, a, b )
+static void check_buffer_( int line, HID_XFER_PACKET *packet, struct hid_expect *expect )
+{
+    ULONG match_len, i;
+
+    match_len = RtlCompareMemory( packet->reportBuffer, expect->report_buf, expect->report_len );
+    ok( match_len == expect->report_len, "unexpected data:\n" );
+    if (match_len == expect->report_len) return;
+
+    for (i = 0; i < packet->reportBufferLen;)
+    {
+        char buffer[256], *buf = buffer;
+        buf += sprintf( buf, "%08x ", i );
+        do buf += sprintf( buf, " %02x", packet->reportBuffer[i] );
+        while (++i % 16 && i < packet->reportBufferLen);
+        ok( 0, "  %s\n", buffer );
+    }
+}
+
 static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
 {
     IO_STACK_LOCATION *stack = IoGetCurrentIrpStackLocation( irp );
@@ -385,6 +495,7 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
     ULONG out_size = stack->Parameters.DeviceIoControl.OutputBufferLength;
     const ULONG code = stack->Parameters.DeviceIoControl.IoControlCode;
     struct hid_expect expect = {0};
+    char context[64];
     NTSTATUS ret;
     BOOL removed;
     KIRQL irql;
@@ -477,13 +588,12 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
         ok( !out_size, "got output size %u\n", out_size );
         ok( packet->reportBufferLen >= expected_size, "got report size %u\n", packet->reportBufferLen );
 
-        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE );
-        winetest_push_context( "%s expect[%d]", expect.context, index );
-        ok( expect.code == code, "got %#x, expected %#x\n", expect.code, code );
+        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE, context, sizeof(context) );
+        winetest_push_context( "%s expect[%d]", context, index );
+        ok( code == expect.code, "got %#x, expected %#x\n", code, expect.code );
         ok( packet->reportId == expect.report_id, "got id %u\n", packet->reportId );
         ok( packet->reportBufferLen == expect.report_len, "got len %u\n", packet->reportBufferLen );
-        ok( RtlCompareMemory( packet->reportBuffer, expect.report_buf, expect.report_len ) == expect.report_len,
-            "unexpected data\n" );
+        check_buffer( packet, &expect );
         winetest_pop_context();
 
         irp->IoStatus.Information = expect.ret_length ? expect.ret_length : expect.report_len;
@@ -501,9 +611,9 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
         ok( packet->reportBufferLen >= expected_size, "got len %u\n", packet->reportBufferLen );
         ok( !!packet->reportBuffer, "got buffer %p\n", packet->reportBuffer );
 
-        expect_queue_next( &expect_queue, code, packet, &index, &expect, FALSE );
-        winetest_push_context( "%s expect[%d]", expect.context, index );
-        ok( expect.code == code, "got %#x, expected %#x\n", expect.code, code );
+        expect_queue_next( &expect_queue, code, packet, &index, &expect, FALSE, context, sizeof(context) );
+        winetest_push_context( "%s expect[%d]", context, index );
+        ok( code == expect.code, "got %#x, expected %#x\n", code, expect.code );
         ok( packet->reportId == expect.report_id, "got id %u\n", packet->reportId );
         ok( packet->reportBufferLen == expect.report_len, "got len %u\n", packet->reportBufferLen );
         winetest_pop_context();
@@ -524,13 +634,12 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
         ok( packet->reportBufferLen >= expected_size, "got len %u\n", packet->reportBufferLen );
         ok( !!packet->reportBuffer, "got buffer %p\n", packet->reportBuffer );
 
-        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE );
-        winetest_push_context( "%s expect[%d]", expect.context, index );
-        ok( expect.code == code, "got %#x, expected %#x\n", expect.code, code );
+        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE, context, sizeof(context) );
+        winetest_push_context( "%s expect[%d]", context, index );
+        ok( code == expect.code, "got %#x, expected %#x\n", code, expect.code );
         ok( packet->reportId == expect.report_id, "got id %u\n", packet->reportId );
         ok( packet->reportBufferLen == expect.report_len, "got len %u\n", packet->reportBufferLen );
-        ok( RtlCompareMemory( packet->reportBuffer, expect.report_buf, expect.report_len ) == expect.report_len,
-            "unexpected data\n" );
+        check_buffer( packet, &expect );
         winetest_pop_context();
 
         irp->IoStatus.Information = expect.ret_length ? expect.ret_length : expect.report_len;
@@ -548,9 +657,9 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
         ok( packet->reportBufferLen >= expected_size, "got len %u\n", packet->reportBufferLen );
         ok( !!packet->reportBuffer, "got buffer %p\n", packet->reportBuffer );
 
-        expect_queue_next( &expect_queue, code, packet, &index, &expect, FALSE );
-        winetest_push_context( "%s expect[%d]", expect.context, index );
-        ok( expect.code == code, "got %#x, expected %#x\n", expect.code, code );
+        expect_queue_next( &expect_queue, code, packet, &index, &expect, FALSE, context, sizeof(context) );
+        winetest_push_context( "%s expect[%d]", context, index );
+        ok( code == expect.code, "got %#x, expected %#x\n", code, expect.code );
         ok( packet->reportId == expect.report_id, "got id %u\n", packet->reportId );
         ok( packet->reportBufferLen == expect.report_len, "got len %u\n", packet->reportBufferLen );
         winetest_pop_context();
@@ -571,13 +680,12 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
         ok( packet->reportBufferLen >= expected_size, "got len %u\n", packet->reportBufferLen );
         ok( !!packet->reportBuffer, "got buffer %p\n", packet->reportBuffer );
 
-        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE );
-        winetest_push_context( "%s expect[%d]", expect.context, index );
-        ok( expect.code == code, "got %#x, expected %#x\n", expect.code, code );
+        expect_queue_next( &expect_queue, code, packet, &index, &expect, TRUE, context, sizeof(context) );
+        winetest_push_context( "%s expect[%d]", context, index );
+        ok( code == expect.code, "got %#x, expected %#x\n", code, expect.code );
         ok( packet->reportId == expect.report_id, "got id %u\n", packet->reportId );
         ok( packet->reportBufferLen == expect.report_len, "got len %u\n", packet->reportBufferLen );
-        ok( RtlCompareMemory( packet->reportBuffer, expect.report_buf, expect.report_len ) == expect.report_len,
-            "unexpected data\n" );
+        check_buffer( packet, &expect );
         winetest_pop_context();
 
         irp->IoStatus.Information = expect.ret_length ? expect.ret_length : expect.report_len;
@@ -586,12 +694,14 @@ static NTSTATUS WINAPI driver_internal_ioctl( DEVICE_OBJECT *device, IRP *irp )
     }
 
     case IOCTL_HID_GET_STRING:
-        todo_wine
-        ok( 0, "unexpected IOCTL_HID_GET_STRING\n" );
+    {
+        static unsigned int once;
+        if (!once++) todo_wine ok( 0, "unexpected IOCTL_HID_GET_STRING\n" );
         memcpy( irp->UserBuffer, L"Wine Test", sizeof(L"Wine Test") );
         irp->IoStatus.Information = sizeof(L"Wine Test");
         ret = STATUS_SUCCESS;
         break;
+    }
 
     default:
         ok( 0, "unexpected ioctl %#x\n", code );
@@ -614,6 +724,7 @@ static NTSTATUS WINAPI driver_ioctl( DEVICE_OBJECT *device, IRP *irp )
     IO_STACK_LOCATION *stack = IoGetCurrentIrpStackLocation( irp );
     ULONG in_size = stack->Parameters.DeviceIoControl.InputBufferLength;
     ULONG code = stack->Parameters.DeviceIoControl.IoControlCode;
+    KIRQL irql;
 
     switch (code)
     {
@@ -622,8 +733,18 @@ static NTSTATUS WINAPI driver_ioctl( DEVICE_OBJECT *device, IRP *irp )
         irp->IoStatus.Status = STATUS_SUCCESS;
         IoCompleteRequest( irp, IO_NO_INCREMENT );
         return STATUS_SUCCESS;
+    case IOCTL_WINETEST_HID_WAIT_EXPECT:
+        return expect_queue_wait( &expect_queue, irp );
     case IOCTL_WINETEST_HID_SEND_INPUT:
         input_queue_reset( &input_queue, irp->AssociatedIrp.SystemBuffer, in_size );
+        irp->IoStatus.Status = STATUS_SUCCESS;
+        IoCompleteRequest( irp, IO_NO_INCREMENT );
+        return STATUS_SUCCESS;
+    case IOCTL_WINETEST_HID_SET_CONTEXT:
+        KeAcquireSpinLock( &expect_queue.lock, &irql );
+        memcpy( expect_queue.context, irp->AssociatedIrp.SystemBuffer, in_size );
+        KeReleaseSpinLock( &expect_queue.lock, irql );
+
         irp->IoStatus.Status = STATUS_SUCCESS;
         IoCompleteRequest( irp, IO_NO_INCREMENT );
         return STATUS_SUCCESS;
@@ -747,6 +868,12 @@ NTSTATUS WINAPI DriverEntry( DRIVER_OBJECT *driver, UNICODE_STRING *registry )
     ok( !ret, "ZwQueryValueKey returned %#x\n", ret );
     input_queue_reset( &input_queue, buffer + info_size, size - info_size );
 
+    RtlInitUnicodeString( &name_str, L"Context" );
+    size = info_size + sizeof(expect_queue.context);
+    ret = ZwQueryValueKey( hkey, &name_str, KeyValuePartialInformation, buffer, size, &size );
+    ok( !ret, "ZwQueryValueKey returned %#x\n", ret );
+    memcpy( expect_queue.context, buffer + info_size, size - info_size );
+
     driver->DriverExtension->AddDevice = driver_add_device;
     driver->DriverUnload = driver_unload;
     driver->MajorFunction[IRP_MJ_PNP] = driver_pnp;
