@@ -18,10 +18,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define COBJMACROS
+#if 0
+#pragma makedep unix
+#endif
 
 #include "config.h"
-#include "wine/port.h"
+
+#include <sys/types.h>
+#include <dlfcn.h>
 
 #ifdef HAVE_FT2BUILD_H
 #include <ft2build.h>
@@ -29,8 +33,11 @@
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
 #include FT_TRUETYPE_TABLES_H
+#include FT_SIZES_H
 #endif /* HAVE_FT2BUILD_H */
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "wine/debug.h"
 
@@ -40,14 +47,14 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dwrite);
 
-static CRITICAL_SECTION freetype_cs;
-static CRITICAL_SECTION_DEBUG critsect_debug =
+static RTL_CRITICAL_SECTION freetype_cs;
+static RTL_CRITICAL_SECTION_DEBUG critsect_debug =
 {
     0, 0, &freetype_cs,
     { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
       0, 0, { (DWORD_PTR)(__FILE__ ": freetype_cs") }
 };
-static CRITICAL_SECTION freetype_cs = { &critsect_debug, -1, 0, 0, 0, 0 };
+static RTL_CRITICAL_SECTION freetype_cs = { &critsect_debug, -1, 0, 0, 0, 0 };
 
 static void *ft_handle = NULL;
 static FT_Library library = 0;
@@ -60,9 +67,14 @@ typedef struct
     FT_Int patch;
 } FT_Version_t;
 
+static const struct font_callback_funcs *callback_funcs;
+
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f = NULL
+MAKE_FUNCPTR(FT_Activate_Size);
+MAKE_FUNCPTR(FT_Done_Face);
 MAKE_FUNCPTR(FT_Done_FreeType);
 MAKE_FUNCPTR(FT_Done_Glyph);
+MAKE_FUNCPTR(FT_Done_Size);
 MAKE_FUNCPTR(FT_Get_First_Char);
 MAKE_FUNCPTR(FT_Get_Kerning);
 MAKE_FUNCPTR(FT_Get_Sfnt_Table);
@@ -73,7 +85,9 @@ MAKE_FUNCPTR(FT_Init_FreeType);
 MAKE_FUNCPTR(FT_Library_Version);
 MAKE_FUNCPTR(FT_Load_Glyph);
 MAKE_FUNCPTR(FT_Matrix_Multiply);
+MAKE_FUNCPTR(FT_MulDiv);
 MAKE_FUNCPTR(FT_New_Memory_Face);
+MAKE_FUNCPTR(FT_New_Size);
 MAKE_FUNCPTR(FT_Outline_Copy);
 MAKE_FUNCPTR(FT_Outline_Decompose);
 MAKE_FUNCPTR(FT_Outline_Done);
@@ -82,6 +96,7 @@ MAKE_FUNCPTR(FT_Outline_Get_Bitmap);
 MAKE_FUNCPTR(FT_Outline_New);
 MAKE_FUNCPTR(FT_Outline_Transform);
 MAKE_FUNCPTR(FT_Outline_Translate);
+MAKE_FUNCPTR(FT_Set_Pixel_Sizes);
 MAKE_FUNCPTR(FTC_ImageCache_Lookup);
 MAKE_FUNCPTR(FTC_ImageCache_New);
 MAKE_FUNCPTR(FTC_Manager_New);
@@ -92,98 +107,82 @@ MAKE_FUNCPTR(FTC_Manager_RemoveFaceID);
 #undef MAKE_FUNCPTR
 static FT_Error (*pFT_Outline_EmboldenXY)(FT_Outline *, FT_Pos, FT_Pos);
 
-struct face_finalizer_data
-{
-    IDWriteFontFileStream *stream;
-    void *context;
-};
-
 static void face_finalizer(void *object)
 {
     FT_Face face = object;
-    struct face_finalizer_data *data = (struct face_finalizer_data *)face->generic.data;
-
-    IDWriteFontFileStream_ReleaseFileFragment(data->stream, data->context);
-    IDWriteFontFileStream_Release(data->stream);
-    heap_free(data);
+    callback_funcs->release_font_data((struct font_data_context *)face->generic.data);
 }
 
 static FT_Error face_requester(FTC_FaceID face_id, FT_Library library, FT_Pointer request_data, FT_Face *face)
 {
-    IDWriteFontFace *fontface = (IDWriteFontFace*)face_id;
-    IDWriteFontFileStream *stream;
-    IDWriteFontFile *file;
+    struct font_data_context *context;
     const void *data_ptr;
-    UINT32 index, count;
     FT_Error fterror;
     UINT64 data_size;
-    void *context;
-    HRESULT hr;
+    UINT32 index;
 
     *face = NULL;
 
-    if (!fontface) {
+    if (!face_id)
+    {
         WARN("NULL fontface requested.\n");
         return FT_Err_Ok;
     }
 
-    count = 1;
-    hr = IDWriteFontFace_GetFiles(fontface, &count, &file);
-    if (FAILED(hr))
+    if (callback_funcs->get_font_data(face_id, &data_ptr, &data_size, &index, &context))
         return FT_Err_Ok;
 
-    hr = get_filestream_from_file(file, &stream);
-    IDWriteFontFile_Release(file);
-    if (FAILED(hr))
-        return FT_Err_Ok;
-
-    hr = IDWriteFontFileStream_GetFileSize(stream, &data_size);
-    if (FAILED(hr)) {
-        fterror = FT_Err_Invalid_Stream_Read;
-        goto fail;
-    }
-
-    hr = IDWriteFontFileStream_ReadFileFragment(stream, &data_ptr, 0, data_size, &context);
-    if (FAILED(hr)) {
-        fterror = FT_Err_Invalid_Stream_Read;
-        goto fail;
-    }
-
-    index = IDWriteFontFace_GetIndex(fontface);
     fterror = pFT_New_Memory_Face(library, data_ptr, data_size, index, face);
-    if (fterror == FT_Err_Ok) {
-        struct face_finalizer_data *data;
-
-        data = heap_alloc(sizeof(*data));
-        data->stream = stream;
-        data->context = context;
-
-        (*face)->generic.data = data;
+    if (fterror == FT_Err_Ok)
+    {
+        (*face)->generic.data = context;
         (*face)->generic.finalizer = face_finalizer;
-        return fterror;
     }
     else
-        IDWriteFontFileStream_ReleaseFileFragment(stream, context);
-
-fail:
-    IDWriteFontFileStream_Release(stream);
+        callback_funcs->release_font_data(context);
 
     return fterror;
 }
 
-BOOL init_freetype(void)
+static FT_Size freetype_set_face_size(FT_Face face, FT_UInt emsize)
+{
+    FT_Size size;
+
+    if (pFT_New_Size(face, &size)) return NULL;
+
+    pFT_Activate_Size(size);
+
+    if (pFT_Set_Pixel_Sizes(face, emsize, emsize))
+    {
+        pFT_Done_Size(size);
+        return NULL;
+    }
+
+    return size;
+}
+
+static BOOL freetype_glyph_has_contours(FT_Face face)
+{
+    return face->glyph->format == FT_GLYPH_FORMAT_OUTLINE && face->glyph->outline.n_contours;
+}
+
+static BOOL init_freetype(void)
 {
     FT_Version_t FT_Version;
 
     ft_handle = dlopen(SONAME_LIBFREETYPE, RTLD_NOW);
-    if (!ft_handle) {
+    if (!ft_handle)
+    {
         WINE_MESSAGE("Wine cannot find the FreeType font library.\n");
-	return FALSE;
+        return FALSE;
     }
 
 #define LOAD_FUNCPTR(f) if((p##f = dlsym(ft_handle, #f)) == NULL){WARN("Can't find symbol %s\n", #f); goto sym_not_found;}
+    LOAD_FUNCPTR(FT_Activate_Size)
+    LOAD_FUNCPTR(FT_Done_Face)
     LOAD_FUNCPTR(FT_Done_FreeType)
     LOAD_FUNCPTR(FT_Done_Glyph)
+    LOAD_FUNCPTR(FT_Done_Size)
     LOAD_FUNCPTR(FT_Get_First_Char)
     LOAD_FUNCPTR(FT_Get_Kerning)
     LOAD_FUNCPTR(FT_Get_Sfnt_Table)
@@ -194,7 +193,9 @@ BOOL init_freetype(void)
     LOAD_FUNCPTR(FT_Library_Version)
     LOAD_FUNCPTR(FT_Load_Glyph)
     LOAD_FUNCPTR(FT_Matrix_Multiply)
+    LOAD_FUNCPTR(FT_MulDiv)
     LOAD_FUNCPTR(FT_New_Memory_Face)
+    LOAD_FUNCPTR(FT_New_Size)
     LOAD_FUNCPTR(FT_Outline_Copy)
     LOAD_FUNCPTR(FT_Outline_Decompose)
     LOAD_FUNCPTR(FT_Outline_Done)
@@ -203,6 +204,7 @@ BOOL init_freetype(void)
     LOAD_FUNCPTR(FT_Outline_New)
     LOAD_FUNCPTR(FT_Outline_Transform)
     LOAD_FUNCPTR(FT_Outline_Translate)
+    LOAD_FUNCPTR(FT_Set_Pixel_Sizes)
     LOAD_FUNCPTR(FTC_ImageCache_Lookup)
     LOAD_FUNCPTR(FTC_ImageCache_New)
     LOAD_FUNCPTR(FTC_Manager_New)
@@ -243,93 +245,124 @@ sym_not_found:
     return FALSE;
 }
 
-void release_freetype(void)
+static font_object_handle CDECL freetype_create_font_object(const void *data_ptr, UINT64 data_size, unsigned int index)
 {
-    pFTC_Manager_Done(cache_manager);
-    pFT_Done_FreeType(library);
+    FT_Face face = NULL;
+    FT_Error fterror;
+
+    fterror = pFT_New_Memory_Face(library, data_ptr, data_size, index, &face);
+    if (fterror != FT_Err_Ok)
+        WARN("Failed to create a face object, error %d.\n", fterror);
+
+    return face;
 }
 
-void freetype_notify_cacheremove(IDWriteFontFace5 *fontface)
+static void CDECL freetype_release_font_object(font_object_handle object)
 {
-    EnterCriticalSection(&freetype_cs);
-    pFTC_Manager_RemoveFaceID(cache_manager, fontface);
-    LeaveCriticalSection(&freetype_cs);
+    pFT_Done_Face(object);
 }
 
-HRESULT freetype_get_design_glyph_metrics(struct dwrite_fontface *fontface, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
+static void CDECL freetype_notify_release(void *key)
 {
-    FTC_ScalerRec scaler;
+    RtlEnterCriticalSection(&freetype_cs);
+    pFTC_Manager_RemoveFaceID(cache_manager, key);
+    RtlLeaveCriticalSection(&freetype_cs);
+}
+
+static void CDECL freetype_get_design_glyph_metrics(font_object_handle object, UINT16 upem, UINT16 ascent,
+        unsigned int simulations, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
+{
+    FT_Face face = object;
     FT_Size size;
 
-    scaler.face_id = &fontface->IDWriteFontFace5_iface;
-    scaler.width = fontface->metrics.designUnitsPerEm;
-    scaler.height = fontface->metrics.designUnitsPerEm;
-    scaler.pixel = 1;
-    scaler.x_res = 0;
-    scaler.y_res = 0;
+    if (!(size = freetype_set_face_size(face, upem)))
+        return;
 
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_Manager_LookupSize(cache_manager, &scaler, &size) == 0) {
-         if (pFT_Load_Glyph(size->face, glyph, FT_LOAD_NO_SCALE) == 0) {
-             FT_Glyph_Metrics *metrics = &size->face->glyph->metrics;
+    if (!pFT_Load_Glyph(face, glyph, FT_LOAD_NO_SCALE))
+    {
+        FT_Glyph_Metrics *metrics = &face->glyph->metrics;
 
-             ret->leftSideBearing = metrics->horiBearingX;
-             ret->advanceWidth = metrics->horiAdvance;
-             ret->rightSideBearing = metrics->horiAdvance - metrics->horiBearingX - metrics->width;
+        ret->leftSideBearing = metrics->horiBearingX;
+        ret->advanceWidth = metrics->horiAdvance;
+        ret->rightSideBearing = metrics->horiAdvance - metrics->horiBearingX - metrics->width;
 
-             ret->advanceHeight = metrics->vertAdvance;
-             ret->verticalOriginY = fontface->typo_metrics.ascent;
-             ret->topSideBearing = fontface->typo_metrics.ascent - metrics->horiBearingY;
-             ret->bottomSideBearing = metrics->vertAdvance - metrics->height - ret->topSideBearing;
+        ret->advanceHeight = metrics->vertAdvance;
+        ret->verticalOriginY = ascent;
+        ret->topSideBearing = ascent - metrics->horiBearingY;
+        ret->bottomSideBearing = metrics->vertAdvance - metrics->height - ret->topSideBearing;
 
-             /* Adjust in case of bold simulation, glyphs without contours are ignored. */
-             if (fontface->simulations & DWRITE_FONT_SIMULATIONS_BOLD &&
-                     size->face->glyph->format == FT_GLYPH_FORMAT_OUTLINE && size->face->glyph->outline.n_contours)
-             {
-                 if (ret->advanceWidth)
-                     ret->advanceWidth += (fontface->metrics.designUnitsPerEm + 49) / 50;
-             }
-         }
+        /* Adjust in case of bold simulation, glyphs without contours are ignored. */
+        if (simulations & DWRITE_FONT_SIMULATIONS_BOLD && freetype_glyph_has_contours(face))
+        {
+            if (ret->advanceWidth)
+                ret->advanceWidth += (upem + 49) / 50;
+        }
     }
-    LeaveCriticalSection(&freetype_cs);
 
-    return S_OK;
+    pFT_Done_Size(size);
 }
 
-struct decompose_context {
-    IDWriteGeometrySink *sink;
-    D2D1_POINT_2F offset;
+struct decompose_context
+{
+    struct dwrite_outline *outline;
     BOOL figure_started;
     BOOL move_to;     /* last call was 'move_to' */
     FT_Vector origin; /* 'pen' position from last call */
 };
 
-static inline void ft_vector_to_d2d_point(const FT_Vector *v, D2D1_POINT_2F offset, D2D1_POINT_2F *p)
+static inline void ft_vector_to_d2d_point(const FT_Vector *v, D2D1_POINT_2F *p)
 {
-    p->x = (v->x / 64.0f) + offset.x;
-    p->y = (v->y / 64.0f) + offset.y;
+    p->x = v->x / 64.0f;
+    p->y = v->y / 64.0f;
 }
 
-static void decompose_beginfigure(struct decompose_context *ctxt)
+static int dwrite_outline_push_tag(struct dwrite_outline *outline, unsigned char tag)
+{
+    if (outline->tags.size < outline->tags.count + 1)
+        return 1;
+
+    outline->tags.values[outline->tags.count++] = tag;
+
+    return 0;
+}
+
+static int dwrite_outline_push_points(struct dwrite_outline *outline, const D2D1_POINT_2F *points, unsigned int count)
+{
+    if (outline->points.size < outline->points.count + count)
+        return 1;
+
+    memcpy(&outline->points.values[outline->points.count], points, sizeof(*points) * count);
+    outline->points.count += count;
+
+    return 0;
+}
+
+static int decompose_beginfigure(struct decompose_context *ctxt)
 {
     D2D1_POINT_2F point;
+    int ret;
 
     if (!ctxt->move_to)
-        return;
+        return 0;
 
-    ft_vector_to_d2d_point(&ctxt->origin, ctxt->offset, &point);
-    ID2D1SimplifiedGeometrySink_BeginFigure(ctxt->sink, point, D2D1_FIGURE_BEGIN_FILLED);
+    ft_vector_to_d2d_point(&ctxt->origin, &point);
+    if ((ret = dwrite_outline_push_tag(ctxt->outline, OUTLINE_BEGIN_FIGURE))) return ret;
+    if ((ret = dwrite_outline_push_points(ctxt->outline, &point, 1))) return ret;
 
     ctxt->figure_started = TRUE;
     ctxt->move_to = FALSE;
+
+    return 0;
 }
 
 static int decompose_move_to(const FT_Vector *to, void *user)
 {
-    struct decompose_context *ctxt = (struct decompose_context*)user;
+    struct decompose_context *ctxt = (struct decompose_context *)user;
+    int ret;
 
-    if (ctxt->figure_started) {
-        ID2D1SimplifiedGeometrySink_EndFigure(ctxt->sink, D2D1_FIGURE_END_CLOSED);
+    if (ctxt->figure_started)
+    {
+        if ((ret = dwrite_outline_push_tag(ctxt->outline, OUTLINE_END_FIGURE))) return ret;
         ctxt->figure_started = FALSE;
     }
 
@@ -340,17 +373,19 @@ static int decompose_move_to(const FT_Vector *to, void *user)
 
 static int decompose_line_to(const FT_Vector *to, void *user)
 {
-    struct decompose_context *ctxt = (struct decompose_context*)user;
+    struct decompose_context *ctxt = (struct decompose_context *)user;
     D2D1_POINT_2F point;
+    int ret;
 
     /* Special case for empty contours, in a way freetype returns them. */
     if (ctxt->move_to && !memcmp(to, &ctxt->origin, sizeof(*to)))
         return 0;
 
-    decompose_beginfigure(ctxt);
+    ft_vector_to_d2d_point(to, &point);
 
-    ft_vector_to_d2d_point(to, ctxt->offset, &point);
-    ID2D1SimplifiedGeometrySink_AddLines(ctxt->sink, &point, 1);
+    if ((ret = decompose_beginfigure(ctxt))) return ret;
+    if ((ret = dwrite_outline_push_points(ctxt->outline, &point, 1))) return ret;
+    if ((ret = dwrite_outline_push_tag(ctxt->outline, OUTLINE_LINE))) return ret;
 
     ctxt->origin = *to;
     return 0;
@@ -358,11 +393,13 @@ static int decompose_line_to(const FT_Vector *to, void *user)
 
 static int decompose_conic_to(const FT_Vector *control, const FT_Vector *to, void *user)
 {
-    struct decompose_context *ctxt = (struct decompose_context*)user;
+    struct decompose_context *ctxt = (struct decompose_context *)user;
     D2D1_POINT_2F points[3];
     FT_Vector cubic[3];
+    int ret;
 
-    decompose_beginfigure(ctxt);
+    if ((ret = decompose_beginfigure(ctxt)))
+        return ret;
 
     /* convert from quadratic to cubic */
 
@@ -394,10 +431,11 @@ static int decompose_conic_to(const FT_Vector *control, const FT_Vector *to, voi
     cubic[1].y += (to->y + 1) / 3;
     cubic[2] = *to;
 
-    ft_vector_to_d2d_point(cubic, ctxt->offset, points);
-    ft_vector_to_d2d_point(cubic + 1, ctxt->offset, points + 1);
-    ft_vector_to_d2d_point(cubic + 2, ctxt->offset, points + 2);
-    ID2D1SimplifiedGeometrySink_AddBeziers(ctxt->sink, (D2D1_BEZIER_SEGMENT*)points, 1);
+    ft_vector_to_d2d_point(cubic, points);
+    ft_vector_to_d2d_point(cubic + 1, points + 1);
+    ft_vector_to_d2d_point(cubic + 2, points + 2);
+    if ((ret = dwrite_outline_push_points(ctxt->outline, points, 3))) return ret;
+    if ((ret = dwrite_outline_push_tag(ctxt->outline, OUTLINE_BEZIER))) return ret;
     ctxt->origin = *to;
     return 0;
 }
@@ -405,22 +443,28 @@ static int decompose_conic_to(const FT_Vector *control, const FT_Vector *to, voi
 static int decompose_cubic_to(const FT_Vector *control1, const FT_Vector *control2,
     const FT_Vector *to, void *user)
 {
-    struct decompose_context *ctxt = (struct decompose_context*)user;
+    struct decompose_context *ctxt = (struct decompose_context *)user;
     D2D1_POINT_2F points[3];
+    int ret;
 
-    decompose_beginfigure(ctxt);
+    if ((ret = decompose_beginfigure(ctxt)))
+        return ret;
 
-    ft_vector_to_d2d_point(control1, ctxt->offset, points);
-    ft_vector_to_d2d_point(control2, ctxt->offset, points + 1);
-    ft_vector_to_d2d_point(to, ctxt->offset, points + 2);
-    ID2D1SimplifiedGeometrySink_AddBeziers(ctxt->sink, (D2D1_BEZIER_SEGMENT*)points, 1);
+    ft_vector_to_d2d_point(control1, points);
+    ft_vector_to_d2d_point(control2, points + 1);
+    ft_vector_to_d2d_point(to, points + 2);
     ctxt->origin = *to;
+
+    if ((ret = dwrite_outline_push_points(ctxt->outline, points, 3))) return ret;
+    if ((ret = dwrite_outline_push_tag(ctxt->outline, OUTLINE_BEZIER))) return ret;
+
     return 0;
 }
 
-static void decompose_outline(FT_Outline *outline, D2D1_POINT_2F offset, IDWriteGeometrySink *sink)
+static int decompose_outline(FT_Outline *ft_outline, struct dwrite_outline *outline)
 {
-    static const FT_Outline_Funcs decompose_funcs = {
+    static const FT_Outline_Funcs decompose_funcs =
+    {
         decompose_move_to,
         decompose_line_to,
         decompose_conic_to,
@@ -428,26 +472,24 @@ static void decompose_outline(FT_Outline *outline, D2D1_POINT_2F offset, IDWrite
         0,
         0
     };
-    struct decompose_context context;
+    struct decompose_context context = { 0 };
+    int ret;
 
-    context.sink = sink;
-    context.offset = offset;
-    context.figure_started = FALSE;
-    context.move_to = FALSE;
-    context.origin.x = 0;
-    context.origin.y = 0;
+    context.outline = outline;
 
-    pFT_Outline_Decompose(outline, &decompose_funcs, &context);
+    ret = pFT_Outline_Decompose(ft_outline, &decompose_funcs, &context);
 
-    if (context.figure_started)
-        ID2D1SimplifiedGeometrySink_EndFigure(sink, D2D1_FIGURE_END_CLOSED);
+    if (!ret && context.figure_started)
+        ret = dwrite_outline_push_tag(outline, OUTLINE_END_FIGURE);
+
+    return ret;
 }
 
 static void embolden_glyph_outline(FT_Outline *outline, FLOAT emsize)
 {
     FT_Pos strength;
 
-    strength = MulDiv(emsize, 1 << 6, 24);
+    strength = pFT_MulDiv(emsize, 1 << 6, 24);
     if (pFT_Outline_EmboldenXY)
         pFT_Outline_EmboldenXY(outline, strength, 0);
     else
@@ -464,133 +506,52 @@ static void embolden_glyph(FT_Glyph glyph, FLOAT emsize)
     embolden_glyph_outline(&outline_glyph->outline, emsize);
 }
 
-HRESULT freetype_get_glyphrun_outline(IDWriteFontFace5 *fontface, float emSize, UINT16 const *glyphs,
-        float const *advances, DWRITE_GLYPH_OFFSET const *offsets, unsigned int count, BOOL is_rtl,
-        IDWriteGeometrySink *sink)
+static int CDECL freetype_get_glyph_outline(font_object_handle object, float emsize, unsigned int simulations,
+        UINT16 glyph, struct dwrite_outline *outline)
 {
-    FTC_ScalerRec scaler;
-    USHORT simulations;
-    HRESULT hr = S_OK;
+    FT_Face face = object;
     FT_Size size;
+    int ret = 0;
 
-    if (!count)
-        return S_OK;
+    if (!(size = freetype_set_face_size(face, emsize)))
+        return 0;
 
-    ID2D1SimplifiedGeometrySink_SetFillMode(sink, D2D1_FILL_MODE_WINDING);
-
-    simulations = IDWriteFontFace5_GetSimulations(fontface);
-
-    scaler.face_id = fontface;
-    scaler.width  = emSize;
-    scaler.height = emSize;
-    scaler.pixel = 1;
-    scaler.x_res = 0;
-    scaler.y_res = 0;
-
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_Manager_LookupSize(cache_manager, &scaler, &size) == 0)
+    if (!pFT_Load_Glyph(face, glyph, FT_LOAD_NO_BITMAP))
     {
-        D2D1_POINT_2F origin;
-        unsigned int i;
+        FT_Outline *ft_outline = &face->glyph->outline;
+        FT_Matrix m;
 
-        origin.x = origin.y = 0.0f;
-        for (i = 0; i < count; ++i)
+        if (outline->points.values)
         {
-            if (pFT_Load_Glyph(size->face, glyphs[i], FT_LOAD_NO_BITMAP) == 0)
-            {
-                FLOAT ft_advance = size->face->glyph->metrics.horiAdvance >> 6;
-                FT_Outline *outline = &size->face->glyph->outline;
-                D2D1_POINT_2F glyph_origin;
-                FT_Matrix m;
+            if (simulations & DWRITE_FONT_SIMULATIONS_BOLD)
+                embolden_glyph_outline(ft_outline, emsize);
 
-                if (simulations & DWRITE_FONT_SIMULATIONS_BOLD)
-                    embolden_glyph_outline(outline, emSize);
+            m.xx = 1 << 16;
+            m.xy = simulations & DWRITE_FONT_SIMULATIONS_OBLIQUE ? (1 << 16) / 3 : 0;
+            m.yx = 0;
+            m.yy = -(1 << 16); /* flip Y axis */
 
-                m.xx = 1 << 16;
-                m.xy = simulations & DWRITE_FONT_SIMULATIONS_OBLIQUE ? (1 << 16) / 3 : 0;
-                m.yx = 0;
-                m.yy = -(1 << 16); /* flip Y axis */
+            pFT_Outline_Transform(ft_outline, &m);
 
-                pFT_Outline_Transform(outline, &m);
-
-                glyph_origin = origin;
-
-                if (is_rtl)
-                {
-                    glyph_origin.x -= ft_advance;
-
-                    if (offsets)
-                    {
-                        glyph_origin.x -= offsets[i].advanceOffset;
-                        glyph_origin.y -= offsets[i].ascenderOffset;
-                    }
-
-                    origin.x -= advances ? advances[i] : ft_advance;
-                }
-                else
-                {
-                    if (offsets)
-                    {
-                        glyph_origin.x += offsets[i].advanceOffset;
-                        glyph_origin.y -= offsets[i].ascenderOffset;
-                    }
-
-                    origin.x += advances ? advances[i] : ft_advance;
-                }
-
-                decompose_outline(outline, glyph_origin, sink);
-            }
+            ret = decompose_outline(ft_outline, outline);
+        }
+        else
+        {
+            /* Intentionally overestimate numbers to keep it simple. */
+            outline->points.count = ft_outline->n_points * 3;
+            outline->tags.count = ft_outline->n_points + ft_outline->n_contours * 2;
         }
     }
-    else
-        hr = E_FAIL;
-    LeaveCriticalSection(&freetype_cs);
 
-    return hr;
+    pFT_Done_Size(size);
+
+    return ret;
 }
 
-UINT16 freetype_get_glyphcount(IDWriteFontFace5 *fontface)
+static UINT16 CDECL freetype_get_glyph_count(font_object_handle object)
 {
-    UINT16 count = 0;
-    FT_Face face;
-
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
-        count = face->num_glyphs;
-    LeaveCriticalSection(&freetype_cs);
-
-    return count;
-}
-
-BOOL freetype_has_kerning_pairs(IDWriteFontFace5 *fontface)
-{
-    BOOL has_kerning_pairs = FALSE;
-    FT_Face face;
-
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
-        has_kerning_pairs = !!FT_HAS_KERNING(face);
-    LeaveCriticalSection(&freetype_cs);
-
-    return has_kerning_pairs;
-}
-
-INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace5 *fontface, UINT16 left, UINT16 right)
-{
-    INT32 adjustment = 0;
-    FT_Face face;
-
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0) {
-        FT_Vector kern;
-        if (FT_HAS_KERNING(face)) {
-            pFT_Get_Kerning(face, left, right, FT_KERNING_UNSCALED, &kern);
-            adjustment = kern.x;
-        }
-    }
-    LeaveCriticalSection(&freetype_cs);
-
-    return adjustment;
+    FT_Face face = object;
+    return face ? face->num_glyphs : 0;
 }
 
 static inline void ft_matrix_from_dwrite_matrix(const DWRITE_MATRIX *m, FT_Matrix *ft_matrix)
@@ -602,10 +563,10 @@ static inline void ft_matrix_from_dwrite_matrix(const DWRITE_MATRIX *m, FT_Matri
 }
 
 /* Should be used only while holding 'freetype_cs' */
-static BOOL is_face_scalable(IDWriteFontFace4 *fontface)
+static BOOL is_face_scalable(void *key)
 {
     FT_Face face;
-    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
+    if (pFTC_Manager_LookupFace(cache_manager, key, &face) == 0)
         return FT_IS_SCALABLE(face);
     else
         return FALSE;
@@ -622,7 +583,7 @@ static BOOL get_glyph_transform(struct dwrite_glyphbitmap *bitmap, FT_Matrix *re
 
     /* Some fonts provide mostly bitmaps and very few outlines, for example for .notdef.
        Disable transform if that's the case. */
-    if (!is_face_scalable(bitmap->fontface) || (!bitmap->m && bitmap->simulations == 0))
+    if (!is_face_scalable(bitmap->key) || (!bitmap->m && bitmap->simulations == 0))
         return FALSE;
 
     if (bitmap->simulations & DWRITE_FONT_SIMULATIONS_OBLIQUE) {
@@ -641,7 +602,7 @@ static BOOL get_glyph_transform(struct dwrite_glyphbitmap *bitmap, FT_Matrix *re
     return TRUE;
 }
 
-void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
+static void CDECL freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
 {
     FTC_ImageTypeRec imagetype;
     FT_BBox bbox = { 0 };
@@ -649,11 +610,11 @@ void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
     FT_Glyph glyph;
     FT_Matrix m;
 
-    EnterCriticalSection(&freetype_cs);
+    RtlEnterCriticalSection(&freetype_cs);
 
     needs_transform = get_glyph_transform(bitmap, &m);
 
-    imagetype.face_id = bitmap->fontface;
+    imagetype.face_id = bitmap->key;
     imagetype.width = 0;
     imagetype.height = bitmap->emsize;
     imagetype.flags = needs_transform ? FT_LOAD_NO_BITMAP : FT_LOAD_DEFAULT;
@@ -676,7 +637,7 @@ void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
             pFT_Glyph_Get_CBox(glyph, FT_GLYPH_BBOX_PIXELS, &bbox);
     }
 
-    LeaveCriticalSection(&freetype_cs);
+    RtlLeaveCriticalSection(&freetype_cs);
 
     /* flip Y axis */
     SetRect(&bitmap->bbox, bbox.xMin, -bbox.yMax, bbox.xMax, -bbox.yMin);
@@ -773,7 +734,7 @@ static BOOL freetype_get_aa_glyph_bitmap(struct dwrite_glyphbitmap *bitmap, FT_G
     return ret;
 }
 
-BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
+static BOOL CDECL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
 {
     FTC_ImageTypeRec imagetype;
     BOOL needs_transform;
@@ -781,11 +742,11 @@ BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
     FT_Glyph glyph;
     FT_Matrix m;
 
-    EnterCriticalSection(&freetype_cs);
+    RtlEnterCriticalSection(&freetype_cs);
 
     needs_transform = get_glyph_transform(bitmap, &m);
 
-    imagetype.face_id = bitmap->fontface;
+    imagetype.face_id = bitmap->key;
     imagetype.width = 0;
     imagetype.height = bitmap->emsize;
     imagetype.flags = needs_transform ? FT_LOAD_NO_BITMAP : FT_LOAD_DEFAULT;
@@ -815,26 +776,26 @@ BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
             pFT_Done_Glyph(glyph_copy);
     }
 
-    LeaveCriticalSection(&freetype_cs);
+    RtlLeaveCriticalSection(&freetype_cs);
 
     return ret;
 }
 
-INT32 freetype_get_glyph_advance(IDWriteFontFace5 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode,
-    BOOL *has_contours)
+static INT32 CDECL freetype_get_glyph_advance(void *key, float emSize, UINT16 index,
+        DWRITE_MEASURING_MODE mode, BOOL *has_contours)
 {
     FTC_ImageTypeRec imagetype;
     FT_Glyph glyph;
     INT32 advance;
 
-    imagetype.face_id = fontface;
+    imagetype.face_id = key;
     imagetype.width = 0;
     imagetype.height = emSize;
     imagetype.flags = FT_LOAD_DEFAULT;
     if (mode == DWRITE_MEASURING_MODE_NATURAL)
         imagetype.flags |= FT_LOAD_NO_HINTING;
 
-    EnterCriticalSection(&freetype_cs);
+    RtlEnterCriticalSection(&freetype_cs);
     if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0) {
         *has_contours = glyph->format == FT_GLYPH_FORMAT_OUTLINE && ((FT_OutlineGlyph)glyph)->outline.n_contours;
         advance = glyph->advance.x >> 16;
@@ -843,68 +804,118 @@ INT32 freetype_get_glyph_advance(IDWriteFontFace5 *fontface, FLOAT emSize, UINT1
         *has_contours = FALSE;
         advance = 0;
     }
-    LeaveCriticalSection(&freetype_cs);
+    RtlLeaveCriticalSection(&freetype_cs);
 
     return advance;
 }
 
+const static struct font_backend_funcs freetype_funcs =
+{
+    freetype_create_font_object,
+    freetype_release_font_object,
+    freetype_notify_release,
+    freetype_get_glyph_outline,
+    freetype_get_glyph_count,
+    freetype_get_glyph_advance,
+    freetype_get_glyph_bbox,
+    freetype_get_glyph_bitmap,
+    freetype_get_design_glyph_metrics,
+};
+
+static NTSTATUS init_freetype_lib(HMODULE module, DWORD reason, const void *ptr_in, void *ptr_out)
+{
+    callback_funcs = ptr_in;
+    if (!init_freetype()) return STATUS_DLL_NOT_FOUND;
+    *(const struct font_backend_funcs **)ptr_out = &freetype_funcs;
+    return STATUS_SUCCESS;
+}
+
+static NTSTATUS release_freetype_lib(void)
+{
+    pFTC_Manager_Done(cache_manager);
+    pFT_Done_FreeType(library);
+    return STATUS_SUCCESS;
+}
+
 #else /* HAVE_FREETYPE */
 
-BOOL init_freetype(void)
+static font_object_handle CDECL null_create_font_object(const void *data_ptr, UINT64 data_size, unsigned int index)
 {
-    return FALSE;
+    return NULL;
 }
 
-void release_freetype(void)
+static void CDECL null_release_font_object(font_object_handle object)
 {
 }
 
-void freetype_notify_cacheremove(IDWriteFontFace5 *fontface)
+static void CDECL null_notify_release(void *key)
 {
 }
 
-HRESULT freetype_get_design_glyph_metrics(struct dwrite_fontface *fontface, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
+static int CDECL null_get_glyph_outline(font_object_handle object, float emSize, unsigned int simulations,
+        UINT16 glyph, struct dwrite_outline *outline)
 {
-    return E_NOTIMPL;
+    return 1;
 }
 
-HRESULT freetype_get_glyphrun_outline(IDWriteFontFace5 *fontface, float emSize, UINT16 const *glyphs,
-        float const *advances, DWRITE_GLYPH_OFFSET const *offsets, unsigned int count, BOOL is_rtl,
-        IDWriteGeometrySink *sink)
-{
-    return E_NOTIMPL;
-}
-
-UINT16 freetype_get_glyphcount(IDWriteFontFace5 *fontface)
+static UINT16 CDECL null_get_glyph_count(font_object_handle object)
 {
     return 0;
 }
 
-BOOL freetype_has_kerning_pairs(IDWriteFontFace5 *fontface)
-{
-    return FALSE;
-}
-
-INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace5 *fontface, UINT16 left, UINT16 right)
-{
-    return 0;
-}
-
-void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
-{
-    SetRectEmpty(&bitmap->bbox);
-}
-
-BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
-{
-    return FALSE;
-}
-
-INT32 freetype_get_glyph_advance(IDWriteFontFace5 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode,
+static INT32 CDECL null_get_glyph_advance(void *key, float emSize, UINT16 index, DWRITE_MEASURING_MODE mode,
     BOOL *has_contours)
 {
     *has_contours = FALSE;
     return 0;
 }
 
+static void CDECL null_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
+{
+    SetRectEmpty(&bitmap->bbox);
+}
+
+static BOOL CDECL null_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
+{
+    return FALSE;
+}
+
+static void CDECL null_get_design_glyph_metrics(font_object_handle object, UINT16 upem, UINT16 ascent, unsigned int simulations,
+        UINT16 glyph, DWRITE_GLYPH_METRICS *metrics)
+{
+}
+
+const static struct font_backend_funcs null_funcs =
+{
+    null_create_font_object,
+    null_release_font_object,
+    null_notify_release,
+    null_get_glyph_outline,
+    null_get_glyph_count,
+    null_get_glyph_advance,
+    null_get_glyph_bbox,
+    null_get_glyph_bitmap,
+    null_get_design_glyph_metrics,
+};
+
+static NTSTATUS init_freetype_lib(HMODULE module, DWORD reason, const void *ptr_in, void *ptr_out)
+{
+    *(const struct font_backend_funcs **)ptr_out = &null_funcs;
+    return STATUS_DLL_NOT_FOUND;
+}
+
+static NTSTATUS release_freetype_lib(void)
+{
+    return STATUS_DLL_NOT_FOUND;
+}
+
 #endif /* HAVE_FREETYPE */
+
+NTSTATUS CDECL __wine_init_unix_lib(HMODULE module, DWORD reason, const void *ptr_in, void *ptr_out)
+{
+    if (reason == DLL_PROCESS_ATTACH)
+        return init_freetype_lib(module, reason, ptr_in, ptr_out);
+    else if (reason == DLL_PROCESS_DETACH)
+        return release_freetype_lib();
+    return STATUS_SUCCESS;
+}
