@@ -38,6 +38,7 @@
 
 static int    myARGC;
 static char** myARGV;
+static BOOL is_wow64;
 
 static BOOL (WINAPI *pCheckRemoteDebuggerPresent)(HANDLE,PBOOL);
 
@@ -51,6 +52,7 @@ static NTSTATUS  (WINAPI *pDbgUiConnectToDbg)(void);
 static HANDLE    (WINAPI *pDbgUiGetThreadDebugObject)(void);
 static void      (WINAPI *pDbgUiSetThreadDebugObject)(HANDLE);
 static DWORD     (WINAPI *pGetMappedFileNameW)(HANDLE,void*,WCHAR*,DWORD);
+static BOOL      (WINAPI *pIsWow64Process)(HANDLE,PBOOL);
 
 static LONG child_failures;
 
@@ -58,11 +60,11 @@ static HMODULE ntdll;
 
 static void WINAPIV __WINE_PRINTF_ATTR(2, 3) test_child_ok(int condition, const char *msg, ...)
 {
-    __ms_va_list valist;
+    va_list valist;
 
-    __ms_va_start(valist, msg);
+    va_start(valist, msg);
     winetest_vok(condition, msg, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     if (!condition) ++child_failures;
 }
 
@@ -281,6 +283,15 @@ static void *get_ip(const CONTEXT *ctx)
 #endif
 }
 
+static void set_ip(CONTEXT *ctx, void *ip)
+{
+#ifdef __i386__
+    ctx->Eip = (DWORD_PTR)ip;
+#elif defined(__x86_64__)
+    ctx->Rip = (DWORD_PTR)ip;
+#endif
+}
+
 #define fetch_thread_context(a) fetch_thread_context_(__LINE__,a)
 static void fetch_thread_context_(unsigned line, struct debuggee_thread *thread)
 {
@@ -896,11 +907,33 @@ static void doChild(int argc, char **argv)
     CloseHandle( map );
     UnmapViewOfFile( mod );
 
+    if (sizeof(void *) > sizeof(int))
+    {
+        GetSystemWow64DirectoryW( path, MAX_PATH );
+        wcscat( path, L"\\oleacc.dll" );
+    }
+    else if (is_wow64)
+    {
+        wcscpy( path, L"c:\\windows\\sysnative\\oleacc.dll" );
+    }
+    else goto done;
+
+    file = CreateFileW( path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0 );
+    child_ok( file != INVALID_HANDLE_VALUE, "failed to open %s: %u\n", debugstr_w(path), GetLastError());
+    map = CreateFileMappingW( file, NULL, SEC_IMAGE | PAGE_READONLY, 0, 0, NULL );
+    child_ok( map != NULL, "failed to create mapping %s: %u\n", debugstr_w(path), GetLastError() );
+    mod = MapViewOfFile( map, FILE_MAP_READ, 0, 0, 0 );
+    child_ok( mod != NULL, "failed to map %s: %u\n", debugstr_w(path), GetLastError() );
+    CloseHandle( file );
+    CloseHandle( map );
+    UnmapViewOfFile( mod );
+
+done:
     blackbox.failures = child_failures;
     save_blackbox(blackbox_file, &blackbox, sizeof(blackbox), NULL);
 }
 
-static HMODULE ole32_mod, oleaut32_mod;
+static HMODULE ole32_mod, oleaut32_mod, oleacc_mod;
 
 static void check_dll_event( HANDLE process, DEBUG_EVENT *ev )
 {
@@ -916,10 +949,12 @@ static void check_dll_event( HANDLE process, DEBUG_EVENT *ev )
         else p = module;
         if (!wcsicmp( p, L"ole32.dll" )) ole32_mod = ev->u.LoadDll.lpBaseOfDll;
         else if (!wcsicmp( p, L"oleaut32.dll" )) oleaut32_mod = ev->u.LoadDll.lpBaseOfDll;
+        else if (!wcsicmp( p, L"oleacc.dll" )) oleacc_mod = ev->u.LoadDll.lpBaseOfDll;
         break;
     case UNLOAD_DLL_DEBUG_EVENT:
         if (ev->u.UnloadDll.lpBaseOfDll == ole32_mod) ole32_mod = (HMODULE)1;
         if (ev->u.UnloadDll.lpBaseOfDll == oleaut32_mod) oleaut32_mod = (HMODULE)1;
+        if (ev->u.UnloadDll.lpBaseOfDll == oleacc_mod) oleacc_mod = (HMODULE)1;
         break;
     }
 }
@@ -987,6 +1022,11 @@ static void test_debug_loop(int argc, char **argv)
 
     ok( ole32_mod == (HMODULE)1, "ole32.dll was not reported\n" );
     ok( oleaut32_mod == (HMODULE)1, "oleaut32.dll was not reported\n" );
+#ifdef _WIN64
+    ok( oleacc_mod == (HMODULE)1, "oleacc.dll was not reported\n" );
+#else
+    ok( oleacc_mod == NULL, "oleacc.dll was reported\n" );
+#endif
 
     ret = CloseHandle(pi.hThread);
     ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
@@ -1220,9 +1260,9 @@ static void test_debug_children(const char *name, DWORD flag, BOOL debug_child,
     ok(ret, "DeleteFileA failed, last error %d.\n", GetLastError());
 }
 
-static void wait_debugger(HANDLE event)
+static void wait_debugger(HANDLE event, unsigned int cnt)
 {
-    WaitForSingleObject(event, INFINITE);
+    while (cnt--) WaitForSingleObject(event, INFINITE);
     ExitProcess(0);
 }
 
@@ -1348,11 +1388,11 @@ static void test_debugger(const char *argv0)
     char *cmd;
     BOOL ret;
 
-    event = CreateEventW(&sa, TRUE, FALSE, NULL);
+    event = CreateEventW(&sa, FALSE, FALSE, NULL);
     ok(event != NULL, "CreateEvent failed: %u\n", GetLastError());
 
     cmd = heap_alloc(strlen(argv0) + strlen(arguments) + 16);
-    sprintf(cmd, "%s%s%x\n", argv0, arguments, (DWORD)(DWORD_PTR)event);
+    sprintf(cmd, "%s%s%x %u\n", argv0, arguments, (DWORD)(DWORD_PTR)event, OP_BP ? 3 : 1);
 
     memset(&si, 0, sizeof(si));
     si.cb = sizeof(si);
@@ -1737,6 +1777,161 @@ static void test_debugger(const char *argv0)
         }
     }
 
+    if (OP_BP)
+    {
+        CONTEXT orig_context;
+        char instr, *ip;
+
+        /* main thread sleeps inside ntdll waiting for the event. set breakpoint there and make sure
+         * ntdll can handle that. */
+        SuspendThread(ctx.main_thread->handle);
+
+        fetch_thread_context(ctx.main_thread);
+        ret = ReadProcessMemory(pi.hProcess, get_ip(&ctx.main_thread->ctx), &instr, 1, NULL);
+        ok(ret, "ReadProcessMemory failed: %u\n", GetLastError());
+
+        orig_context = ctx.main_thread->ctx;
+        ip = get_ip(&ctx.main_thread->ctx);
+
+#if defined(__i386__)
+        ctx.main_thread->ctx.Eax = 101;
+        ctx.main_thread->ctx.Ebx = 102;
+        ctx.main_thread->ctx.Ecx = 103;
+        ctx.main_thread->ctx.Edx = 104;
+        ctx.main_thread->ctx.Esi = 105;
+        ctx.main_thread->ctx.Edi = 106;
+#elif defined(__x86_64__)
+        ctx.main_thread->ctx.Rax = 101;
+        ctx.main_thread->ctx.Rbx = 102;
+        ctx.main_thread->ctx.Rcx = 103;
+        ctx.main_thread->ctx.Rdx = 104;
+        ctx.main_thread->ctx.Rsi = 105;
+        ctx.main_thread->ctx.Rdi = 106;
+        ctx.main_thread->ctx.R8  = 107;
+        ctx.main_thread->ctx.R9  = 108;
+        ctx.main_thread->ctx.R10 = 109;
+        ctx.main_thread->ctx.R11 = 110;
+        ctx.main_thread->ctx.R12 = 111;
+        ctx.main_thread->ctx.R13 = 112;
+        ctx.main_thread->ctx.R14 = 113;
+        ctx.main_thread->ctx.R15 = 114;
+#endif
+        set_thread_context(&ctx, ctx.main_thread);
+
+        fetch_thread_context(ctx.main_thread);
+#if defined(__i386__)
+        /* win2k8 do not preserve eax, rcx and edx; newer versions do */
+        ok(ctx.main_thread->ctx.Ebx == 102, "Ebx = %x\n", ctx.main_thread->ctx.Ebx);
+        ok(ctx.main_thread->ctx.Esi == 105, "Esi = %x\n", ctx.main_thread->ctx.Esi);
+        ok(ctx.main_thread->ctx.Edi == 106, "Edi = %x\n", ctx.main_thread->ctx.Edi);
+#elif defined(__x86_64__)
+        ok(ctx.main_thread->ctx.Rax == 101,   "Rax = %x\n", ctx.main_thread->ctx.Rax);
+        ok(ctx.main_thread->ctx.Rbx == 102, "Rbx = %x\n", ctx.main_thread->ctx.Rbx);
+        ok(ctx.main_thread->ctx.Rcx == 103, "Rcx = %x\n", ctx.main_thread->ctx.Rcx);
+        ok(ctx.main_thread->ctx.Rdx == 104, "Rdx = %x\n", ctx.main_thread->ctx.Rdx);
+        ok(ctx.main_thread->ctx.Rsi == 105, "Rsi = %x\n", ctx.main_thread->ctx.Rsi);
+        ok(ctx.main_thread->ctx.Rdi == 106, "Rdi = %x\n", ctx.main_thread->ctx.Rdi);
+        ok(ctx.main_thread->ctx.R8  == 107, "R8 = %x\n",  ctx.main_thread->ctx.R8);
+        ok(ctx.main_thread->ctx.R9  == 108, "R9 = %x\n",  ctx.main_thread->ctx.R9);
+        ok(ctx.main_thread->ctx.R10 == 109, "R10 = %x\n", ctx.main_thread->ctx.R10);
+        ok(ctx.main_thread->ctx.R11 == 110, "R11 = %x\n", ctx.main_thread->ctx.R11);
+        ok(ctx.main_thread->ctx.R12 == 111, "R12 = %x\n", ctx.main_thread->ctx.R12);
+        ok(ctx.main_thread->ctx.R13 == 112, "R13 = %x\n", ctx.main_thread->ctx.R13);
+        ok(ctx.main_thread->ctx.R14 == 113, "R14 = %x\n", ctx.main_thread->ctx.R14);
+        ok(ctx.main_thread->ctx.R15 == 114, "R15 = %x\n", ctx.main_thread->ctx.R15);
+#endif
+
+        byte = OP_BP;
+        ret = WriteProcessMemory(pi.hProcess, ip, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        SetEvent(event);
+        ResumeThread(ctx.main_thread->handle);
+
+        next_event(&ctx, 2000);
+        ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
+           ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
+        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == ip,
+           "ExceptionAddress = %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress);
+
+        fetch_thread_context(ctx.main_thread);
+        ok(get_ip(&ctx.main_thread->ctx) == ip + 1, "unexpected instruction pointer %p\n", get_ip(&ctx.main_thread->ctx));
+
+#if defined(__i386__)
+        ok(ctx.main_thread->ctx.Eax == 0,   "Eax = %x\n", ctx.main_thread->ctx.Eax);
+        ok(ctx.main_thread->ctx.Ebx == 102, "Ebx = %x\n", ctx.main_thread->ctx.Ebx);
+        ok(ctx.main_thread->ctx.Ecx != 103, "Ecx = %x\n", ctx.main_thread->ctx.Ecx);
+        ok(ctx.main_thread->ctx.Edx != 104, "Edx = %x\n", ctx.main_thread->ctx.Edx);
+        ok(ctx.main_thread->ctx.Esi == 105, "Esi = %x\n", ctx.main_thread->ctx.Esi);
+        ok(ctx.main_thread->ctx.Edi == 106, "Edi = %x\n", ctx.main_thread->ctx.Edi);
+#elif defined(__x86_64__)
+        ok(ctx.main_thread->ctx.Rax == 0,   "Rax = %x\n", ctx.main_thread->ctx.Rax);
+        ok(ctx.main_thread->ctx.Rbx == 102, "Rbx = %x\n", ctx.main_thread->ctx.Rbx);
+        ok(ctx.main_thread->ctx.Rcx != 103, "Rcx = %x\n", ctx.main_thread->ctx.Rcx);
+        ok(ctx.main_thread->ctx.Rdx != 104, "Rdx = %x\n", ctx.main_thread->ctx.Rdx);
+        ok(ctx.main_thread->ctx.Rsi == 105, "Rsi = %x\n", ctx.main_thread->ctx.Rsi);
+        ok(ctx.main_thread->ctx.Rdi == 106, "Rdi = %x\n", ctx.main_thread->ctx.Rdi);
+        ok(ctx.main_thread->ctx.R8  != 107, "R8 = %x\n",  ctx.main_thread->ctx.R8);
+        ok(ctx.main_thread->ctx.R9  != 108, "R9 = %x\n",  ctx.main_thread->ctx.R9);
+        ok(ctx.main_thread->ctx.R10 != 109, "R10 = %x\n", ctx.main_thread->ctx.R10);
+        ok(ctx.main_thread->ctx.R11 != 110, "R11 = %x\n", ctx.main_thread->ctx.R11);
+        ok(ctx.main_thread->ctx.R12 == 111, "R12 = %x\n", ctx.main_thread->ctx.R12);
+        ok(ctx.main_thread->ctx.R13 == 112, "R13 = %x\n", ctx.main_thread->ctx.R13);
+        ok(ctx.main_thread->ctx.R14 == 113, "R14 = %x\n", ctx.main_thread->ctx.R14);
+        ok(ctx.main_thread->ctx.R15 == 114, "R15 = %x\n", ctx.main_thread->ctx.R15);
+#endif
+
+        ctx.main_thread->ctx = orig_context;
+        set_ip(&ctx.main_thread->ctx, ip);
+        set_thread_context(&ctx, ctx.main_thread);
+
+        ret = WriteProcessMemory(pi.hProcess, ip, &instr, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        memset(buf + 10, 0x90, 10); /* nop */
+        ret = WriteProcessMemory(pi.hProcess, mem + 10, buf + 10, 10, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        next_event(&ctx, POLL_EVENT_TIMEOUT);
+
+        /* try single step while debuggee is in a syscall */
+        fetch_thread_context(ctx.main_thread);
+        orig_context = ctx.main_thread->ctx;
+        ip = get_ip(&ctx.main_thread->ctx);
+
+#if defined(__i386__)
+        ctx.main_thread->ctx.EFlags |= 0x100;
+        ctx.main_thread->ctx.Eip = (ULONG_PTR)mem + 10;
+#elif defined(__x86_64__)
+        ctx.main_thread->ctx.EFlags |= 0x100;
+        ctx.main_thread->ctx.Rip = (ULONG64)mem + 10;
+#endif
+        set_thread_context(&ctx, ctx.main_thread);
+
+        SetEvent(event);
+
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        if (sizeof(void*) != 4 || ctx.ev.u.Exception.ExceptionRecord.ExceptionCode != EXCEPTION_BREAKPOINT)
+        {
+            ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+            ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_SINGLE_STEP, "ExceptionCode = %x\n",
+               ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
+            ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == mem + 10 ||
+               ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == mem + 11,
+               "ExceptionAddress = %p expected %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress, mem + 10);
+
+            fetch_thread_context(ctx.main_thread);
+            ok(get_ip(&ctx.main_thread->ctx) == ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress,
+               "ip = %p\n", get_ip(&ctx.main_thread->ctx));
+
+        }
+        else win_skip("got breakpoint instead of single step exception\n");
+
+        ctx.main_thread->ctx = orig_context;
+        set_thread_context(&ctx, ctx.main_thread);
+    }
+
     SetEvent(event);
 
     do
@@ -1956,6 +2151,7 @@ START_TEST(debugger)
 
     hdll=GetModuleHandleA("kernel32.dll");
     pCheckRemoteDebuggerPresent=(void*)GetProcAddress(hdll, "CheckRemoteDebuggerPresent");
+    pIsWow64Process=(void*)GetProcAddress(hdll, "IsWow64Process");
     pGetMappedFileNameW = (void*)GetProcAddress(hdll, "GetMappedFileNameW");
     if (!pGetMappedFileNameW) pGetMappedFileNameW = (void*)GetProcAddress(LoadLibraryA("psapi.dll"),
                                                                           "GetMappedFileNameW");
@@ -1969,6 +2165,8 @@ START_TEST(debugger)
     pDbgUiGetThreadDebugObject = (void*)GetProcAddress(ntdll, "DbgUiGetThreadDebugObject");
     pDbgUiSetThreadDebugObject = (void*)GetProcAddress(ntdll, "DbgUiSetThreadDebugObject");
 
+    if (pIsWow64Process) pIsWow64Process( GetCurrentProcess(), &is_wow64 );
+
     myARGC=winetest_get_mainargs(&myARGV);
     if (myARGC >= 3 && strcmp(myARGV[2], "crash") == 0)
     {
@@ -1988,9 +2186,10 @@ START_TEST(debugger)
     }
     else if (myARGC >= 4 && !strcmp(myARGV[2], "wait"))
     {
-        DWORD event;
+        DWORD event, cnt = 1;
         sscanf(myARGV[3], "%x", &event);
-        wait_debugger((HANDLE)(DWORD_PTR)event);
+        if (myARGC >= 5) cnt = atoi(myARGV[4]);
+        wait_debugger((HANDLE)(DWORD_PTR)event, cnt);
     }
     else
     {
