@@ -18,8 +18,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -42,9 +45,9 @@
 #define USE_WS_PREFIX
 #include "winsock2.h"
 #include "ws2ipdef.h"
-#include "nldef.h"
-#include "netioapi.h"
 #include "dhcpcsdk.h"
+#include "unixlib.h"
+
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mountmgr);
@@ -67,13 +70,11 @@ static void appeared_callback( DADiskRef disk, void *context )
     const void *ref;
     char device[64];
     char mount_point[PATH_MAX];
-    char model[64];
     size_t model_len = 0;
     GUID guid, *guid_ptr = NULL;
     enum device_type type = DEVICE_UNKNOWN;
-    UINT pdtype = 0;
-    SCSI_ADDRESS scsi_addr;
-    UNICODE_STRING devname;
+    struct scsi_info scsi_info = { 0 };
+    BOOL removable = FALSE;
     int fd;
 
     if (!dict) return;
@@ -100,13 +101,13 @@ static void appeared_callback( DADiskRef disk, void *context )
         if (!CFStringCompare( ref, CFSTR("IOCDMedia"), 0 ))
         {
             type = DEVICE_CDROM;
-            pdtype = 5;
+            scsi_info.type = 5;
         }
         if (!CFStringCompare( ref, CFSTR("IODVDMedia"), 0 ) ||
             !CFStringCompare( ref, CFSTR("IOBDMedia"), 0 ))
         {
             type = DEVICE_DVD;
-            pdtype = 5;
+            scsi_info.type = 5;
         }
         if (!CFStringCompare( ref, CFSTR("IOMedia"), 0 ))
             type = DEVICE_HARDDISK;
@@ -116,41 +117,38 @@ static void appeared_callback( DADiskRef disk, void *context )
     {
         CFIndex i;
 
-        CFStringGetCString( ref, model, sizeof(model), kCFStringEncodingASCII );
+        CFStringGetCString( ref, scsi_info.model, sizeof(scsi_info.model), kCFStringEncodingASCII );
         model_len += CFStringGetLength( ref );
         /* Pad to 8 characters */
         for (i = 0; i < (CFIndex)8 - CFStringGetLength( ref ); ++i)
-            model[model_len++] = ' ';
+            scsi_info.model[model_len++] = ' ';
     }
     if ((ref = CFDictionaryGetValue( dict, CFSTR("DADeviceModel") )))
     {
         CFIndex i;
 
-        CFStringGetCString( ref, model+model_len, sizeof(model)-model_len, kCFStringEncodingASCII );
+        CFStringGetCString( ref, scsi_info.model+model_len, sizeof(scsi_info.model)-model_len, kCFStringEncodingASCII );
         model_len += CFStringGetLength( ref );
         /* Pad to 16 characters */
         for (i = 0; i < (CFIndex)16 - CFStringGetLength( ref ); ++i)
-            model[model_len++] = ' ';
+            scsi_info.model[model_len++] = ' ';
     }
     if ((ref = CFDictionaryGetValue( dict, CFSTR("DADeviceRevision") )))
     {
         CFIndex i;
 
-        CFStringGetCString( ref, model+model_len, sizeof(model)-model_len, kCFStringEncodingASCII );
+        CFStringGetCString( ref, scsi_info.model+model_len, sizeof(scsi_info.model)-model_len, kCFStringEncodingASCII );
         model_len += CFStringGetLength( ref );
         /* Pad to 4 characters */
         for (i = 0; i < (CFIndex)4 - CFStringGetLength( ref ); ++i)
-            model[model_len++] = ' ';
+            scsi_info.model[model_len++] = ' ';
     }
 
     TRACE( "got mount notification for '%s' on '%s' uuid %s\n",
            device, mount_point, wine_dbgstr_guid(guid_ptr) );
 
-    devname.Buffer = NULL;
-    if ((ref = CFDictionaryGetValue( dict, CFSTR("DAMediaRemovable") )) && CFBooleanGetValue( ref ))
-        add_dos_device( -1, device, device, mount_point, type, guid_ptr, &devname );
-    else
-        if (guid_ptr) add_volume( device, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr, NULL );
+    if ((ref = CFDictionaryGetValue( dict, CFSTR("DAMediaRemovable") )))
+        removable = CFBooleanGetValue( ref );
 
     if (!access( device, R_OK ) &&
         (fd = open( device, O_RDONLY )) >= 0)
@@ -159,18 +157,21 @@ static void appeared_callback( DADiskRef disk, void *context )
 
         if (ioctl( fd, DKIOCSCSIIDENTIFY, &dsi ) >= 0)
         {
-            scsi_addr.PortNumber = dsi.bus;
-            scsi_addr.PathId = dsi.port;
-            scsi_addr.TargetId = dsi.target;
-            scsi_addr.Lun = dsi.lun;
-
-            /* FIXME: get real controller Id for SCSI */
-            /* FIXME: get real driver name */
-            create_scsi_entry( &scsi_addr, 255, "WINE SCSI", pdtype, model, &devname );
+            scsi_info.addr.PortNumber = dsi.bus;
+            scsi_info.addr.PathId = dsi.port;
+            scsi_info.addr.TargetId = dsi.target;
+            scsi_info.addr.Lun = dsi.lun;
+            scsi_info.init_id = 255; /* FIXME */
+            strcpy( scsi_info.driver, "WINE SCSI" ); /* FIXME */
         }
         close( fd );
     }
 
+    if (removable)
+        queue_device_op( ADD_DOS_DEVICE, device, device, mount_point, type, guid_ptr, NULL, &scsi_info );
+    else
+        if (guid_ptr) queue_device_op( ADD_VOLUME, device, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr, NULL, &scsi_info );
+
 done:
     CFRelease( dict );
 }
@@ -195,20 +196,17 @@ static void disappeared_callback( DADiskRef disk, void *context )
 
     TRACE( "got unmount notification for '%s'\n", device );
 
-    if ((ref = CFDictionaryGetValue( dict, CFSTR("DAMediaRemovable") )) && CFBooleanGetValue( ref ))
-        remove_dos_device( -1, device );
-    else
-        remove_volume( device );
+    queue_device_op( REMOVE_DEVICE, device, NULL, NULL, 0, NULL, NULL, NULL );
 
 done:
     CFRelease( dict );
 }
 
-static DWORD WINAPI runloop_thread( void *arg )
+void run_diskarbitration_loop(void)
 {
     DASessionRef session = DASessionCreate( NULL );
 
-    if (!session) return 1;
+    if (!session) return;
 
     DASessionScheduleWithRunLoop( session, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode );
     DARegisterDiskAppearedCallback( session, kDADiskDescriptionMatchVolumeMountable,
@@ -220,20 +218,11 @@ static DWORD WINAPI runloop_thread( void *arg )
     CFRunLoopRun();
     DASessionUnscheduleFromRunLoop( session, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode );
     CFRelease( session );
-    return 0;
-}
-
-void initialize_diskarbitration(void)
-{
-    HANDLE handle;
-
-    if (!(handle = CreateThread( NULL, 0, runloop_thread, NULL, 0, NULL ))) return;
-    CloseHandle( handle );
 }
 
 #else  /*  HAVE_DISKARBITRATION_DISKARBITRATION_H */
 
-void initialize_diskarbitration(void)
+void run_diskarbitration_loop(void)
 {
     TRACE( "Skipping, Disk Arbitration support not compiled in\n" );
 }
@@ -258,22 +247,14 @@ static UInt8 map_option( ULONG option )
     }
 }
 
-#define IF_NAMESIZE 16
-static BOOL map_adapter_name( const NET_LUID *luid, WCHAR *unix_name, DWORD len )
-{
-    return !ConvertInterfaceLuidToAlias( luid, unix_name, len );
-}
-
-static CFStringRef find_service_id( const NET_LUID *adapter )
+static CFStringRef find_service_id( const char *unix_name )
 {
     SCPreferencesRef prefs;
     SCNetworkSetRef set = NULL;
     CFArrayRef services = NULL;
     CFStringRef id, ret = NULL;
-    WCHAR unix_name[IF_NAMESIZE];
     CFIndex i;
 
-    if (!map_adapter_name( adapter, unix_name, ARRAY_SIZE(unix_name) )) return NULL;
     if (!(prefs = SCPreferencesCreate( NULL, CFSTR("mountmgr.sys"), NULL ))) return NULL;
     if (!(set = SCNetworkSetCopyCurrent( prefs ))) goto done;
     if (!(services = SCNetworkSetCopyServices( set ))) goto done;
@@ -281,15 +262,14 @@ static CFStringRef find_service_id( const NET_LUID *adapter )
     for (i = 0; i < CFArrayGetCount( services ); i++)
     {
         SCNetworkServiceRef service;
-        UniChar buf[IF_NAMESIZE] = {0};
+        char buf[16];
         CFStringRef name;
 
         service = CFArrayGetValueAtIndex( services, i );
         name = SCNetworkInterfaceGetBSDName( SCNetworkServiceGetInterface(service) );
-        if (name && CFStringGetLength( name ) < ARRAY_SIZE( buf ))
+        if (name && CFStringGetCString( name, buf, sizeof(buf), kCFStringEncodingUTF8 ))
         {
-            CFStringGetCharacters( name, CFRangeMake(0, CFStringGetLength(name)), buf );
-            if (!lstrcmpW( buf, unix_name ) && (id = SCNetworkServiceGetServiceID( service )))
+            if (!strcmp( buf, unix_name ) && (id = SCNetworkServiceGetServiceID( service )))
             {
                 ret = CFStringCreateCopy( NULL, id );
                 break;
@@ -304,17 +284,17 @@ done:
     return ret;
 }
 
-ULONG get_dhcp_request_param( const NET_LUID *adapter, struct mountmgr_dhcp_request_param *param, char *buf, ULONG offset,
-                              ULONG size )
+NTSTATUS dhcp_request( void *args )
 {
-    CFStringRef service_id = find_service_id( adapter );
+    const struct dhcp_request_params *params = args;
+    CFStringRef service_id = find_service_id( params->unix_name );
     CFDictionaryRef dict;
     CFDataRef value;
     DWORD ret = 0;
     CFIndex len;
 
-    param->offset = 0;
-    param->size   = 0;
+    params->req->offset = 0;
+    params->req->size   = 0;
 
     if (!service_id) return 0;
     if (!(dict = SCDynamicStoreCopyDHCPInfo( NULL, service_id )))
@@ -323,25 +303,25 @@ ULONG get_dhcp_request_param( const NET_LUID *adapter, struct mountmgr_dhcp_requ
         return 0;
     }
     CFRelease( service_id );
-    if (!(value = DHCPInfoGetOptionData( dict, map_option(param->id) )))
+    if (!(value = DHCPInfoGetOptionData( dict, map_option(params->req->id) )))
     {
         CFRelease( dict );
         return 0;
     }
     len = CFDataGetLength( value );
 
-    switch (param->id)
+    switch (params->req->id)
     {
     case OPTION_SUBNET_MASK:
     case OPTION_ROUTER_ADDRESS:
     case OPTION_BROADCAST_ADDRESS:
     {
-        DWORD *ptr = (DWORD *)(buf + offset);
-        if (len == sizeof(*ptr) && size >= sizeof(*ptr))
+        DWORD *ptr = (DWORD *)(params->buffer + params->offset);
+        if (len == sizeof(*ptr) && params->size >= sizeof(*ptr))
         {
             CFDataGetBytes( value, CFRangeMake(0, len), (UInt8 *)ptr );
-            param->offset = offset;
-            param->size   = sizeof(*ptr);
+            params->req->offset = params->offset;
+            params->req->size   = sizeof(*ptr);
             TRACE( "returning %08x\n", *ptr );
         }
         ret = sizeof(*ptr);
@@ -351,33 +331,32 @@ ULONG get_dhcp_request_param( const NET_LUID *adapter, struct mountmgr_dhcp_requ
     case OPTION_DOMAIN_NAME:
     case OPTION_MSFT_IE_PROXY:
     {
-        char *ptr = buf + offset;
-        if (size >= len)
+        char *ptr = params->buffer + params->offset;
+        if (params->size >= len)
         {
             CFDataGetBytes( value, CFRangeMake(0, len), (UInt8 *)ptr );
-            param->offset = offset;
-            param->size   = len;
+            params->req->offset = params->offset;
+            params->req->size   = len;
             TRACE( "returning %s\n", debugstr_an(ptr, len) );
         }
         ret = len;
         break;
     }
     default:
-        FIXME( "option %u not supported\n", param->id );
+        FIXME( "option %u not supported\n", params->req->id );
         break;
     }
 
     CFRelease( dict );
-    return ret;
+    *params->ret_size = ret;
+    return STATUS_SUCCESS;
 }
 
 #elif !defined(SONAME_LIBDBUS_1)
 
-ULONG get_dhcp_request_param( const NET_LUID *adapter, struct mountmgr_dhcp_request_param *param, char *buf, ULONG offset,
-                              ULONG size )
+NTSTATUS dhcp_request( void *args )
 {
-    FIXME( "support not compiled in\n" );
-    return 0;
+    return STATUS_NOT_SUPPORTED;
 }
 
 #endif
