@@ -755,7 +755,7 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
     {
         AMD64_CONTEXT *to = dst;
 
-        to_flags = to->ContextFlags & ~CONTEXT_i386;
+        to_flags = to->ContextFlags & ~CONTEXT_AMD64;
         if ((from->flags & SERVER_CTX_CONTROL) && (to_flags & CONTEXT_AMD64_CONTROL))
         {
             to->ContextFlags |= CONTEXT_AMD64_CONTROL;
@@ -830,7 +830,7 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
     {
         AMD64_CONTEXT *to = dst;
 
-        to_flags = to->ContextFlags & ~CONTEXT_i386;
+        to_flags = to->ContextFlags & ~CONTEXT_AMD64;
         if ((from->flags & SERVER_CTX_CONTROL) && (to_flags & CONTEXT_AMD64_CONTROL))
         {
             to->ContextFlags |= CONTEXT_AMD64_CONTROL;
@@ -850,14 +850,6 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
             to->Rbx = from->integer.i386_regs.ebx;
             to->Rsi = from->integer.i386_regs.esi;
             to->Rdi = from->integer.i386_regs.edi;
-            to->R8  = 0;
-            to->R9  = 0;
-            to->R10 = 0;
-            to->R11 = 0;
-            to->R12 = 0;
-            to->R13 = 0;
-            to->R14 = 0;
-            to->R15 = 0;
         }
         if ((from->flags & SERVER_CTX_SEGMENTS) && (to_flags & CONTEXT_AMD64_SEGMENTS))
         {
@@ -1440,7 +1432,7 @@ void wait_suspend( CONTEXT *context )
 
     contexts_to_server( server_contexts, context );
     /* wait with 0 timeout, will only return once the thread is no longer suspended */
-    server_select( NULL, 0, SELECT_INTERRUPTIBLE, 0, server_contexts, NULL, NULL );
+    server_select( NULL, 0, SELECT_INTERRUPTIBLE, 0, server_contexts, NULL );
     contexts_from_server( context, server_contexts );
     errno = saved_errno;
 }
@@ -1489,7 +1481,7 @@ NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_c
 
         contexts_to_server( server_contexts, context );
         server_select( &select_op, offsetof( select_op_t, wait.handles[1] ), SELECT_INTERRUPTIBLE,
-                       TIMEOUT_INFINITE, server_contexts, NULL, NULL );
+                       TIMEOUT_INFINITE, server_contexts, NULL );
 
         SERVER_START_REQ( get_exception_status )
         {
@@ -2043,9 +2035,9 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_END_REQ;
         return status;
 
-    case ThreadDescription:
+    case ThreadNameInformation:
     {
-        THREAD_DESCRIPTION_INFORMATION *info = data;
+        THREAD_NAME_INFORMATION *info = data;
         data_size_t len, desc_len = 0;
         WCHAR *ptr;
 
@@ -2064,8 +2056,8 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         if (!info) status = STATUS_BUFFER_TOO_SMALL;
         else if (status == STATUS_SUCCESS)
         {
-            info->Description.Length = info->Description.MaximumLength = desc_len;
-            info->Description.Buffer = ptr;
+            info->ThreadName.Length = info->ThreadName.MaximumLength = desc_len;
+            info->ThreadName.Buffer = ptr;
         }
 
         if (ret_len && (status == STATUS_SUCCESS || status == STATUS_BUFFER_TOO_SMALL))
@@ -2236,20 +2228,20 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
         return status;
     }
 
-    case ThreadDescription:
+    case ThreadNameInformation:
     {
-        const THREAD_DESCRIPTION_INFORMATION *info = data;
+        const THREAD_NAME_INFORMATION *info = data;
 
         if (length != sizeof(*info)) return STATUS_INFO_LENGTH_MISMATCH;
         if (!info) return STATUS_ACCESS_VIOLATION;
-        if (info->Description.Length != info->Description.MaximumLength) return STATUS_INVALID_PARAMETER;
-        if (info->Description.Length && !info->Description.Buffer) return STATUS_ACCESS_VIOLATION;
+        if (info->ThreadName.Length != info->ThreadName.MaximumLength) return STATUS_INVALID_PARAMETER;
+        if (info->ThreadName.Length && !info->ThreadName.Buffer) return STATUS_ACCESS_VIOLATION;
 
         SERVER_START_REQ( set_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
             req->mask   = SET_THREAD_INFO_DESCRIPTION;
-            wine_server_add_data( req, info->Description.Buffer, info->Description.Length );
+            wine_server_add_data( req, info->ThreadName.Buffer, info->ThreadName.Length );
             status = wine_server_call( req );
         }
         SERVER_END_REQ;
