@@ -29,6 +29,7 @@
 #include "winreg.h"
 #include "winternl.h"
 #include "hidusage.h"
+#include "wine/gdi_driver.h"
 #include "wine/heap.h"
 
 #define GET_WORD(ptr)  (*(const WORD *)(ptr))
@@ -61,66 +62,10 @@ enum wine_internal_message
     WM_WINE_LAST_DRIVER_MSG = 0x80001fff
 };
 
-typedef struct tagUSER_DRIVER {
-    /* keyboard functions */
-    BOOL   (CDECL *pActivateKeyboardLayout)(HKL, UINT);
-    void   (CDECL *pBeep)(void);
-    INT    (CDECL *pGetKeyNameText)(LONG, LPWSTR, INT);
-    UINT   (CDECL *pGetKeyboardLayoutList)(INT, HKL *);
-    UINT   (CDECL *pMapVirtualKeyEx)(UINT, UINT, HKL);
-    BOOL   (CDECL *pRegisterHotKey)(HWND, UINT, UINT);
-    INT    (CDECL *pToUnicodeEx)(UINT, UINT, const BYTE *, LPWSTR, int, UINT, HKL);
-    void   (CDECL *pUnregisterHotKey)(HWND, UINT, UINT);
-    SHORT  (CDECL *pVkKeyScanEx)(WCHAR, HKL);
-    /* cursor/icon functions */
-    void   (CDECL *pDestroyCursorIcon)(HCURSOR);
-    void   (CDECL *pSetCursor)(HCURSOR);
-    BOOL   (CDECL *pGetCursorPos)(LPPOINT);
-    BOOL   (CDECL *pSetCursorPos)(INT,INT);
-    BOOL   (CDECL *pClipCursor)(LPCRECT);
-    /* clipboard functions */
-    void   (CDECL *pUpdateClipboard)(void);
-    /* display modes */
-    LONG   (CDECL *pChangeDisplaySettingsEx)(LPCWSTR,LPDEVMODEW,HWND,DWORD,LPVOID);
-    BOOL   (CDECL *pEnumDisplayMonitors)(HDC,LPRECT,MONITORENUMPROC,LPARAM);
-    BOOL   (CDECL *pEnumDisplaySettingsEx)(LPCWSTR,DWORD,LPDEVMODEW,DWORD);
-    BOOL   (CDECL *pGetMonitorInfo)(HMONITOR,MONITORINFO*);
-    /* windowing functions */
-    BOOL   (CDECL *pCreateDesktopWindow)(HWND);
-    BOOL   (CDECL *pCreateWindow)(HWND);
-    void   (CDECL *pDestroyWindow)(HWND);
-    void   (CDECL *pFlashWindowEx)(FLASHWINFO*);
-    void   (CDECL *pGetDC)(HDC,HWND,HWND,const RECT *,const RECT *,DWORD);
-    DWORD  (CDECL *pMsgWaitForMultipleObjectsEx)(DWORD,const HANDLE*,DWORD,DWORD,DWORD);
-    void   (CDECL *pReleaseDC)(HWND,HDC);
-    BOOL   (CDECL *pScrollDC)(HDC,INT,INT,HRGN);
-    void   (CDECL *pSetCapture)(HWND,UINT);
-    void   (CDECL *pSetFocus)(HWND);
-    void   (CDECL *pSetLayeredWindowAttributes)(HWND,COLORREF,BYTE,DWORD);
-    void   (CDECL *pSetParent)(HWND,HWND,HWND);
-    void   (CDECL *pSetWindowRgn)(HWND,HRGN,BOOL);
-    void   (CDECL *pSetWindowIcon)(HWND,UINT,HICON);
-    void   (CDECL *pSetWindowStyle)(HWND,INT,STYLESTRUCT*);
-    void   (CDECL *pSetWindowText)(HWND,LPCWSTR);
-    UINT   (CDECL *pShowWindow)(HWND,INT,RECT*,UINT);
-    LRESULT (CDECL *pSysCommand)(HWND,WPARAM,LPARAM);
-    BOOL    (CDECL *pUpdateLayeredWindow)(HWND,const UPDATELAYEREDWINDOWINFO *,const RECT *);
-    LRESULT (CDECL *pWindowMessage)(HWND,UINT,WPARAM,LPARAM);
-    BOOL   (CDECL *pWindowPosChanging)(HWND,HWND,UINT,const RECT *,const RECT *,RECT *,struct window_surface**);
-    void   (CDECL *pWindowPosChanged)(HWND,HWND,UINT,const RECT *,const RECT *,const RECT *,const RECT *,struct window_surface*);
-    /* system parameters */
-    BOOL   (CDECL *pSystemParametersInfo)(UINT,UINT,void*,UINT);
-    /* thread management */
-    void   (CDECL *pThreadDetach)(void);
-} USER_DRIVER;
-
-extern const USER_DRIVER *USER_Driver DECLSPEC_HIDDEN;
+extern const struct user_driver_funcs *USER_Driver DECLSPEC_HIDDEN;
 
 extern void USER_unload_driver(void) DECLSPEC_HIDDEN;
 
-extern BOOL CDECL nulldrv_EnumDisplayMonitors( HDC hdc, RECT *rect, MONITORENUMPROC proc, LPARAM lp ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_GetMonitorInfo( HMONITOR handle, MONITORINFO *info ) DECLSPEC_HIDDEN;
-
 struct received_message_info;
 
 enum user_obj_type
@@ -175,36 +120,6 @@ struct rawinput_thread_data
     RAWINPUT buffer[1]; /* rawinput message data buffer */
 };
 
-/* this is the structure stored in TEB->Win32ClientInfo */
-/* no attempt is made to keep the layout compatible with the Windows one */
-struct user_thread_info
-{
-    HANDLE                        server_queue;           /* Handle to server-side queue */
-    DWORD                         wake_mask;              /* Current queue wake mask */
-    DWORD                         changed_mask;           /* Current queue changed mask */
-    WORD                          recursion_count;        /* SendMessage recursion counter */
-    WORD                          message_count;          /* Get/PeekMessage loop counter */
-    WORD                          hook_call_depth;        /* Number of recursively called hook procs */
-    WORD                          hook_unicode;           /* Is current hook unicode? */
-    HHOOK                         hook;                   /* Current hook */
-    UINT                          active_hooks;           /* Bitmap of active hooks */
-    DPI_AWARENESS                 dpi_awareness;          /* DPI awareness */
-    INPUT_MESSAGE_SOURCE          msg_source;             /* Message source for current message */
-    struct received_message_info *receive_info;           /* Message being currently received */
-    struct wm_char_mapping_data  *wmchar_data;            /* Data for WM_CHAR mappings */
-    DWORD                         GetMessageTimeVal;      /* Value for GetMessageTime */
-    DWORD                         GetMessagePosVal;       /* Value for GetMessagePos */
-    ULONG_PTR                     GetMessageExtraInfoVal; /* Value for GetMessageExtraInfo */
-    struct user_key_state_info   *key_state;              /* Cache of global key state */
-    HKL                           kbd_layout;             /* Current keyboard layout */
-    DWORD                         kbd_layout_id;          /* Current keyboard layout ID */
-    HWND                          top_window;             /* Desktop window */
-    HWND                          msg_window;             /* HWND_MESSAGE parent window */
-    struct rawinput_thread_data  *rawinput;               /* RawInput thread local data / buffer */
-};
-
-C_ASSERT( sizeof(struct user_thread_info) <= sizeof(((TEB *)0)->Win32ClientInfo) );
-
 extern INT global_key_state_counter DECLSPEC_HIDDEN;
 extern BOOL (WINAPI *imm_register_window)(HWND) DECLSPEC_HIDDEN;
 extern void (WINAPI *imm_unregister_window)(HWND) DECLSPEC_HIDDEN;
@@ -246,7 +161,6 @@ extern BOOL rawinput_from_hardware_message(RAWINPUT *rawinput, const struct hard
 extern BOOL rawinput_device_get_usages(HANDLE handle, USAGE *usage_page, USAGE *usage);
 extern struct rawinput_thread_data *rawinput_thread_data(void);
 
-extern void keyboard_init(void) DECLSPEC_HIDDEN;
 extern void create_offscreen_window_surface( const RECT *visible_rect, struct window_surface **surface ) DECLSPEC_HIDDEN;
 
 extern void CLIPBOARD_ReleaseOwner( HWND hwnd ) DECLSPEC_HIDDEN;
@@ -302,6 +216,10 @@ extern BOOL WINPROC_call_window( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 extern const WCHAR *CLASS_GetVersionedName(const WCHAR *classname, UINT *basename_offset,
         WCHAR *combined, BOOL register_class) DECLSPEC_HIDDEN;
 
+/* kernel callbacks */
+
+BOOL WINAPI User32CallEnumDisplayMonitor( struct enum_display_monitor_params *params, ULONG size );
+
 /* message spy definitions */
 
 #define SPY_DISPATCHMESSAGE       0x0100
