@@ -56,7 +56,7 @@ struct d3dkmt_device
     struct list entry;                  /* List entry */
 };
 
-const struct gdi_dc_funcs *driver_funcs;
+static const struct user_driver_funcs lazy_load_driver;
 
 static struct list d3dkmt_adapters = LIST_INIT( d3dkmt_adapters );
 static struct list d3dkmt_devices = LIST_INIT( d3dkmt_devices );
@@ -70,30 +70,17 @@ static pthread_mutex_t driver_lock = PTHREAD_MUTEX_INITIALIZER;
  */
 const struct gdi_dc_funcs *get_display_driver(void)
 {
-    if (!driver_funcs)
+    if (user_driver == &lazy_load_driver)
     {
-        if (!user_callbacks || !user_callbacks->pGetDesktopWindow() || !driver_funcs)
+        if (!user_callbacks || !user_callbacks->pGetDesktopWindow() ||
+            user_driver == &lazy_load_driver)
         {
-            static struct gdi_dc_funcs empty_funcs;
+            static struct user_driver_funcs empty_funcs;
             WARN( "failed to load the display driver, falling back to null driver\n" );
-            driver_funcs = &empty_funcs;
+            __wine_set_display_driver( &empty_funcs, WINE_GDI_DRIVER_VERSION );
         }
     }
-    return driver_funcs;
-}
-
-void CDECL set_display_driver( void *proc )
-{
-    const struct gdi_dc_funcs * (CDECL *wine_get_gdi_driver)( unsigned int ) = proc;
-    const struct gdi_dc_funcs *funcs = NULL;
-
-    funcs = wine_get_gdi_driver( WINE_GDI_DRIVER_VERSION );
-    if (!funcs)
-    {
-        ERR( "Could not create graphics driver\n" );
-        NtTerminateProcess( GetCurrentThread(), 1 );
-    }
-    InterlockedExchangePointer( (void **)&driver_funcs, (void *)funcs );
+    return &user_driver->dc_funcs;
 }
 
 struct monitor_info
@@ -137,8 +124,8 @@ static BOOL CDECL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT
 
 static BOOL CDECL nulldrv_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
 {
-    if (!driver_funcs || !driver_funcs->pCreateCompatibleDC) return TRUE;
-    return driver_funcs->pCreateCompatibleDC( NULL, pdev );
+    if (!user_driver->dc_funcs.pCreateCompatibleDC) return TRUE;
+    return user_driver->dc_funcs.pCreateCompatibleDC( NULL, pdev );
 }
 
 static BOOL CDECL nulldrv_CreateDC( PHYSDEV *dev, LPCWSTR device, LPCWSTR output,
@@ -276,18 +263,18 @@ static INT CDECL nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
     }
     case BITSPIXEL:
     {
+        UNICODE_STRING display;
         DEVMODEW devmode;
-        WCHAR *display;
         DC *dc;
 
-        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY && user_callbacks)
+        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY)
         {
             dc = get_nulldrv_dc( dev );
-            display = dc->display[0] ? dc->display : NULL;
             memset( &devmode, 0, sizeof(devmode) );
             devmode.dmSize = sizeof(devmode);
-            if (user_callbacks->pEnumDisplaySettingsW( display, ENUM_CURRENT_SETTINGS, &devmode )
-                && devmode.dmFields & DM_BITSPERPEL && devmode.dmBitsPerPel)
+            init_unicode_string( &display, dc->display );
+            if (NtUserEnumDisplaySettings( &display, ENUM_CURRENT_SETTINGS, &devmode, 0 ) &&
+                (devmode.dmFields & DM_BITSPERPEL) && devmode.dmBitsPerPel)
                 return devmode.dmBitsPerPel;
         }
         return 32;
@@ -326,46 +313,35 @@ static INT CDECL nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
     case SCALINGFACTORY:  return 0;
     case VREFRESH:
     {
+        UNICODE_STRING display;
         DEVMODEW devmode;
-        WCHAR *display;
         DC *dc;
 
         if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) != DT_RASDISPLAY)
             return 0;
 
-        if (user_callbacks)
-        {
-            dc = get_nulldrv_dc( dev );
-
-            memset( &devmode, 0, sizeof(devmode) );
-            devmode.dmSize = sizeof(devmode);
-            display = dc->display[0] ? dc->display : NULL;
-            if (user_callbacks->pEnumDisplaySettingsW( display, ENUM_CURRENT_SETTINGS, &devmode ))
-                return devmode.dmDisplayFrequency ? devmode.dmDisplayFrequency : 1;
-        }
+        dc = get_nulldrv_dc( dev );
 
+        memset( &devmode, 0, sizeof(devmode) );
+        devmode.dmSize = sizeof(devmode);
+        init_unicode_string( &display, dc->display );
+        if (NtUserEnumDisplaySettings( &display, ENUM_CURRENT_SETTINGS, &devmode, 0 ) &&
+            devmode.dmDisplayFrequency)
+            return devmode.dmDisplayFrequency;
         return 1;
     }
     case DESKTOPHORZRES:
-        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY && user_callbacks)
+        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY)
         {
-            DPI_AWARENESS_CONTEXT context;
-            UINT ret;
-            context = user_callbacks->pSetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
-            ret = user_callbacks->pGetSystemMetrics( SM_CXVIRTUALSCREEN );
-            user_callbacks->pSetThreadDpiAwarenessContext( context );
-            if (ret) return ret;
+            RECT rect = get_virtual_screen_rect();
+            return rect.right - rect.left;
         }
         return NtGdiGetDeviceCaps( dev->hdc, HORZRES );
     case DESKTOPVERTRES:
-        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY && user_callbacks)
+        if (NtGdiGetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY)
         {
-            DPI_AWARENESS_CONTEXT context;
-            UINT ret;
-            context = user_callbacks->pSetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
-            ret = user_callbacks->pGetSystemMetrics( SM_CYVIRTUALSCREEN );
-            user_callbacks->pSetThreadDpiAwarenessContext( context );
-            if (ret) return ret;
+            RECT rect = get_virtual_screen_rect();
+            return rect.bottom - rect.top;
         }
         return NtGdiGetDeviceCaps( dev->hdc, VERTRES );
     case BLTALIGNMENT:    return 0;
@@ -745,6 +721,414 @@ const struct gdi_dc_funcs null_driver =
 };
 
 
+/**********************************************************************
+ * Null user driver
+ *
+ * These are fallbacks for entry points that are not implemented in the real driver.
+ */
+
+static BOOL CDECL nulldrv_ActivateKeyboardLayout( HKL layout, UINT flags )
+{
+    return TRUE;
+}
+
+static void CDECL nulldrv_Beep(void)
+{
+}
+
+static UINT CDECL nulldrv_GetKeyboardLayoutList( INT size, HKL *layouts )
+{
+    return ~0; /* use default implementation */
+}
+
+static INT CDECL nulldrv_GetKeyNameText( LONG lparam, LPWSTR buffer, INT size )
+{
+    return -1; /* use default implementation */
+}
+
+static UINT CDECL nulldrv_MapVirtualKeyEx( UINT code, UINT type, HKL layout )
+{
+    return -1; /* use default implementation */
+}
+
+static BOOL CDECL nulldrv_RegisterHotKey( HWND hwnd, UINT modifiers, UINT vk )
+{
+    return TRUE;
+}
+
+static INT CDECL nulldrv_ToUnicodeEx( UINT virt, UINT scan, const BYTE *state, LPWSTR str,
+                                      int size, UINT flags, HKL layout )
+{
+    return -2; /* use default implementation */
+}
+
+static void CDECL nulldrv_UnregisterHotKey( HWND hwnd, UINT modifiers, UINT vk )
+{
+}
+
+static SHORT CDECL nulldrv_VkKeyScanEx( WCHAR ch, HKL layout )
+{
+    return -256; /* use default implementation */
+}
+
+static void CDECL nulldrv_DestroyCursorIcon( HCURSOR cursor )
+{
+}
+
+static void CDECL nulldrv_SetCursor( HCURSOR cursor )
+{
+}
+
+static BOOL CDECL nulldrv_GetCursorPos( LPPOINT pt )
+{
+    return TRUE;
+}
+
+static BOOL CDECL nulldrv_SetCursorPos( INT x, INT y )
+{
+    return TRUE;
+}
+
+static BOOL CDECL nulldrv_ClipCursor( LPCRECT clip )
+{
+    return TRUE;
+}
+
+static void CDECL nulldrv_UpdateClipboard(void)
+{
+}
+
+static LONG CDECL nulldrv_ChangeDisplaySettingsEx( LPCWSTR name, LPDEVMODEW mode, HWND hwnd,
+                                                   DWORD flags, LPVOID lparam )
+{
+    return DISP_CHANGE_FAILED;
+}
+
+static BOOL CDECL nulldrv_EnumDisplaySettingsEx( LPCWSTR name, DWORD num, LPDEVMODEW mode, DWORD flags )
+{
+    return FALSE;
+}
+
+static void CDECL nulldrv_UpdateDisplayDevices( const struct gdi_device_manager *manager,
+                                                BOOL force, void *param )
+{
+}
+
+static BOOL CDECL nulldrv_CreateDesktopWindow( HWND hwnd )
+{
+    return TRUE;
+}
+
+static BOOL CDECL nulldrv_CreateWindow( HWND hwnd )
+{
+    return TRUE;
+}
+
+static void CDECL nulldrv_DestroyWindow( HWND hwnd )
+{
+}
+
+static void CDECL nulldrv_FlashWindowEx( FLASHWINFO *info )
+{
+}
+
+static void CDECL nulldrv_GetDC( HDC hdc, HWND hwnd, HWND top_win, const RECT *win_rect,
+                                 const RECT *top_rect, DWORD flags )
+{
+}
+
+/* helper for kernel32->ntdll timeout format conversion */
+static inline LARGE_INTEGER *get_nt_timeout( LARGE_INTEGER *time, DWORD timeout )
+{
+    if (timeout == INFINITE) return NULL;
+    time->QuadPart = (ULONGLONG)timeout * -10000;
+    return time;
+}
+
+static HANDLE normalize_std_handle( HANDLE handle )
+{
+    if (handle == (HANDLE)STD_INPUT_HANDLE)
+        return NtCurrentTeb()->Peb->ProcessParameters->hStdInput;
+    if (handle == (HANDLE)STD_OUTPUT_HANDLE)
+        return NtCurrentTeb()->Peb->ProcessParameters->hStdOutput;
+    if (handle == (HANDLE)STD_ERROR_HANDLE)
+        return NtCurrentTeb()->Peb->ProcessParameters->hStdError;
+
+    return handle;
+}
+
+static DWORD CDECL nulldrv_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles, DWORD timeout,
+                                                        DWORD mask, DWORD flags )
+{
+    NTSTATUS status;
+    HANDLE hloc[MAXIMUM_WAIT_OBJECTS];
+    LARGE_INTEGER time;
+    unsigned int i;
+
+    if (!count && !timeout) return WAIT_TIMEOUT;
+    if (count > MAXIMUM_WAIT_OBJECTS)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return WAIT_FAILED;
+    }
+    for (i = 0; i < count; i++) hloc[i] = normalize_std_handle( handles[i] );
+
+    status = NtWaitForMultipleObjects( count, hloc, !(flags & MWMO_WAITALL), !!(flags & MWMO_ALERTABLE),
+                                       get_nt_timeout( &time, timeout ) );
+    if (HIWORD(status))  /* is it an error code? */
+    {
+        SetLastError( RtlNtStatusToDosError(status) );
+        status = WAIT_FAILED;
+    }
+    return status;
+}
+
+static void CDECL nulldrv_ReleaseDC( HWND hwnd, HDC hdc )
+{
+}
+
+static BOOL CDECL nulldrv_ScrollDC( HDC hdc, INT dx, INT dy, HRGN update )
+{
+    RECT rect;
+
+    NtGdiGetAppClipBox( hdc, &rect );
+    return NtGdiBitBlt( hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
+                        hdc, rect.left - dx, rect.top - dy, SRCCOPY, 0, 0 );
+}
+
+static void CDECL nulldrv_SetCapture( HWND hwnd, UINT flags )
+{
+}
+
+static void CDECL nulldrv_SetFocus( HWND hwnd )
+{
+}
+
+static void CDECL nulldrv_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
+{
+}
+
+static void CDECL nulldrv_SetParent( HWND hwnd, HWND parent, HWND old_parent )
+{
+}
+
+static void CDECL nulldrv_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
+{
+}
+
+static void CDECL nulldrv_SetWindowIcon( HWND hwnd, UINT type, HICON icon )
+{
+}
+
+static void CDECL nulldrv_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
+{
+}
+
+static void CDECL nulldrv_SetWindowText( HWND hwnd, LPCWSTR text )
+{
+}
+
+static UINT CDECL nulldrv_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
+{
+    return ~0; /* use default implementation */
+}
+
+static LRESULT CDECL nulldrv_SysCommand( HWND hwnd, WPARAM wparam, LPARAM lparam )
+{
+    return -1;
+}
+
+static BOOL CDECL nulldrv_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO *info,
+                                               const RECT *window_rect )
+{
+    return TRUE;
+}
+
+static LRESULT CDECL nulldrv_WindowMessage( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
+{
+    return 0;
+}
+
+static BOOL CDECL nulldrv_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
+                                             const RECT *window_rect, const RECT *client_rect,
+                                             RECT *visible_rect, struct window_surface **surface )
+{
+    return FALSE;
+}
+
+static void CDECL nulldrv_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags,
+                                            const RECT *window_rect, const RECT *client_rect,
+                                            const RECT *visible_rect, const RECT *valid_rects,
+                                            struct window_surface *surface )
+{
+}
+
+static BOOL CDECL nulldrv_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param, UINT flags )
+{
+    return FALSE;
+}
+
+static void CDECL nulldrv_ThreadDetach( void )
+{
+}
+
+/**********************************************************************
+ * Lazy loading user driver
+ *
+ * Initial driver used before another driver is loaded.
+ * Each entry point simply loads the real driver and chains to it.
+ */
+
+static const struct user_driver_funcs *load_driver(void)
+{
+    get_display_driver();
+    return user_driver;
+}
+
+static BOOL CDECL loaderdrv_ActivateKeyboardLayout( HKL layout, UINT flags )
+{
+    return load_driver()->pActivateKeyboardLayout( layout, flags );
+}
+
+static INT CDECL loaderdrv_GetKeyNameText( LONG lparam, LPWSTR buffer, INT size )
+{
+    return load_driver()->pGetKeyNameText( lparam, buffer, size );
+}
+
+static UINT CDECL loaderdrv_GetKeyboardLayoutList( INT size, HKL *layouts )
+{
+    return load_driver()->pGetKeyboardLayoutList( size, layouts );
+}
+
+static UINT CDECL loaderdrv_MapVirtualKeyEx( UINT code, UINT type, HKL layout )
+{
+    return load_driver()->pMapVirtualKeyEx( code, type, layout );
+}
+
+static INT CDECL loaderdrv_ToUnicodeEx( UINT virt, UINT scan, const BYTE *state, LPWSTR str,
+                                        int size, UINT flags, HKL layout )
+{
+    return load_driver()->pToUnicodeEx( virt, scan, state, str, size, flags, layout );
+}
+
+static void CDECL loaderdrv_UnregisterHotKey( HWND hwnd, UINT modifiers, UINT vk )
+{
+    load_driver()->pUnregisterHotKey( hwnd, modifiers, vk );
+}
+
+static SHORT CDECL loaderdrv_VkKeyScanEx( WCHAR ch, HKL layout )
+{
+    return load_driver()->pVkKeyScanEx( ch, layout );
+}
+
+static LONG CDECL loaderdrv_ChangeDisplaySettingsEx( LPCWSTR name, LPDEVMODEW mode, HWND hwnd,
+                                                     DWORD flags, LPVOID lparam )
+{
+    return load_driver()->pChangeDisplaySettingsEx( name, mode, hwnd, flags, lparam );
+}
+
+static BOOL CDECL loaderdrv_EnumDisplaySettingsEx( LPCWSTR name, DWORD num, LPDEVMODEW mode, DWORD flags )
+{
+    return load_driver()->pEnumDisplaySettingsEx( name, num, mode, flags );
+}
+
+static void CDECL loaderdrv_SetCursor( HCURSOR cursor )
+{
+    load_driver()->pSetCursor( cursor );
+}
+
+static void CDECL loaderdrv_UpdateClipboard(void)
+{
+    load_driver()->pUpdateClipboard();
+}
+
+static void CDECL loaderdrv_UpdateDisplayDevices( const struct gdi_device_manager *manager,
+                                                  BOOL force, void *param )
+{
+    load_driver()->pUpdateDisplayDevices( manager, force, param );
+}
+
+static const struct user_driver_funcs lazy_load_driver =
+{
+    .pActivateKeyboardLayout = loaderdrv_ActivateKeyboardLayout,
+    .pGetKeyNameText = loaderdrv_GetKeyNameText,
+    .pGetKeyboardLayoutList = loaderdrv_GetKeyboardLayoutList,
+    .pMapVirtualKeyEx = loaderdrv_MapVirtualKeyEx,
+    .pToUnicodeEx = loaderdrv_ToUnicodeEx,
+    .pUnregisterHotKey = loaderdrv_UnregisterHotKey,
+    .pVkKeyScanEx = loaderdrv_VkKeyScanEx,
+    .pChangeDisplaySettingsEx = loaderdrv_ChangeDisplaySettingsEx,
+    .pEnumDisplaySettingsEx = loaderdrv_EnumDisplaySettingsEx,
+    .pUpdateDisplayDevices = loaderdrv_UpdateDisplayDevices,
+    .pSetCursor = loaderdrv_SetCursor,
+    .pUpdateClipboard = loaderdrv_UpdateClipboard,
+    .pScrollDC = nulldrv_ScrollDC,
+};
+
+const struct user_driver_funcs *user_driver = &lazy_load_driver;
+
+/******************************************************************************
+ *	     __wine_set_display_driver   (win32u.@)
+ */
+void CDECL __wine_set_display_driver( struct user_driver_funcs *driver, UINT version )
+{
+    if (version != WINE_GDI_DRIVER_VERSION)
+    {
+        ERR( "version mismatch, driver wants %u but win32u has %u\n",
+             version, WINE_GDI_DRIVER_VERSION );
+        return;
+    }
+
+#define SET_USER_FUNC(name) \
+    do { if (!driver->p##name) driver->p##name = nulldrv_##name; } while(0)
+
+    SET_USER_FUNC(ActivateKeyboardLayout);
+    SET_USER_FUNC(Beep);
+    SET_USER_FUNC(GetKeyNameText);
+    SET_USER_FUNC(GetKeyboardLayoutList);
+    SET_USER_FUNC(MapVirtualKeyEx);
+    SET_USER_FUNC(RegisterHotKey);
+    SET_USER_FUNC(ToUnicodeEx);
+    SET_USER_FUNC(UnregisterHotKey);
+    SET_USER_FUNC(VkKeyScanEx);
+    SET_USER_FUNC(DestroyCursorIcon);
+    SET_USER_FUNC(SetCursor);
+    SET_USER_FUNC(GetCursorPos);
+    SET_USER_FUNC(SetCursorPos);
+    SET_USER_FUNC(ClipCursor);
+    SET_USER_FUNC(UpdateClipboard);
+    SET_USER_FUNC(ChangeDisplaySettingsEx);
+    SET_USER_FUNC(EnumDisplaySettingsEx);
+    SET_USER_FUNC(UpdateDisplayDevices);
+    SET_USER_FUNC(CreateDesktopWindow);
+    SET_USER_FUNC(CreateWindow);
+    SET_USER_FUNC(DestroyWindow);
+    SET_USER_FUNC(FlashWindowEx);
+    SET_USER_FUNC(GetDC);
+    SET_USER_FUNC(MsgWaitForMultipleObjectsEx);
+    SET_USER_FUNC(ReleaseDC);
+    SET_USER_FUNC(ScrollDC);
+    SET_USER_FUNC(SetCapture);
+    SET_USER_FUNC(SetFocus);
+    SET_USER_FUNC(SetLayeredWindowAttributes);
+    SET_USER_FUNC(SetParent);
+    SET_USER_FUNC(SetWindowRgn);
+    SET_USER_FUNC(SetWindowIcon);
+    SET_USER_FUNC(SetWindowStyle);
+    SET_USER_FUNC(SetWindowText);
+    SET_USER_FUNC(ShowWindow);
+    SET_USER_FUNC(SysCommand);
+    SET_USER_FUNC(UpdateLayeredWindow);
+    SET_USER_FUNC(WindowMessage);
+    SET_USER_FUNC(WindowPosChanging);
+    SET_USER_FUNC(WindowPosChanged);
+    SET_USER_FUNC(SystemParametersInfo);
+    SET_USER_FUNC(ThreadDetach);
+#undef SET_USER_FUNC
+
+    InterlockedExchangePointer( (void **)&user_driver, driver );
+}
+
 /******************************************************************************
  *		NtGdiExtEscape   (win32u.@)
  *
