@@ -55,7 +55,7 @@ const struct user_callbacks *user_callbacks = NULL;
 static inline HGDIOBJ entry_to_handle( GDI_HANDLE_ENTRY *entry )
 {
     unsigned int idx = entry - gdi_shared->Handles;
-    return LongToHandle( idx | (entry->Unique << NTGDI_HANDLE_TYPE_SHIFT) );
+    return ULongToHandle( idx | (entry->Unique << NTGDI_HANDLE_TYPE_SHIFT) );
 }
 
 static inline GDI_HANDLE_ENTRY *handle_entry( HGDIOBJ handle )
@@ -1167,9 +1167,28 @@ static struct unix_funcs unix_funcs =
     NtGdiUnrealizeObject,
     NtGdiUpdateColors,
     NtGdiWidenPath,
+    NtUserActivateKeyboardLayout,
+    NtUserCallOneParam,
+    NtUserCallTwoParam,
+    NtUserChangeDisplaySettings,
+    NtUserCountClipboardFormats,
+    NtUserEnumDisplayDevices,
+    NtUserEnumDisplayMonitors,
+    NtUserEnumDisplaySettings,
+    NtUserGetDisplayConfigBufferSizes,
+    NtUserGetKeyNameText,
+    NtUserGetKeyboardLayoutList,
+    NtUserGetPriorityClipboardFormat,
+    NtUserGetUpdatedClipboardFormats,
+    NtUserIsClipboardFormatAvailable,
+    NtUserMapVirtualKeyEx,
+    NtUserScrollDC,
+    NtUserSelectPalette,
+    NtUserShowCursor,
+    NtUserToUnicodeEx,
+    NtUserUnregisterHotKey,
+    NtUserVkKeyScanEx,
 
-    GDIRealizePalette,
-    GDISelectPalette,
     GetDCHook,
     MirrorRgn,
     SetDCHook,
@@ -1181,7 +1200,7 @@ static struct unix_funcs unix_funcs =
     __wine_get_vulkan_driver,
     __wine_get_wgl_driver,
     __wine_make_gdi_object_system,
-    set_display_driver,
+    __wine_set_display_driver,
     __wine_set_visible_region,
 };
 
