@@ -23,14 +23,14 @@
 #endif
 
 #include "config.h"
-#include "wine/port.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdarg.h>
 #include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
+#include <sys/types.h>
+#include <dlfcn.h>
 #ifdef HAVE_SDL_H
 # include <SDL.h>
 #endif
@@ -55,12 +55,10 @@
 
 #include "unix_private.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(plugplay);
+WINE_DEFAULT_DEBUG_CHANNEL(hid);
 
 #ifdef SONAME_LIBSDL2
 
-WINE_DECLARE_DEBUG_CHANNEL(hid_report);
-
 static pthread_mutex_t sdl_cs = PTHREAD_MUTEX_INITIALIZER;
 static struct sdl_bus_options options;
 
@@ -80,7 +78,7 @@ MAKE_FUNCPTR(SDL_JoystickInstanceID);
 MAKE_FUNCPTR(SDL_JoystickName);
 MAKE_FUNCPTR(SDL_JoystickNumAxes);
 MAKE_FUNCPTR(SDL_JoystickOpen);
-MAKE_FUNCPTR(SDL_WaitEvent);
+MAKE_FUNCPTR(SDL_WaitEventTimeout);
 MAKE_FUNCPTR(SDL_JoystickNumButtons);
 MAKE_FUNCPTR(SDL_JoystickNumBalls);
 MAKE_FUNCPTR(SDL_JoystickNumHats);
@@ -95,6 +93,7 @@ MAKE_FUNCPTR(SDL_GameControllerOpen);
 MAKE_FUNCPTR(SDL_GameControllerEventState);
 MAKE_FUNCPTR(SDL_HapticClose);
 MAKE_FUNCPTR(SDL_HapticDestroyEffect);
+MAKE_FUNCPTR(SDL_HapticGetEffectStatus);
 MAKE_FUNCPTR(SDL_HapticNewEffect);
 MAKE_FUNCPTR(SDL_HapticOpenFromJoystick);
 MAKE_FUNCPTR(SDL_HapticPause);
@@ -112,6 +111,7 @@ MAKE_FUNCPTR(SDL_JoystickIsHaptic);
 MAKE_FUNCPTR(SDL_GameControllerAddMapping);
 MAKE_FUNCPTR(SDL_RegisterEvents);
 MAKE_FUNCPTR(SDL_PushEvent);
+MAKE_FUNCPTR(SDL_GetTicks);
 static int (*pSDL_JoystickRumble)(SDL_Joystick *joystick, Uint16 low_frequency_rumble, Uint16 high_frequency_rumble, Uint32 duration_ms);
 static Uint16 (*pSDL_JoystickGetProduct)(SDL_Joystick * joystick);
 static Uint16 (*pSDL_JoystickGetProductVersion)(SDL_Joystick * joystick);
@@ -138,6 +138,8 @@ struct sdl_device
     SDL_Haptic *sdl_haptic;
     int haptic_effect_id;
     int effect_ids[256];
+    int effect_state[256];
+    LONG effect_flags;
 };
 
 static inline struct sdl_device *impl_from_unix_device(struct unix_device *iface)
@@ -455,9 +457,11 @@ static NTSTATUS sdl_device_physical_device_control(struct unix_device *iface, US
     {
     case PID_USAGE_DC_ENABLE_ACTUATORS:
         pSDL_HapticSetGain(impl->sdl_haptic, 100);
+        InterlockedOr(&impl->effect_flags, EFFECT_STATE_ACTUATORS_ENABLED);
         return STATUS_SUCCESS;
     case PID_USAGE_DC_DISABLE_ACTUATORS:
         pSDL_HapticSetGain(impl->sdl_haptic, 0);
+        InterlockedAnd(&impl->effect_flags, ~EFFECT_STATE_ACTUATORS_ENABLED);
         return STATUS_SUCCESS;
     case PID_USAGE_DC_STOP_ALL_EFFECTS:
         pSDL_HapticStopAll(impl->sdl_haptic);
@@ -473,15 +477,28 @@ static NTSTATUS sdl_device_physical_device_control(struct unix_device *iface, US
         return STATUS_SUCCESS;
     case PID_USAGE_DC_DEVICE_PAUSE:
         pSDL_HapticPause(impl->sdl_haptic);
+        InterlockedOr(&impl->effect_flags, EFFECT_STATE_DEVICE_PAUSED);
         return STATUS_SUCCESS;
     case PID_USAGE_DC_DEVICE_CONTINUE:
         pSDL_HapticUnpause(impl->sdl_haptic);
+        InterlockedAnd(&impl->effect_flags, ~EFFECT_STATE_DEVICE_PAUSED);
         return STATUS_SUCCESS;
     }
 
     return STATUS_NOT_SUPPORTED;
 }
 
+static NTSTATUS sdl_device_physical_device_set_gain(struct unix_device *iface, BYTE percent)
+{
+    struct sdl_device *impl = impl_from_unix_device(iface);
+
+    TRACE("iface %p, percent %#x.\n", iface, percent);
+
+    pSDL_HapticSetGain(impl->sdl_haptic, percent);
+
+    return STATUS_SUCCESS;
+}
+
 static NTSTATUS sdl_device_physical_effect_control(struct unix_device *iface, BYTE index,
                                                    USAGE control, BYTE iterations)
 {
@@ -556,12 +573,18 @@ static NTSTATUS sdl_device_physical_effect_update(struct unix_device *iface, BYT
     struct sdl_device *impl = impl_from_unix_device(iface);
     int id = impl->effect_ids[index];
     SDL_HapticEffect effect = {0};
+    UINT16 direction;
     NTSTATUS status;
 
     TRACE("iface %p, index %u, params %p.\n", iface, index, params);
 
+    if (params->effect_type == PID_USAGE_UNDEFINED) return STATUS_SUCCESS;
     if ((status = set_effect_type_from_usage(&effect, params->effect_type))) return status;
 
+    /* The first direction we get from PID is in polar coordinate space, so we need to
+     * remove 90° to make it match SDL spherical coordinates. */
+    direction = (params->direction[0] - 9000) % 36000;
+
     switch (params->effect_type)
     {
     case PID_USAGE_ET_SINE:
@@ -574,10 +597,10 @@ static NTSTATUS sdl_device_physical_effect_update(struct unix_device *iface, BYT
         effect.periodic.button = params->trigger_button;
         effect.periodic.interval = params->trigger_repeat_interval;
         effect.periodic.direction.type = SDL_HAPTIC_SPHERICAL;
-        effect.periodic.direction.dir[0] = params->direction[0] * 36000 / 256;
-        effect.periodic.direction.dir[1] = params->direction[1] * 36000 / 256;
+        effect.periodic.direction.dir[0] = direction;
+        effect.periodic.direction.dir[1] = params->direction[1];
         effect.periodic.period = params->periodic.period;
-        effect.periodic.magnitude = params->periodic.magnitude * 128;
+        effect.periodic.magnitude = params->periodic.magnitude;
         effect.periodic.offset = params->periodic.offset;
         effect.periodic.phase = params->periodic.phase;
         effect.periodic.attack_length = params->envelope.attack_time;
@@ -595,8 +618,8 @@ static NTSTATUS sdl_device_physical_effect_update(struct unix_device *iface, BYT
         effect.condition.button = params->trigger_button;
         effect.condition.interval = params->trigger_repeat_interval;
         effect.condition.direction.type = SDL_HAPTIC_SPHERICAL;
-        effect.condition.direction.dir[0] = params->direction[0] * 36000 / 256;
-        effect.condition.direction.dir[1] = params->direction[1] * 36000 / 256;
+        effect.condition.direction.dir[0] = direction;
+        effect.condition.direction.dir[1] = params->direction[1];
         if (params->condition_count >= 1)
         {
             effect.condition.right_sat[0] = params->condition[0].positive_saturation;
@@ -623,8 +646,8 @@ static NTSTATUS sdl_device_physical_effect_update(struct unix_device *iface, BYT
         effect.constant.button = params->trigger_button;
         effect.constant.interval = params->trigger_repeat_interval;
         effect.constant.direction.type = SDL_HAPTIC_SPHERICAL;
-        effect.constant.direction.dir[0] = params->direction[0] * 36000 / 256;
-        effect.constant.direction.dir[1] = params->direction[1] * 36000 / 256;
+        effect.constant.direction.dir[0] = direction;
+        effect.constant.direction.dir[1] = params->direction[1];
         effect.constant.level = params->constant_force.magnitude;
         effect.constant.attack_length = params->envelope.attack_time;
         effect.constant.attack_level = params->envelope.attack_level;
@@ -638,8 +661,8 @@ static NTSTATUS sdl_device_physical_effect_update(struct unix_device *iface, BYT
         effect.ramp.button = params->trigger_button;
         effect.ramp.interval = params->trigger_repeat_interval;
         effect.ramp.direction.type = SDL_HAPTIC_SPHERICAL;
-        effect.ramp.direction.dir[0] = params->direction[0] * 36000 / 256;
-        effect.ramp.direction.dir[1] = params->direction[1] * 36000 / 256;
+        effect.ramp.direction.dir[0] = params->direction[0];
+        effect.ramp.direction.dir[1] = params->direction[1];
         effect.ramp.start = params->ramp_force.ramp_start;
         effect.ramp.end = params->ramp_force.ramp_end;
         effect.ramp.attack_length = params->envelope.attack_time;
@@ -666,10 +689,47 @@ static const struct hid_device_vtbl sdl_device_vtbl =
     sdl_device_stop,
     sdl_device_haptics_start,
     sdl_device_physical_device_control,
+    sdl_device_physical_device_set_gain,
     sdl_device_physical_effect_control,
     sdl_device_physical_effect_update,
 };
 
+static void check_device_effects_state(struct sdl_device *impl)
+{
+    struct unix_device *iface = &impl->unix_device;
+    struct hid_effect_state *effect_state = &iface->hid_physical.effect_state;
+    ULONG effect_flags = InterlockedOr(&impl->effect_flags, 0);
+    unsigned int i, ret;
+
+    if (!impl->sdl_haptic) return;
+    if (!(impl->effect_support & SDL_HAPTIC_STATUS)) return;
+
+    for (i = 0; i < ARRAY_SIZE(impl->effect_ids); ++i)
+    {
+        if (impl->effect_ids[i] == -1) continue;
+        ret = pSDL_HapticGetEffectStatus(impl->sdl_haptic, impl->effect_ids[i]);
+        if (impl->effect_state[i] == ret) continue;
+        impl->effect_state[i] = ret;
+        hid_device_set_effect_state(iface, i, effect_flags | (ret == 1 ? EFFECT_STATE_EFFECT_PLAYING : 0));
+        bus_event_queue_input_report(&event_queue, iface, effect_state->report_buf, effect_state->report_len);
+    }
+}
+
+static void check_all_devices_effects_state(void)
+{
+    static UINT last_ticks = 0;
+    UINT ticks = pSDL_GetTicks();
+    struct sdl_device *impl;
+
+    if (ticks - last_ticks < 10) return;
+    last_ticks = ticks;
+
+    pthread_mutex_lock(&sdl_cs);
+    LIST_FOR_EACH_ENTRY(impl, &device_list, struct sdl_device, unix_device.entry)
+        check_device_effects_state(impl);
+    pthread_mutex_unlock(&sdl_cs);
+}
+
 static BOOL set_report_from_joystick_event(struct sdl_device *impl, SDL_Event *event)
 {
     struct unix_device *iface = &impl->unix_device;
@@ -677,7 +737,7 @@ static BOOL set_report_from_joystick_event(struct sdl_device *impl, SDL_Event *e
 
     if (impl->sdl_controller) return TRUE; /* use controller events instead */
 
-    switch(event->type)
+    switch (event->type)
     {
         case SDL_JOYBUTTONDOWN:
         case SDL_JOYBUTTONUP:
@@ -716,6 +776,8 @@ static BOOL set_report_from_joystick_event(struct sdl_device *impl, SDL_Event *e
         default:
             ERR("TODO: Process Report (0x%x)\n",event->type);
     }
+
+    check_device_effects_state(impl);
     return FALSE;
 }
 
@@ -724,7 +786,7 @@ static BOOL set_report_from_controller_event(struct sdl_device *impl, SDL_Event
     struct unix_device *iface = &impl->unix_device;
     struct hid_device_state *state = &iface->hid_device_state;
 
-    switch(event->type)
+    switch (event->type)
     {
         case SDL_CONTROLLERBUTTONDOWN:
         case SDL_CONTROLLERBUTTONUP:
@@ -770,6 +832,8 @@ static BOOL set_report_from_controller_event(struct sdl_device *impl, SDL_Event
         default:
             ERR("TODO: Process Report (%x)\n",event->type);
     }
+
+    check_device_effects_state(impl);
     return FALSE;
 }
 
@@ -847,7 +911,7 @@ static void process_device_event(SDL_Event *event)
     struct sdl_device *impl;
     SDL_JoystickID id;
 
-    TRACE_(hid_report)("Received action %x\n", event->type);
+    TRACE("Received action %x\n", event->type);
 
     pthread_mutex_lock(&sdl_cs);
 
@@ -908,7 +972,7 @@ NTSTATUS sdl_bus_init(void *args)
     LOAD_FUNCPTR(SDL_JoystickName);
     LOAD_FUNCPTR(SDL_JoystickNumAxes);
     LOAD_FUNCPTR(SDL_JoystickOpen);
-    LOAD_FUNCPTR(SDL_WaitEvent);
+    LOAD_FUNCPTR(SDL_WaitEventTimeout);
     LOAD_FUNCPTR(SDL_JoystickNumButtons);
     LOAD_FUNCPTR(SDL_JoystickNumBalls);
     LOAD_FUNCPTR(SDL_JoystickNumHats);
@@ -923,6 +987,7 @@ NTSTATUS sdl_bus_init(void *args)
     LOAD_FUNCPTR(SDL_GameControllerEventState);
     LOAD_FUNCPTR(SDL_HapticClose);
     LOAD_FUNCPTR(SDL_HapticDestroyEffect);
+    LOAD_FUNCPTR(SDL_HapticGetEffectStatus);
     LOAD_FUNCPTR(SDL_HapticNewEffect);
     LOAD_FUNCPTR(SDL_HapticOpenFromJoystick);
     LOAD_FUNCPTR(SDL_HapticPause);
@@ -940,6 +1005,7 @@ NTSTATUS sdl_bus_init(void *args)
     LOAD_FUNCPTR(SDL_GameControllerAddMapping);
     LOAD_FUNCPTR(SDL_RegisterEvents);
     LOAD_FUNCPTR(SDL_PushEvent);
+    LOAD_FUNCPTR(SDL_GetTicks);
 #undef LOAD_FUNCPTR
     pSDL_JoystickRumble = dlsym(sdl_handle, "SDL_JoystickRumble");
     pSDL_JoystickGetProduct = dlsym(sdl_handle, "SDL_JoystickGetProduct");
@@ -997,8 +1063,8 @@ NTSTATUS sdl_bus_wait(void *args)
     do
     {
         if (bus_event_queue_pop(&event_queue, result)) return STATUS_PENDING;
-        if (pSDL_WaitEvent(&event) != 0) process_device_event(&event);
-        else WARN("SDL_WaitEvent failed: %s\n", pSDL_GetError());
+        if (pSDL_WaitEventTimeout(&event, 10) != 0) process_device_event(&event);
+        else check_all_devices_effects_state();
     } while (event.type != quit_event);
 
     TRACE("SDL main loop exiting\n");
