@@ -34,12 +34,7 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#ifdef HAVE_POLL_H
-# include <poll.h>
-#endif
-#ifdef HAVE_SYS_POLL_H
-# include <sys/poll.h>
-#endif
+#include <poll.h>
 #ifdef HAVE_LIBUDEV_H
 # include <libudev.h>
 #endif
@@ -52,6 +47,7 @@
 #ifdef HAVE_SYS_INOTIFY_H
 # include <sys/inotify.h>
 #endif
+#include <limits.h>
 
 #ifdef HAVE_LINUX_INPUT_H
 # include <linux/input.h>
@@ -96,12 +92,10 @@
 
 #include "unix_private.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(plugplay);
+WINE_DEFAULT_DEBUG_CHANNEL(hid);
 
 #ifdef HAVE_UDEV
 
-WINE_DECLARE_DEBUG_CHANNEL(hid_report);
-
 static pthread_mutex_t udev_cs = PTHREAD_MUTEX_INITIALIZER;
 
 static struct udev *udev_context = NULL;
@@ -203,6 +197,7 @@ struct lnxev_device
 
     int haptic_effect_id;
     int effect_ids[256];
+    LONG effect_flags;
 };
 
 static inline struct lnxev_device *lnxev_impl_from_unix_device(struct unix_device *iface)
@@ -343,9 +338,9 @@ static void hidraw_device_read_report(struct unix_device *iface)
 
     int size = read(impl->base.device_fd, report_buffer, sizeof(report_buffer));
     if (size == -1)
-        TRACE_(hid_report)("Read failed. Likely an unplugged device %d %s\n", errno, strerror(errno));
+        TRACE("Read failed. Likely an unplugged device %d %s\n", errno, strerror(errno));
     else if (size == 0)
-        TRACE_(hid_report)("Failed to read report\n");
+        TRACE("Failed to read report\n");
     else
     {
         /* As described in the Linux kernel driver, when connected over bluetooth, DS4 controllers
@@ -379,7 +374,7 @@ static void hidraw_device_set_output_report(struct unix_device *iface, HID_XFER_
     if ((buffer[0] = packet->reportId))
         count = write(impl->base.device_fd, packet->reportBuffer, length);
     else if (length > sizeof(buffer) - 1)
-        ERR_(hid_report)("id %d length %u >= 8192, cannot write\n", packet->reportId, length);
+        ERR("id %d length %u >= 8192, cannot write\n", packet->reportId, length);
     else
     {
         memcpy(buffer + 1, packet->reportBuffer, length);
@@ -393,7 +388,7 @@ static void hidraw_device_set_output_report(struct unix_device *iface, HID_XFER_
     }
     else
     {
-        ERR_(hid_report)("id %d write failed error: %d %s\n", packet->reportId, errno, strerror(errno));
+        ERR("id %d write failed error: %d %s\n", packet->reportId, errno, strerror(errno));
         io->Information = 0;
         io->Status = STATUS_UNSUCCESSFUL;
     }
@@ -411,7 +406,7 @@ static void hidraw_device_get_feature_report(struct unix_device *iface, HID_XFER
     if ((buffer[0] = packet->reportId) && length <= 0x1fff)
         count = ioctl(impl->base.device_fd, HIDIOCGFEATURE(length), packet->reportBuffer);
     else if (length > sizeof(buffer) - 1)
-        ERR_(hid_report)("id %d length %u >= 8192, cannot read\n", packet->reportId, length);
+        ERR("id %d length %u >= 8192, cannot read\n", packet->reportId, length);
     else
     {
         count = ioctl(impl->base.device_fd, HIDIOCGFEATURE(length + 1), buffer);
@@ -425,7 +420,7 @@ static void hidraw_device_get_feature_report(struct unix_device *iface, HID_XFER
     }
     else
     {
-        ERR_(hid_report)("id %d read failed, error: %d %s\n", packet->reportId, errno, strerror(errno));
+        ERR("id %d read failed, error: %d %s\n", packet->reportId, errno, strerror(errno));
         io->Information = 0;
         io->Status = STATUS_UNSUCCESSFUL;
     }
@@ -447,7 +442,7 @@ static void hidraw_device_set_feature_report(struct unix_device *iface, HID_XFER
     if ((buffer[0] = packet->reportId) && length <= 0x1fff)
         count = ioctl(impl->base.device_fd, HIDIOCSFEATURE(length), packet->reportBuffer);
     else if (length > sizeof(buffer) - 1)
-        ERR_(hid_report)("id %d length %u >= 8192, cannot write\n", packet->reportId, length);
+        ERR("id %d length %u >= 8192, cannot write\n", packet->reportId, length);
     else
     {
         memcpy(buffer + 1, packet->reportBuffer, length);
@@ -461,7 +456,7 @@ static void hidraw_device_set_feature_report(struct unix_device *iface, HID_XFER
     }
     else
     {
-        ERR_(hid_report)("id %d write failed, error: %d %s\n", packet->reportId, errno, strerror(errno));
+        ERR("id %d write failed, error: %d %s\n", packet->reportId, errno, strerror(errno));
         io->Information = 0;
         io->Status = STATUS_UNSUCCESSFUL;
     }
@@ -725,7 +720,10 @@ static NTSTATUS build_report_descriptor(struct unix_device *iface, struct udev_d
 
 static BOOL set_report_from_event(struct unix_device *iface, struct input_event *ie)
 {
+    struct hid_effect_state *effect_state = &iface->hid_physical.effect_state;
     struct lnxev_device *impl = lnxev_impl_from_unix_device(iface);
+    ULONG effect_flags = InterlockedOr(&impl->effect_flags, 0);
+    unsigned int i;
 
     switch (ie->type)
     {
@@ -756,6 +754,14 @@ static BOOL set_report_from_event(struct unix_device *iface, struct input_event
     case EV_REL:
         hid_device_set_rel_axis(iface, impl->rel_map[ie->code], ie->value);
         return FALSE;
+    case EV_FF_STATUS:
+        for (i = 0; i < ARRAY_SIZE(impl->effect_ids); ++i) if (impl->effect_ids[i] == ie->code) break;
+        if (i == ARRAY_SIZE(impl->effect_ids)) return FALSE;
+
+        if (ie->value == FF_STATUS_PLAYING) effect_flags |= EFFECT_STATE_EFFECT_PLAYING;
+        hid_device_set_effect_state(iface, i, effect_flags);
+        bus_event_queue_input_report(&event_queue, iface, effect_state->report_buf, effect_state->report_len);
+        return FALSE;
     default:
         ERR("TODO: Process Report (%i, %i)\n",ie->type, ie->code);
         return FALSE;
@@ -801,9 +807,9 @@ static void lnxev_device_read_report(struct unix_device *iface)
 
     size = read(impl->base.device_fd, &ie, sizeof(ie));
     if (size == -1)
-        TRACE_(hid_report)("Read failed. Likely an unplugged device\n");
+        TRACE("Read failed. Likely an unplugged device\n");
     else if (size == 0)
-        TRACE_(hid_report)("Failed to read report\n");
+        TRACE("Failed to read report\n");
     else if (set_report_from_event(iface, &ie))
         bus_event_queue_input_report(&event_queue, iface, state->report_buf, state->report_len);
 }
@@ -889,6 +895,8 @@ static NTSTATUS lnxev_device_physical_device_control(struct unix_device *iface,
         };
         if (write(impl->base.device_fd, &ie, sizeof(ie)) == -1)
             WARN("write failed %d %s\n", errno, strerror(errno));
+        else
+            InterlockedOr(&impl->effect_flags, EFFECT_STATE_ACTUATORS_ENABLED);
         return STATUS_SUCCESS;
     }
     case PID_USAGE_DC_DISABLE_ACTUATORS:
@@ -901,6 +909,8 @@ static NTSTATUS lnxev_device_physical_device_control(struct unix_device *iface,
         };
         if (write(impl->base.device_fd, &ie, sizeof(ie)) == -1)
             WARN("write failed %d %s\n", errno, strerror(errno));
+        else
+            InterlockedAnd(&impl->effect_flags, ~EFFECT_STATE_ACTUATORS_ENABLED);
         return STATUS_SUCCESS;
     }
     case PID_USAGE_DC_STOP_ALL_EFFECTS:
@@ -921,15 +931,35 @@ static NTSTATUS lnxev_device_physical_device_control(struct unix_device *iface,
         return STATUS_SUCCESS;
     case PID_USAGE_DC_DEVICE_PAUSE:
         WARN("device pause not supported\n");
+        InterlockedOr(&impl->effect_flags, EFFECT_STATE_DEVICE_PAUSED);
         return STATUS_NOT_SUPPORTED;
     case PID_USAGE_DC_DEVICE_CONTINUE:
         WARN("device continue not supported\n");
+        InterlockedAnd(&impl->effect_flags, ~EFFECT_STATE_DEVICE_PAUSED);
         return STATUS_NOT_SUPPORTED;
     }
 
     return STATUS_NOT_SUPPORTED;
 }
 
+static NTSTATUS lnxev_device_physical_device_set_gain(struct unix_device *iface, BYTE percent)
+{
+    struct lnxev_device *impl = lnxev_impl_from_unix_device(iface);
+    struct input_event ie =
+    {
+        .type = EV_FF,
+        .code = FF_GAIN,
+        .value = percent,
+    };
+
+    TRACE("iface %p, percent %#x.\n", iface, percent);
+
+    if (write(impl->base.device_fd, &ie, sizeof(ie)) == -1)
+        WARN("write failed %d %s\n", errno, strerror(errno));
+
+    return STATUS_SUCCESS;
+}
+
 static NTSTATUS lnxev_device_physical_effect_control(struct unix_device *iface, BYTE index,
                                                      USAGE control, BYTE iterations)
 {
@@ -1012,14 +1042,22 @@ static NTSTATUS lnxev_device_physical_effect_update(struct unix_device *iface, B
 
     TRACE("iface %p, index %u, params %p.\n", iface, index, params);
 
+    if (params->effect_type == PID_USAGE_UNDEFINED) return STATUS_SUCCESS;
     if ((status = set_effect_type_from_usage(&effect, params->effect_type))) return status;
 
     effect.replay.length = params->duration;
     effect.replay.delay = params->start_delay;
     effect.trigger.button = params->trigger_button;
     effect.trigger.interval = params->trigger_repeat_interval;
-    /* only supports polar with one direction angle */
-    effect.direction = params->direction[0] * 256;
+
+    /* Linux FF only supports polar direction, and uses an inverted convention compared
+     * to SDL or dinput (see SDL src/haptic/linux/SDL_syshaptic.c), where the force pulls
+     * into the specified direction, instead of coming from it.
+     *
+     * The first direction we get from PID is in polar coordinate space, so we need to
+     * add 180° to make it match Linux coordinates. */
+    effect.direction = (params->direction[0] + 18000) % 36000;
+    effect.direction = effect.direction * 0x800 / 1125;
 
     switch (params->effect_type)
     {
@@ -1029,9 +1067,9 @@ static NTSTATUS lnxev_device_physical_effect_update(struct unix_device *iface, B
     case PID_USAGE_ET_SAWTOOTH_UP:
     case PID_USAGE_ET_SAWTOOTH_DOWN:
         effect.u.periodic.period = params->periodic.period;
-        effect.u.periodic.magnitude = params->periodic.magnitude * 128;
+        effect.u.periodic.magnitude = params->periodic.magnitude;
         effect.u.periodic.offset = params->periodic.offset;
-        effect.u.periodic.phase = params->periodic.phase;
+        effect.u.periodic.phase = params->periodic.phase * 0x800 / 1125;
         effect.u.periodic.envelope.attack_length = params->envelope.attack_time;
         effect.u.periodic.envelope.attack_level = params->envelope.attack_level;
         effect.u.periodic.envelope.fade_length = params->envelope.fade_time;
@@ -1102,6 +1140,7 @@ static const struct hid_device_vtbl lnxev_device_vtbl =
     lnxev_device_stop,
     lnxev_device_haptics_start,
     lnxev_device_physical_device_control,
+    lnxev_device_physical_device_set_gain,
     lnxev_device_physical_effect_control,
     lnxev_device_physical_effect_update,
 };
@@ -1127,13 +1166,13 @@ static void get_device_subsystem_info(struct udev_device *dev, char const *subsy
             if (!strncmp(ptr, "HID_UNIQ=", 9))
             {
                 if (desc->serialnumber[0]) continue;
-                if (sscanf(ptr, "HID_UNIQ=%256s\n", buffer) == 1)
+                if (sscanf(ptr, "HID_UNIQ=%256[^\n]", buffer) == 1)
                     ntdll_umbstowcs(buffer, strlen(buffer) + 1, desc->serialnumber, ARRAY_SIZE(desc->serialnumber));
             }
             if (!strncmp(ptr, "HID_NAME=", 9))
             {
                 if (desc->product[0]) continue;
-                if (sscanf(ptr, "HID_NAME=%256s\n", buffer) == 1)
+                if (sscanf(ptr, "HID_NAME=%256[^\n]", buffer) == 1)
                     ntdll_umbstowcs(buffer, strlen(buffer) + 1, desc->product, ARRAY_SIZE(desc->product));
             }
             if (!strncmp(ptr, "HID_PHYS=", 9) || !strncmp(ptr, "PHYS=\"", 6))
