@@ -18,7 +18,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include "wined3d_private.h"
 
@@ -41,8 +40,10 @@ ULONG CDECL wined3d_sampler_decref(struct wined3d_sampler *sampler)
 
     if (!refcount)
     {
+        wined3d_mutex_lock();
         sampler->parent_ops->wined3d_object_destroyed(sampler->parent);
         sampler->device->adapter->adapter_ops->adapter_destroy_sampler(sampler);
+        wined3d_mutex_unlock();
     }
 
     return refcount;
@@ -172,6 +173,33 @@ static VkSamplerAddressMode vk_address_mode_from_wined3d(enum wined3d_texture_ad
     }
 }
 
+static VkBorderColor vk_border_colour_from_wined3d(const struct wined3d_color *colour)
+{
+    unsigned int i;
+
+    static const struct
+    {
+        struct wined3d_color wined3d_colour;
+        VkBorderColor vk_colour;
+    }
+    colours[] =
+    {
+        {{0.0f, 0.0f, 0.0f, 0.0f}, VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK},
+        {{0.0f, 0.0f, 0.0f, 1.0f}, VK_BORDER_COLOR_FLOAT_OPAQUE_BLACK},
+        {{1.0f, 1.0f, 1.0f, 1.0f}, VK_BORDER_COLOR_FLOAT_OPAQUE_WHITE},
+    };
+
+    for (i = 0; i < ARRAY_SIZE(colours); ++i)
+    {
+        if (!memcmp(&colours[i], colour, sizeof(*colour)))
+            return colours[i].vk_colour;
+    }
+
+    FIXME("Unhandled border colour {%.8e, %.8e, %.8e, %.8e}.\n", colour->r, colour->g, colour->b, colour->a);
+
+    return VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK;
+}
+
 static void wined3d_sampler_vk_cs_init(void *object)
 {
     struct wined3d_sampler_vk *sampler_vk = object;
@@ -204,20 +232,16 @@ static void wined3d_sampler_vk_cs_init(void *object)
     sampler_desc.mipLodBias = desc->lod_bias;
     sampler_desc.anisotropyEnable = desc->max_anisotropy != 1;
     sampler_desc.maxAnisotropy = desc->max_anisotropy;
-    sampler_desc.compareEnable = desc->compare;
+    sampler_desc.compareEnable = !!desc->compare;
     sampler_desc.compareOp = vk_compare_op_from_wined3d(desc->comparison_func);
     sampler_desc.minLod = desc->min_lod;
     sampler_desc.maxLod = desc->max_lod;
     sampler_desc.borderColor = VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK;
     sampler_desc.unnormalizedCoordinates = VK_FALSE;
 
-    if ((desc->address_u == WINED3D_TADDRESS_BORDER || desc->address_v == WINED3D_TADDRESS_BORDER
+    if (desc->address_u == WINED3D_TADDRESS_BORDER || desc->address_v == WINED3D_TADDRESS_BORDER
             || desc->address_w == WINED3D_TADDRESS_BORDER)
-            && (desc->border_color[0] != 0.0f || desc->border_color[1] != 0.0f
-            || desc->border_color[2] != 0.0f || desc->border_color[3] != 0.0f))
-        FIXME("Unhandled border colour {%.8e, %.8e, %.8e, %.8e}.\n",
-                desc->border_color[0], desc->border_color[1],
-                desc->border_color[2], desc->border_color[3]);
+        sampler_desc.borderColor = vk_border_colour_from_wined3d((const struct wined3d_color *)desc->border_color);
     if (desc->mip_base_level)
         FIXME("Unhandled mip_base_level %u.\n", desc->mip_base_level);
     if ((d3d_info->wined3d_creation_flags & WINED3D_SRGB_READ_WRITE_CONTROL) && !desc->srgb_decode)
