@@ -29,6 +29,7 @@
 
 #include "wine/debug.h"
 #include "psdrv.h"
+#include "ddk/winddiui.h"
 
 #include "winuser.h"
 #include "wine/wingdi16.h"
@@ -434,81 +435,12 @@ static INT_PTR CALLBACK PSDRV_PaperDlgProc(HWND hwnd, UINT msg,
 static HPROPSHEETPAGE (WINAPI *pCreatePropertySheetPage) (LPCPROPSHEETPAGEW);
 static int (WINAPI *pPropertySheet) (LPCPROPSHEETHEADERW);
 
-static PRINTERINFO *PSDRV_FindPrinterInfoA(LPCSTR name)
-{
-    int len = MultiByteToWideChar( CP_ACP, 0, name, -1, NULL, 0 );
-    WCHAR *nameW = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
-    PRINTERINFO *pi;
-
-    MultiByteToWideChar( CP_ACP, 0, name, -1, nameW, len );
-    pi = PSDRV_FindPrinterInfo( nameW );
-    HeapFree( GetProcessHeap(), 0, nameW );
-
-    return pi;
-}
-
-/***********************************************************
- *      DEVMODEdupWtoA
- *
- * Creates an ascii copy of supplied devmode on the process heap
- *
- * Copied from dlls/winspool/info.c until full unicodification
- */
-static DEVMODEA *DEVMODEdupWtoA( const DEVMODEW *dmW )
-{
-    DEVMODEA *dmA;
-    DWORD size;
-    BOOL formname;
-    /* there is no pointer dereference here, if your code checking tool complains it's broken */
-    ptrdiff_t off_formname = (const char *)dmW->dmFormName - (const char *)dmW;
-
-    if (!dmW) return NULL;
-    formname = (dmW->dmSize > off_formname);
-    size = dmW->dmSize - CCHDEVICENAME - (formname ? CCHFORMNAME : 0);
-    dmA = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, size + dmW->dmDriverExtra );
-    WideCharToMultiByte( CP_ACP, 0, dmW->dmDeviceName, -1, (LPSTR)dmA->dmDeviceName,
-                         CCHDEVICENAME, NULL, NULL );
-    if (!formname)
-    {
-        memcpy( &dmA->dmSpecVersion, &dmW->dmSpecVersion,
-                dmW->dmSize - CCHDEVICENAME * sizeof(WCHAR) );
-    }
-    else
-    {
-        memcpy( &dmA->dmSpecVersion, &dmW->dmSpecVersion,
-               off_formname - CCHDEVICENAME * sizeof(WCHAR) );
-        WideCharToMultiByte( CP_ACP, 0, dmW->dmFormName, -1, (LPSTR)dmA->dmFormName,
-                             CCHFORMNAME, NULL, NULL );
-        memcpy( &dmA->dmLogPixels, &dmW->dmLogPixels, dmW->dmSize -
-                (off_formname + CCHFORMNAME * sizeof(WCHAR)) );
-    }
-    dmA->dmSize = size;
-    memcpy( (char *)dmA + dmA->dmSize, (const char *)dmW + dmW->dmSize,
-            dmW->dmDriverExtra );
-    return dmA;
-}
-
- /******************************************************************
- *         PSDRV_ExtDeviceMode
+/******************************************************************************
+ *           DrvDocumentProperties    (wineps.drv.@)
  *
  *  Retrieves or modifies device-initialization information for the PostScript
  *  driver, or displays a driver-supplied dialog box for configuring the driver.
  *
- * PARAMETERS
- *  lpszDriver  -- Driver name
- *  hwnd        -- Parent window for the dialog box
- *  lpdmOutput  -- Address of a DEVMODE structure for writing initialization information
- *  lpszDevice  -- Device name
- *  lpszPort    -- Port name
- *  lpdmInput   -- Address of a DEVMODE structure for reading initialization information
- *  lpProfile   -- Name of initialization file, defaults to WIN.INI if NULL
- *  wMode      -- Operation to perform.  Can be a combination if > 0.
- *      (0)             -- Returns number of bytes required by DEVMODE structure
- *      DM_UPDATE (1)   -- Write current settings to environment and initialization file
- *      DM_COPY (2)     -- Write current settings to lpdmOutput
- *      DM_PROMPT (4)   -- Presents the driver's modal dialog box (USER.240)
- *      DM_MODIFY (8)   -- Changes current settings according to lpdmInput before any other operation
- *
  * RETURNS
  *  Returns size of DEVMODE structure if wMode is 0.  Otherwise, IDOK is returned for success
  *  for both dialog and non-dialog operations.  IDCANCEL is returned if the dialog box was cancelled.
@@ -518,31 +450,29 @@ static DEVMODEA *DEVMODEdupWtoA( const DEVMODEW *dmW )
  *
  * Just returns default devmode at the moment.  No use of initialization file.
  */
-INT CDECL PSDRV_ExtDeviceMode(LPSTR lpszDriver, HWND hwnd, LPDEVMODEA lpdmOutput,
-                              LPSTR lpszDevice, LPSTR lpszPort, LPDEVMODEA lpdmInput,
-                              LPSTR lpszProfile, DWORD dwMode)
+INT WINAPI DrvDocumentProperties(HWND hwnd, const WCHAR *device, DEVMODEW *output,
+                                 DEVMODEW *input, DWORD mode)
 {
-  PRINTERINFO *pi = PSDRV_FindPrinterInfoA(lpszDevice);
-  if(!pi) return -1;
+  PRINTERINFO *pi;
 
-  TRACE("(Driver=%s, hwnd=%p, devOut=%p, Device='%s', Port='%s', devIn=%p, Profile='%s', Mode=%04x)\n",
-  lpszDriver, hwnd, lpdmOutput, lpszDevice, lpszPort, lpdmInput, debugstr_a(lpszProfile), dwMode);
+  TRACE("(hwnd=%p, Device='%s', devOut=%p, devIn=%p, Mode=%04x)\n",
+        hwnd, debugstr_w(device), output, input, mode);
 
-  /* If dwMode == 0, return size of DEVMODE structure */
-  if(!dwMode)
-      return pi->Devmode->dmPublic.dmSize + pi->Devmode->dmPublic.dmDriverExtra - CCHDEVICENAME - CCHFORMNAME;
+  if (!(pi = PSDRV_FindPrinterInfo(device))) return -1;
+
+  /* If mode == 0, return size of DEVMODE structure */
+  if (!mode)
+      return pi->Devmode->dmPublic.dmSize + pi->Devmode->dmPublic.dmDriverExtra;
 
   /* If DM_MODIFY is set, change settings in accordance with lpdmInput */
-  if((dwMode & DM_MODIFY) && lpdmInput)
+  if ((mode & DM_MODIFY) && input)
   {
-    DEVMODEW *dmW = GdiConvertToDevmodeW( lpdmInput );
-    TRACE("DM_MODIFY set. devIn->dmFields = %08x\n", lpdmInput->dmFields);
-    if (dmW) PSDRV_MergeDevmodes(pi->Devmode, (PSDRV_DEVMODE *)dmW, pi);
-    HeapFree( GetProcessHeap(), 0, dmW );
+    TRACE("DM_MODIFY set. devIn->dmFields = %08x\n", input->dmFields);
+    PSDRV_MergeDevmodes(pi->Devmode, (PSDRV_DEVMODE *)input, pi);
   }
 
   /* If DM_PROMPT is set, present modal dialog box */
-  if(dwMode & DM_PROMPT) {
+  if (mode & DM_PROMPT) {
     HINSTANCE hinstComctl32;
     HPROPSHEETPAGE hpsp[1];
     PROPSHEETPAGEW psp;
@@ -580,72 +510,49 @@ INT CDECL PSDRV_ExtDeviceMode(LPSTR lpszDriver, HWND hwnd, LPDEVMODEA lpdmOutput
   }
   
   /* If DM_UPDATE is set, should write settings to environment and initialization file */
-  if(dwMode & DM_UPDATE)
+  if (mode & DM_UPDATE)
     FIXME("Mode DM_UPDATE.  Just do the same as DM_COPY\n");
 
   /* If DM_COPY is set, should write settings to lpdmOutput */
-  if((dwMode & DM_COPY) || (dwMode & DM_UPDATE)) {
-    if (lpdmOutput)
-    {
-        DEVMODEA *dmA = DEVMODEdupWtoA( &pi->Devmode->dmPublic );
-        if (dmA) memcpy( lpdmOutput, dmA, dmA->dmSize + dmA->dmDriverExtra );
-        HeapFree( GetProcessHeap(), 0, dmA );
-    }
-    else
-        FIXME("lpdmOutput is NULL what should we do??\n");
-  }
+  if (output && (mode & (DM_COPY | DM_UPDATE)))
+    memcpy( output, &pi->Devmode->dmPublic,
+            pi->Devmode->dmPublic.dmSize + pi->Devmode->dmPublic.dmDriverExtra );
   return IDOK;
 }
-/***********************************************************************
- *	PSDRV_DeviceCapabilities	
- *
- *      Retrieves the capabilities of a printer device driver.
- *
- * Parameters
- *      lpszDriver -- printer driver name
- *      lpszDevice -- printer name
- *      lpszPort -- port name
- *      fwCapability -- device capability
- *      lpszOutput -- output buffer
- *      lpDevMode -- device data buffer
- *
- * Returns
- *      Result depends on the setting of fwCapability.  -1 indicates failure.
+
+/******************************************************************************
+ *           DrvDeviceCapabilities    (wineps.drv.@)
  */
-DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR lpszPort,
-                                     WORD fwCapability, LPSTR lpszOutput, LPDEVMODEA lpDevMode)
+DWORD WINAPI DrvDeviceCapabilities(HANDLE printer, WCHAR *device_name, WORD capability,
+                                   void *output, DEVMODEW *devmode)
 {
   PRINTERINFO *pi;
   DEVMODEW *lpdm;
   DWORD ret;
-  pi = PSDRV_FindPrinterInfoA(lpszDevice);
 
-  TRACE("%s %s %s, %u, %p, %p\n", debugstr_a(lpszDriver), debugstr_a(lpszDevice),
-        debugstr_a(lpszPort), fwCapability, lpszOutput, lpDevMode);
+  TRACE("%s %u, %p, %p\n", debugstr_w(device_name), capability, output, devmode);
 
-  if (!pi) {
-      ERR("no printer info for %s %s, return 0!\n",
-          debugstr_a(lpszDriver), debugstr_a(lpszDevice));
+  if (!(pi = PSDRV_FindPrinterInfo(device_name))) {
+      ERR("no printer info for %s, return 0!\n", debugstr_w(device_name));
       return 0;
   }
 
   lpdm = &pi->Devmode->dmPublic;
-  if (lpDevMode) lpdm = GdiConvertToDevmodeW( lpDevMode );
+  if (devmode) lpdm = devmode;
 
-  switch(fwCapability) {
+  switch(capability) {
 
   case DC_PAPERS:
     {
       PAGESIZE *ps;
-      WORD *wp = (WORD *)lpszOutput;
+      WORD *wp = output;
       int i = 0;
 
       LIST_FOR_EACH_ENTRY(ps, &pi->ppd->PageSizes, PAGESIZE, entry)
       {
         TRACE("DC_PAPERS: %u\n", ps->WinPage);
         i++;
-	if(lpszOutput != NULL)
-	  *wp++ = ps->WinPage;
+        if (output != NULL) *wp++ = ps->WinPage;
       }
       ret = i;
       break;
@@ -654,14 +561,14 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
   case DC_PAPERSIZE:
     {
       PAGESIZE *ps;
-      POINT16 *pt = (POINT16 *)lpszOutput;
+      POINT16 *pt = output;
       int i = 0;
 
       LIST_FOR_EACH_ENTRY(ps, &pi->ppd->PageSizes, PAGESIZE, entry)
       {
         TRACE("DC_PAPERSIZE: %f x %f\n", ps->PaperDimension->x, ps->PaperDimension->y);
         i++;
-	if(lpszOutput != NULL) {
+        if (output != NULL) {
           pt->x = paper_size_from_points( ps->PaperDimension->x );
           pt->y = paper_size_from_points( ps->PaperDimension->y );
 	  pt++;
@@ -674,15 +581,15 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
   case DC_PAPERNAMES:
     {
       PAGESIZE *ps;
-      char *cp = lpszOutput;
+      WCHAR *cp = output;
       int i = 0;
 
       LIST_FOR_EACH_ENTRY(ps, &pi->ppd->PageSizes, PAGESIZE, entry)
       {
         TRACE("DC_PAPERNAMES: %s\n", debugstr_a(ps->FullName));
         i++;
-	if(lpszOutput != NULL) {
-	  lstrcpynA(cp, ps->FullName, 64);
+        if (output != NULL) {
+          MultiByteToWideChar(CP_ACP, 0, ps->FullName, -1, cp, 64);
 	  cp += 64;
 	}
       }
@@ -697,13 +604,13 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
   case DC_BINS:
     {
       INPUTSLOT *slot;
-      WORD *wp = (WORD *)lpszOutput;
+      WORD *wp = output;
       int i = 0;
 
       LIST_FOR_EACH_ENTRY( slot, &pi->ppd->InputSlots, INPUTSLOT, entry )
       {
           i++;
-          if (lpszOutput != NULL)
+          if (output != NULL)
               *wp++ = slot->WinBin;
       }
       ret = i;
@@ -713,15 +620,15 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
   case DC_BINNAMES:
     {
       INPUTSLOT *slot;
-      char *cp = lpszOutput;
+      WCHAR *cp = output;
       int i = 0;
 
       LIST_FOR_EACH_ENTRY( slot, &pi->ppd->InputSlots, INPUTSLOT, entry )
       {
           i++;
-          if (lpszOutput != NULL)
+          if (output != NULL)
           {
-              lstrcpynA( cp, slot->FullName, 24 );
+              MultiByteToWideChar(CP_ACP, 0, slot->FullName, -1, cp, 24);
               cp += 24;
           }
       }
@@ -737,13 +644,13 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
   case DC_ENUMRESOLUTIONS:
     {
         RESOLUTION *res;
-        LONG *lp = (LONG *)lpszOutput;
+        LONG *lp = output;
         int i = 0;
 
         LIST_FOR_EACH_ENTRY(res, &pi->ppd->Resolutions, RESOLUTION, entry)
         {
             i++;
-            if (lpszOutput != NULL)
+            if (output != NULL)
             {
                 lp[0] = res->resx;
                 lp[1] = res->resy;
@@ -936,11 +843,10 @@ DWORD CDECL PSDRV_DeviceCapabilities(LPSTR lpszDriver, LPCSTR lpszDevice, LPCSTR
     break;
 
   default:
-    FIXME("Unsupported capability %d\n", fwCapability);
+    FIXME("Unsupported capability %d\n", capability);
     ret = -1;
   }
 
-  if (lpDevMode) HeapFree( GetProcessHeap(), 0, lpdm );
   return ret;
 }
 
