@@ -64,7 +64,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <string.h>
 #include <stdarg.h>
@@ -74,6 +73,7 @@
 # include <unistd.h>
 #endif
 #include <fcntl.h>
+#include <dlfcn.h>
 #include <limits.h>
 #include <time.h>
 #include <assert.h>
@@ -2064,9 +2064,8 @@ static DWORD WINAPI clipboard_thread( void *arg )
     clipboard_thread_id = GetCurrentThreadId();
     AddClipboardFormatListener( clipboard_hwnd );
     register_builtin_formats();
-    request_selection_contents( clipboard_display, TRUE );
-
     xfixes_init();
+    request_selection_contents( clipboard_display, TRUE );
 
     TRACE( "clipboard thread %04x running\n", GetCurrentThreadId() );
     while (GetMessageW( &msg, 0, 0, 0 )) DispatchMessageW( &msg );
