@@ -70,6 +70,18 @@ struct mem_header
     BYTE               data[1];
 };
 
+/* stack frame for user callbacks */
+struct user_callback_frame
+{
+    struct user_callback_frame *prev_frame;
+    struct mem_header          *temp_list;
+    void                      **ret_ptr;
+    ULONG                      *ret_len;
+    NTSTATUS                    status;
+    __wine_jmp_buf              jmpbuf;
+};
+
+
 SYSTEM_DLL_INIT_BLOCK *pLdrSystemDllInitBlock = NULL;
 
 /* wow64win syscall table */
@@ -79,6 +91,7 @@ static const SYSTEM_SERVICE_TABLE *psdwhwin32;
 static void *   (WINAPI *pBTCpuGetBopCode)(void);
 static void     (WINAPI *pBTCpuProcessInit)(void);
 static void     (WINAPI *pBTCpuSimulate)(void);
+static NTSTATUS (WINAPI *pBTCpuResetToConsistentState)( EXCEPTION_POINTERS * );
 
 
 void *dummy = RtlUnwind;
@@ -142,6 +155,26 @@ NTSTATUS WINAPI wow64_NtAllocateUuids( UINT *args )
 }
 
 
+/***********************************************************************
+ *           wow64_NtCallbackReturn
+ */
+NTSTATUS WINAPI wow64_NtCallbackReturn( UINT *args )
+{
+    void *ret_ptr = get_ptr( &args );
+    ULONG ret_len = get_ulong( &args );
+    NTSTATUS status = get_ulong( &args );
+
+    struct user_callback_frame *frame = NtCurrentTeb()->TlsSlots[WOW64_TLS_USERCALLBACKDATA];
+
+    if (!frame) return STATUS_NO_CALLBACK_ACTIVE;
+
+    *frame->ret_ptr = ret_ptr;
+    *frame->ret_len = ret_len;
+    frame->status = status;
+    __wine_longjmp( &frame->jmpbuf, 1 );
+}
+
+
 /**********************************************************************
  *           wow64_NtClose
  */
@@ -284,6 +317,25 @@ NTSTATUS WINAPI wow64___wine_unix_call( UINT *args )
 }
 
 
+/**********************************************************************
+ *           wow64___wine_unix_spawnvp
+ */
+NTSTATUS WINAPI wow64___wine_unix_spawnvp( UINT *args )
+{
+    ULONG *argv32 = get_ptr( &args );
+    int wait = get_ulong( &args );
+
+    unsigned int i, count = 0;
+    char **argv;
+
+    while (argv32[count]) count++;
+    argv = Wow64AllocateTemp( (count + 1) * sizeof(*argv) );
+    for (i = 0; i < count; i++) argv[i] = ULongToPtr( argv32[i] );
+    argv[count] = NULL;
+    return __wine_unix_spawnvp( argv, wait );
+}
+
+
 /**********************************************************************
  *           wow64_wine_server_call
  */
@@ -515,6 +567,7 @@ static DWORD WINAPI process_init( RTL_RUN_ONCE *once, void *param, void **contex
     module = load_64bit_module( get_cpu_dll_name() );
     GET_PTR( BTCpuGetBopCode );
     GET_PTR( BTCpuProcessInit );
+    GET_PTR( BTCpuResetToConsistentState );
     GET_PTR( BTCpuSimulate );
 
     module = load_64bit_module( L"wow64win.dll" );
@@ -649,6 +702,39 @@ NTSTATUS WINAPI Wow64SystemServiceEx( UINT num, UINT *args )
 }
 
 
+static void cpu_simulate(void);
+
+/**********************************************************************
+ *           simulate_filter
+ */
+static LONG CALLBACK simulate_filter( EXCEPTION_POINTERS *ptrs )
+{
+    Wow64PassExceptionToGuest( ptrs );
+    cpu_simulate();  /* re-enter simulation to run the exception dispatcher */
+    return EXCEPTION_EXECUTE_HANDLER;
+}
+
+
+/**********************************************************************
+ *           cpu_simulate
+ */
+static void cpu_simulate(void)
+{
+    for (;;)
+    {
+        __TRY
+        {
+            pBTCpuSimulate();
+        }
+        __EXCEPT( simulate_filter )
+        {
+            /* restart simulation loop */
+        }
+        __ENDTRY
+    }
+}
+
+
 /**********************************************************************
  *           Wow64AllocateTemp  (wow64.@)
  *
@@ -739,6 +825,86 @@ void WINAPI Wow64ApcRoutine( ULONG_PTR arg1, ULONG_PTR arg2, ULONG_PTR arg3, CON
 }
 
 
+/**********************************************************************
+ *           Wow64KiUserCallbackDispatcher  (wow64.@)
+ */
+NTSTATUS WINAPI Wow64KiUserCallbackDispatcher( ULONG id, void *args, ULONG len,
+                                               void **ret_ptr, ULONG *ret_len )
+{
+    struct user_callback_frame frame;
+
+    frame.prev_frame = NtCurrentTeb()->TlsSlots[WOW64_TLS_USERCALLBACKDATA];
+    frame.temp_list  = NtCurrentTeb()->TlsSlots[WOW64_TLS_TEMPLIST];
+    frame.ret_ptr    = ret_ptr;
+    frame.ret_len    = ret_len;
+
+    NtCurrentTeb()->TlsSlots[WOW64_TLS_USERCALLBACKDATA] = &frame;
+    NtCurrentTeb()->TlsSlots[WOW64_TLS_TEMPLIST] = NULL;
+
+    /* cf. 32-bit KeUserModeCallback */
+    switch (current_machine)
+    {
+    case IMAGE_FILE_MACHINE_I386:
+        {
+            I386_CONTEXT orig_ctx, ctx = { CONTEXT_I386_FULL };
+            void *args_data;
+            ULONG *stack;
+
+            NtQueryInformationThread( GetCurrentThread(), ThreadWow64Context, &ctx, sizeof(ctx), NULL );
+
+            stack = args_data = ULongToPtr( (ctx.Esp - len) & ~15 );
+            memcpy( args_data, args, len );
+            *(--stack) = 0;
+            *(--stack) = len;
+            *(--stack) = PtrToUlong( args_data );
+            *(--stack) = id;
+            *(--stack) = 0xdeadbabe;
+
+            orig_ctx = ctx;
+            ctx.Esp = PtrToUlong( stack );
+            ctx.Eip = pLdrSystemDllInitBlock->pKiUserCallbackDispatcher;
+            NtSetInformationThread( GetCurrentThread(), ThreadWow64Context, &ctx, sizeof(ctx) );
+
+            if (!__wine_setjmpex( &frame.jmpbuf, NULL ))
+                cpu_simulate();
+            else
+                NtSetInformationThread( GetCurrentThread(), ThreadWow64Context,
+                                        &orig_ctx, sizeof(orig_ctx) );
+        }
+        break;
+
+    case IMAGE_FILE_MACHINE_ARMNT:
+        {
+            ARM_CONTEXT orig_ctx, ctx = { CONTEXT_ARM_FULL };
+            void *args_data;
+
+            NtQueryInformationThread( GetCurrentThread(), ThreadWow64Context, &ctx, sizeof(ctx), NULL );
+
+            args_data = ULongToPtr( (ctx.Sp - len) & ~15 );
+            memcpy( args_data, args, len );
+
+            ctx.R0 = id;
+            ctx.R1 = PtrToUlong( args );
+            ctx.R2 = len;
+            ctx.Sp = PtrToUlong( args_data );
+            ctx.Pc = pLdrSystemDllInitBlock->pKiUserCallbackDispatcher;
+            NtSetInformationThread( GetCurrentThread(), ThreadWow64Context, &ctx, sizeof(ctx) );
+
+            if (!__wine_setjmpex( &frame.jmpbuf, NULL ))
+                cpu_simulate();
+            else
+                NtSetInformationThread( GetCurrentThread(), ThreadWow64Context,
+                                        &orig_ctx, sizeof(orig_ctx) );
+        }
+        break;
+    }
+
+    NtCurrentTeb()->TlsSlots[WOW64_TLS_USERCALLBACKDATA] = frame.prev_frame;
+    NtCurrentTeb()->TlsSlots[WOW64_TLS_TEMPLIST] = frame.temp_list;
+    return frame.status;
+}
+
+
 /**********************************************************************
  *           Wow64LdrpInitialize  (wow64.@)
  */
@@ -748,5 +914,16 @@ void WINAPI Wow64LdrpInitialize( CONTEXT *context )
 
     RtlRunOnceExecuteOnce( &init_done, process_init, NULL, NULL );
     thread_init();
-    pBTCpuSimulate();
+    cpu_simulate();
+}
+
+
+/**********************************************************************
+ *           Wow64PrepareForException  (wow64.@)
+ */
+void WINAPI Wow64PrepareForException( EXCEPTION_RECORD *rec, CONTEXT *context )
+{
+    EXCEPTION_POINTERS ptrs = { rec, context };
+
+    pBTCpuResetToConsistentState( &ptrs );
 }
