@@ -179,7 +179,7 @@ struct dbg_thread
                                                  *      - only valid when in_exception is TRUE
                                                  */
     EXCEPTION_RECORD            excpt_record;   /* only valid when in_exception is TRUE */
-    struct
+    struct dbg_frame
     {
         ADDRESS64               addr_pc;
         ADDRESS64               addr_frame;
@@ -188,6 +188,7 @@ struct dbg_thread
         DWORD_PTR               linear_frame;
         DWORD_PTR               linear_stack;
         dbg_ctx_t               context;        /* context we got out of stackwalk for this frame */
+        DWORD                   inline_ctx;
         BOOL                    is_ctx_valid;   /* is the context above valid */
     }*                          frames;
     int                         num_frames;
@@ -308,9 +309,9 @@ extern HANDLE           display_crash_details(HANDLE event);
 extern int              msgbox_res_id(HWND hwnd, UINT textId, UINT captionId, UINT uType);
 
   /* dbg.y */
-extern void             parser_handle(HANDLE);
+extern void             parser_handle(const char*, HANDLE);
 extern int              input_read_line(const char* pfx, char* buffer, int size);
-extern int              input_fetch_entire_line(const char* pfx, char** line);
+extern size_t           input_lex_read_buffer(char* pfx, int size);
 extern HANDLE           WINAPIV parser_generate_command_file(const char*, ...);
 
   /* debug.l */
@@ -386,10 +387,20 @@ extern void             source_free_files(struct dbg_process* p);
 extern void             stack_info(int len);
 extern void             stack_backtrace(DWORD threadID);
 extern BOOL             stack_set_frame(int newframe);
-extern BOOL             stack_get_current_frame(IMAGEHLP_STACK_FRAME* ihsf);
 extern BOOL             stack_get_register_frame(const struct dbg_internal_var* div, DWORD_PTR** pval);
 extern unsigned         stack_fetch_frames(const dbg_ctx_t *ctx);
 extern BOOL             stack_get_current_symbol(SYMBOL_INFO* sym);
+static inline struct dbg_frame*
+                        stack_get_thread_frame(struct dbg_thread* thd, unsigned nf)
+{
+    if (!thd->frames || nf >= thd->num_frames) return NULL;
+    return &thd->frames[nf];
+}
+static inline struct dbg_frame*
+                        stack_get_curr_frame(void)
+{
+    return stack_get_thread_frame(dbg_curr_thread, dbg_curr_thread->curr_frame);
+}
 
   /* symbol.c */
 extern enum sym_get_lval symbol_get_lvalue(const char* name, const int lineno, struct dbg_lvalue* addr, BOOL bp_disp);
@@ -410,7 +421,12 @@ extern enum sym_get_lval symbol_picker_scoped(const char* name, const struct sgv
                                               struct dbg_lvalue* rtn);
 
   /* tgt_active.c */
-extern void             dbg_run_debuggee(const char* args);
+struct list_string
+{
+    char* string;
+    struct list_string* next;
+};
+extern void             dbg_run_debuggee(struct list_string* ls);
 extern void             dbg_wait_next_exception(DWORD cont, int count, int mode);
 extern enum dbg_start   dbg_active_attach(int argc, char* argv[]);
 extern BOOL             dbg_set_curr_thread(DWORD tid);
@@ -465,7 +481,7 @@ extern void             dbg_del_thread(struct dbg_thread* t);
 extern BOOL             dbg_init(HANDLE hProc, const WCHAR* in, BOOL invade);
 extern BOOL             dbg_load_module(HANDLE hProc, HANDLE hFile, const WCHAR* name, DWORD_PTR base, DWORD size);
 extern void             dbg_set_option(const char*, const char*);
-extern void             dbg_start_interactive(HANDLE hFile);
+extern void             dbg_start_interactive(const char*, HANDLE hFile);
 extern void             dbg_init_console(void);
 
   /* gdbproxy.c */
