@@ -87,10 +87,10 @@ static BOOL fill_sym_lvalue(const SYMBOL_INFO* sym, ULONG_PTR base,
         buffer += l;
         lvalue->cookie = DLV_TARGET;
         lvalue->addr.Offset = (ULONG64)*pval + sym->Address;
-        if ((LONG_PTR)sym->Address >= 0)
-            snprintf(buffer, sz, "+%ld]", (ULONG_PTR)sym->Address);
+        if ((LONG64)sym->Address >= 0)
+            snprintf(buffer, sz, "+%I64d]", sym->Address);
         else
-            snprintf(buffer, sz, "-%ld]", -(LONG_PTR)sym->Address);
+            snprintf(buffer, sz, "-%I64d]", -(LONG64)sym->Address);
     }
     else if (sym->Flags & SYMFLAG_VALUEPRESENT)
     {
@@ -105,7 +105,7 @@ static BOOL fill_sym_lvalue(const SYMBOL_INFO* sym, ULONG_PTR base,
             if (buffer) snprintf(buffer, sz, "Couldn't get full value information for %s", sym->Name);
             return FALSE;
         }
-        else if (v.n1.n2.vt & VT_BYREF)
+        else if (V_ISBYREF(&v))
         {
             /* FIXME: this won't work for pointers or arrays, as we don't always
              * know, if the value to be dereferenced lies in debuggee or
@@ -366,7 +366,7 @@ enum sym_get_lval symbol_get_lvalue(const char* name, const int lineno,
     int		                i;
     char                        buffer[512];
     BOOL                        opt;
-    IMAGEHLP_STACK_FRAME        ihsf;
+    struct dbg_frame*           frm;
 
     if (strlen(name) + 4 > sizeof(buffer))
     {
@@ -421,9 +421,9 @@ enum sym_get_lval symbol_get_lvalue(const char* name, const int lineno,
     SymSetExtendedOption(SYMOPT_EX_WINE_NATIVE_MODULES, opt);
 
     /* now grab local symbols */
-    if (stack_get_current_frame(&ihsf) && sgv.num < NUMDBGV)
+    if ((frm = stack_get_curr_frame()) && sgv.num < NUMDBGV && !strchr(name, '!'))
     {
-        sgv.frame_offset = ihsf.FrameOffset;
+        sgv.frame_offset = frm->linear_frame;
         SymEnumSymbols(dbg_curr_process->handle, 0, name, sgv_cb, (void*)&sgv);
     }
 
@@ -491,16 +491,16 @@ enum sym_get_lval symbol_get_lvalue(const char* name, const int lineno,
 BOOL symbol_is_local(const char* name)
 {
     struct sgv_data             sgv;
-    IMAGEHLP_STACK_FRAME        ihsf;
+    struct dbg_frame*           frm;
 
     sgv.num        = 0;
     sgv.num_thunks = 0;
     sgv.name       = name;
     sgv.do_thunks  = FALSE;
 
-    if (stack_get_current_frame(&ihsf))
+    if ((frm = stack_get_curr_frame()))
     {
-        sgv.frame_offset = ihsf.FrameOffset;
+        sgv.frame_offset = frm->linear_frame;
         SymEnumSymbols(dbg_curr_process->handle, 0, name, sgv_cb, (void*)&sgv);
     }
     return sgv.num > 0;
@@ -748,18 +748,18 @@ static BOOL CALLBACK info_locals_cb(PSYMBOL_INFO sym, ULONG size, PVOID ctx)
 
 BOOL symbol_info_locals(void)
 {
-    IMAGEHLP_STACK_FRAME        ihsf;
     ADDRESS64                   addr;
+    struct dbg_frame*           frm;
+
+    if (!(frm = stack_get_curr_frame())) return FALSE;
 
-    stack_get_current_frame(&ihsf);
     addr.Mode = AddrModeFlat;
-    addr.Offset = ihsf.InstructionOffset;
+    addr.Offset = frm->linear_pc;
     print_address(&addr, FALSE);
-    dbg_printf(": (%0*lx)\n", ADDRWIDTH, (DWORD_PTR)ihsf.FrameOffset);
-    SymEnumSymbols(dbg_curr_process->handle, 0, NULL, info_locals_cb, (void*)(DWORD_PTR)ihsf.FrameOffset);
+    dbg_printf(": (%0*Ix)\n", ADDRWIDTH, frm->linear_frame);
+    SymEnumSymbols(dbg_curr_process->handle, 0, NULL, info_locals_cb, (void*)frm->linear_frame);
 
     return TRUE;
-
 }
 
 static BOOL CALLBACK symbols_info_cb(PSYMBOL_INFO sym, ULONG size, PVOID ctx)
@@ -778,7 +778,7 @@ static BOOL CALLBACK symbols_info_cb(PSYMBOL_INFO sym, ULONG size, PVOID ctx)
             mi.ModuleName[len - 5] = '\0';
     }
 
-    dbg_printf("%0*lx: %s!%s", ADDRWIDTH, (ULONG_PTR)sym->Address, mi.ModuleName, sym->Name);
+    dbg_printf("%0*I64x: %s!%s", ADDRWIDTH, sym->Address, mi.ModuleName, sym->Name);
     type.id = sym->TypeIndex;
     type.module = sym->ModBase;
 
