@@ -675,21 +675,22 @@ static void do_field(const unsigned char* start, const unsigned char* end)
             printf("\t\tFriend function V1: '%s' type:%x\n",
                    p_string(&fieldtype->friendfcn_v1.p_name),
                    fieldtype->friendfcn_v1.type);
+            ptr += 2 + 2 + (1 + fieldtype->stmember_v2.p_name.namelen);
             break;
 
         case LF_FRIENDFCN_V2:
             printf("\t\tFriend function V2: '%s' type:%x\n",
                    p_string(&fieldtype->friendfcn_v2.p_name),
                    fieldtype->friendfcn_v2.type);
+            ptr += 2 + 2 + 4 + (1 + fieldtype->stmember_v2.p_name.namelen);
             break;
 
-#if 0
         case LF_FRIENDFCN_V3:
             printf("\t\tFriend function V3: '%s' type:%x\n",
                    fieldtype->friendfcn_v3.name,
                    fieldtype->friendfcn_v3.type);
+            ptr += 2 + 2 + 4 + (strlen(fieldtype->stmember_v3.name) + 1);
             break;
-#endif
 
         case LF_BCLASS_V1:
             leaf_len = numeric_leaf(&value, &fieldtype->bclass_v1.offset);
@@ -735,10 +736,12 @@ static void do_field(const unsigned char* start, const unsigned char* end)
 
         case LF_FRIENDCLS_V1:
             printf("\t\tFriend class V1: type:%x\n", fieldtype->friendcls_v1.type);
+            ptr += 2 + 2;
             break;
 
         case LF_FRIENDCLS_V2:
             printf("\t\tFriend class V2: type:%x\n", fieldtype->friendcls_v2.type);
+            ptr += 2 + 2 + 4;
             break;
 
         case LF_NESTTYPE_V1:
@@ -777,11 +780,23 @@ static void do_field(const unsigned char* start, const unsigned char* end)
         case LF_VFUNCOFF_V1:
             printf("\t\tVirtual function table offset V1: type:%x offset:%x\n",
                    fieldtype->vfuncoff_v1.type, fieldtype->vfuncoff_v1.offset);
+            ptr += 2 + 2 + 4;
             break;
 
         case LF_VFUNCOFF_V2:
             printf("\t\tVirtual function table offset V2: type:%x offset:%x\n",
                    fieldtype->vfuncoff_v2.type, fieldtype->vfuncoff_v2.offset);
+            ptr += 2 + 2 + 4 + 4;
+            break;
+
+        case LF_INDEX_V1:
+            printf("\t\tIndex V1: index:%x\n", fieldtype->index_v1.ref);
+            ptr += 2 + 2;
+            break;
+
+        case LF_INDEX_V2:
+            printf("\t\tIndex V2: index:%x\n", fieldtype->index_v2.ref);
+            ptr += 2 + 2 + 4;
             break;
 
         default:
@@ -1087,6 +1102,20 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
         printf("\n");
         break;
 
+    case LF_VFTABLE_V3:
+        printf("\t%x => VFTable V3 base:%x baseVfTable:%x offset%u\n",
+               curr_type, reftype->vftable_v3.type, reftype->vftable_v3.baseVftable, reftype->vftable_v3.offsetInObjectLayout);
+        {
+            const char* str = reftype->vftable_v3.names;
+            const char* last = str + reftype->vftable_v3.cbstr;
+            while (str < last)
+            {
+                printf("\t\t%s\n", str);
+                str += strlen(str) + 1;
+            }
+        }
+        break;
+
     /* types from IPI (aka #4) stream */
     case LF_FUNC_ID:
         printf("\t%x => FuncId %s scopeId:%04x type:%04x\n",
@@ -1167,13 +1196,13 @@ BOOL codeview_dump_types_from_block(const void* table, unsigned long len)
     return TRUE;
 }
 
-static void dump_defrange(const struct cv_addr_range* range, const void* last, const char* pfx)
+static void dump_defrange(const struct cv_addr_range* range, const void* last, unsigned indent)
 {
     const struct cv_addr_gap* gap;
 
-    printf("%s%04x:%08x range:#%x\n", pfx, range->isectStart, range->offStart, range->cbRange);
+    printf("%*s\\- %04x:%08x range:#%x\n", indent, "", range->isectStart, range->offStart, range->cbRange);
     for (gap = (const struct cv_addr_gap*)(range + 1); (const void*)(gap + 1) <= last; ++gap)
-        printf("%s\toffset:%x range:#%x\n", pfx, gap->gapStartOffset, gap->cbRange);
+        printf("%*s  | offset:%x range:#%x\n", indent, "", gap->gapStartOffset, gap->cbRange);
 }
 
 /* return address of first byte after the symbol */
@@ -1206,7 +1235,12 @@ static unsigned binannot_uncompress(const unsigned char** pptr)
     return res;
 }
 
-static void dump_binannot(const unsigned char* ba, const char* last, const char* pfx)
+static inline int binannot_getsigned(unsigned i)
+{
+    return (i & 1) ? -(int)(i >> 1) : (i >> 1);
+}
+
+static void dump_binannot(const unsigned char* ba, const char* last, unsigned indent)
 {
     while (ba < (const unsigned char*)last)
     {
@@ -1215,76 +1249,138 @@ static void dump_binannot(const unsigned char* ba, const char* last, const char*
         {
         case BA_OP_Invalid:
             /* not clear if param? */
-            printf("%sInvalid\n", pfx);
+            printf("%*s  | Invalid\n", indent, "");
             break;
         case BA_OP_CodeOffset:
-            printf("%sCodeOffset %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | CodeOffset %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeCodeOffsetBase:
-            printf("%sChangeCodeOffsetBase %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeCodeOffsetBase %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeCodeOffset:
-            printf("%sChangeCodeOffset %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeCodeOffset %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeCodeLength:
-            printf("%sChangeCodeLength %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeCodeLength %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeFile:
-            printf("%sChangeFile %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeFile %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeLineOffset:
-            printf("%sChangeLineOffset %d\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeLineOffset %d\n", indent, "", binannot_getsigned(binannot_uncompress(&ba)));
             break;
         case BA_OP_ChangeLineEndDelta:
-            printf("%sChangeLineEndDelta %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeLineEndDelta %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeRangeKind:
-            printf("%sChangeRangeKind %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeRangeKind %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeColumnStart:
-            printf("%sChangeColumnStart %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeColumnStart %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeColumnEndDelta:
-            printf("%sChangeColumnEndDelta %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeColumnEndDelta %u\n", indent, "", binannot_uncompress(&ba));
             break;
         case BA_OP_ChangeCodeOffsetAndLineOffset:
             {
                 unsigned p1 = binannot_uncompress(&ba);
-                printf("%sChangeCodeOffsetAndLineOffset %u %u (0x%x)\n", pfx, p1 & 0xf, p1 >> 4, p1);
+                printf("%*s  | ChangeCodeOffsetAndLineOffset %u %u (0x%x)\n", indent, "", p1 & 0xf, binannot_getsigned(p1 >> 4), p1);
             }
             break;
         case BA_OP_ChangeCodeLengthAndCodeOffset:
             {
                 unsigned p1 = binannot_uncompress(&ba);
                 unsigned p2 = binannot_uncompress(&ba);
-                printf("%sChangeCodeLengthAndCodeOffset %u %u\n", pfx, p1, p2);
+                printf("%*s  | ChangeCodeLengthAndCodeOffset %u %u\n", indent, "", p1, p2);
             }
             break;
         case BA_OP_ChangeColumnEnd:
-            printf("%sChangeColumnEnd %u\n", pfx, binannot_uncompress(&ba));
+            printf("%*s  | ChangeColumnEnd %u\n", indent, "", binannot_uncompress(&ba));
             break;
 
-        default: printf("%sUnsupported op %d %x\n", pfx, opcode, opcode); /* may cause issues because of param */
+        default: printf(">>> Unsupported op %d %x\n", opcode, opcode); /* may cause issues because of param */
         }
     }
 }
 
-BOOL codeview_dump_symbols(const void* root, unsigned long size)
+struct symbol_dumper
+{
+    unsigned depth;
+    unsigned alloc;
+    struct
+    {
+        unsigned end;
+        const union codeview_symbol* sym;
+    }* stack;
+};
+
+static void init_symbol_dumper(struct symbol_dumper* sd)
+{
+    sd->depth = 0;
+    sd->alloc = 16;
+    sd->stack = malloc(sd->alloc * sizeof(sd->stack[0]));
+}
+
+static void push_symbol_dumper(struct symbol_dumper* sd, const union codeview_symbol* sym, unsigned end)
+{
+    if (!sd->stack) return;
+    if (sd->depth >= sd->alloc &&
+        !(sd->stack = realloc(sd->stack, (sd->alloc *= 2) * sizeof(sd->stack[0]))))
+        return;
+    sd->stack[sd->depth].end = end;
+    sd->stack[sd->depth].sym = sym;
+    sd->depth++;
+}
+
+static unsigned short pop_symbol_dumper(struct symbol_dumper* sd, unsigned end)
+{
+    if (!sd->stack) return 0;
+    if (!sd->depth)
+    {
+        printf(">>> Error in stack\n");
+        return 0;
+    }
+    sd->depth--;
+    if (sd->stack[sd->depth].end != end)
+        printf(">>> Wrong end reference\n");
+    return sd->stack[sd->depth].sym->generic.id;
+}
+
+static void dispose_symbol_dumper(struct symbol_dumper* sd)
+{
+    free(sd->stack);
+}
+
+BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long size)
 {
     unsigned int i;
     int          length;
-    char*        curr_func = NULL;
-    int          nest_block = 0;
+    struct symbol_dumper sd;
+
+    init_symbol_dumper(&sd);
     /*
      * Loop over the different types of records and whenever we
      * find something we are interested in, record it and move on.
      */
-    for (i = 0; i < size; i += length)
+    for (i = start; i < size; i += length)
     {
         const union codeview_symbol* sym = (const union codeview_symbol*)((const char*)root + i);
+        unsigned indent, ref;
+
         length = sym->generic.len + 2;
         if (!sym->generic.id || length < 4) break;
-        printf("\t%04x => ", i + 4); /* ref is made after id and len */
+        indent = printf("        %04x => ", i);
+
+        switch (sym->generic.id)
+        {
+        case S_END:
+        case S_INLINESITE_END:
+            indent += printf("%*s", 2 * sd.depth - 2, "");
+            break;
+        default:
+            indent += printf("%*s", 2 * sd.depth, "");
+            break;
+        }
 
         switch (sym->generic.id)
         {
@@ -1351,7 +1447,7 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                    p_string(&sym->thunk_v1.p_name),
                    sym->thunk_v1.segment, sym->thunk_v1.offset,
                    sym->thunk_v1.thunk_len, sym->thunk_v1.thtype);
-            curr_func = xstrdup(p_string(&sym->thunk_v1.p_name));
+	    push_symbol_dumper(&sd, sym, sym->thunk_v1.pend);
 	    break;
 
 	case S_THUNK32:
@@ -1359,50 +1455,38 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                    sym->thunk_v3.name,
                    sym->thunk_v3.segment, sym->thunk_v3.offset,
                    sym->thunk_v3.thunk_len, sym->thunk_v3.thtype);
-            curr_func = xstrdup(sym->thunk_v3.name);
+	    push_symbol_dumper(&sd, sym, sym->thunk_v3.pend);
 	    break;
 
         /* Global and static functions */
 	case S_GPROC32_16t:
 	case S_LPROC32_16t:
             printf("%s-Proc V1: '%s' (%04x:%08x#%x) type:%x attr:%x\n",
-                   sym->generic.id == S_GPROC32_16t ? "Global" : "-Local",
+                   sym->generic.id == S_GPROC32_16t ? "Global" : "Local",
                    p_string(&sym->proc_v1.p_name),
                    sym->proc_v1.segment, sym->proc_v1.offset,
                    sym->proc_v1.proc_len, sym->proc_v1.proctype,
                    sym->proc_v1.flags);
-            printf("\t\tDebug: start=%08x end=%08x\n",
-                   sym->proc_v1.debug_start, sym->proc_v1.debug_end);
-            if (nest_block)
-            {
-                printf(">>> prev func '%s' still has nest_block %u count\n", curr_func, nest_block);
-                nest_block = 0;
-            }
-            curr_func = xstrdup(p_string(&sym->proc_v1.p_name));
-/* EPP 	unsigned int	pparent; */
-/* EPP 	unsigned int	pend; */
-/* EPP 	unsigned int	next; */
+            printf("%*s\\- Debug: start=%08x end=%08x\n",
+                   indent, "", sym->proc_v1.debug_start, sym->proc_v1.debug_end);
+	    printf("%*s\\- parent:<%x> end:<%x> next<%x>\n",
+                   indent, "", sym->proc_v1.pparent, sym->proc_v1.pend, sym->proc_v1.next);
+	    push_symbol_dumper(&sd, sym, sym->proc_v1.pend);
 	    break;
 
 	case S_GPROC32_ST:
 	case S_LPROC32_ST:
             printf("%s-Proc V2: '%s' (%04x:%08x#%x) type:%x attr:%x\n",
-                   sym->generic.id == S_GPROC32_ST ? "Global" : "-Local",
+                   sym->generic.id == S_GPROC32_ST ? "Global" : "Local",
                    p_string(&sym->proc_v2.p_name),
                    sym->proc_v2.segment, sym->proc_v2.offset,
                    sym->proc_v2.proc_len, sym->proc_v2.proctype,
                    sym->proc_v2.flags);
-            printf("\t\tDebug: start=%08x end=%08x\n",
-                   sym->proc_v2.debug_start, sym->proc_v2.debug_end);
-            if (nest_block)
-            {
-                printf(">>> prev func '%s' still has nest_block %u count\n", curr_func, nest_block);
-                nest_block = 0;
-            }
-            curr_func = xstrdup(p_string(&sym->proc_v2.p_name));
-/* EPP 	unsigned int	pparent; */
-/* EPP 	unsigned int	pend; */
-/* EPP 	unsigned int	next; */
+            printf("%*s\\- Debug: start=%08x end=%08x\n",
+                   indent, "", sym->proc_v2.debug_start, sym->proc_v2.debug_end);
+            printf("%*s\\- parent:<%x> end:<%x> next<%x>\n",
+                   indent, "", sym->proc_v2.pparent, sym->proc_v2.pend, sym->proc_v2.next);
+	    push_symbol_dumper(&sd, sym, sym->proc_v2.pend);
 	    break;
 
         case S_LPROC32:
@@ -1413,77 +1497,69 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                    sym->proc_v3.segment, sym->proc_v3.offset,
                    sym->proc_v3.proc_len, sym->proc_v3.proctype,
                    sym->proc_v3.flags);
-            printf("\t\tDebug: start=%08x end=%08x\n",
-                   sym->proc_v3.debug_start, sym->proc_v3.debug_end);
-            if (nest_block)
-            {
-                printf(">>> prev func '%s' still has nest_block %u count\n", curr_func, nest_block);
-                nest_block = 0;
-            }
-            curr_func = xstrdup(sym->proc_v3.name);
-/* EPP 	unsigned int	pparent; */
-/* EPP 	unsigned int	pend; */
-/* EPP 	unsigned int	next; */
+            printf("%*s\\- Debug: start=%08x end=%08x\n",
+                   indent, "", sym->proc_v3.debug_start, sym->proc_v3.debug_end);
+            printf("%*s\\- parent:<%x> end:<%x> next<%x>\n",
+                   indent, "", sym->proc_v3.pparent, sym->proc_v3.pend, sym->proc_v3.next);
+            push_symbol_dumper(&sd, sym, sym->proc_v3.pend);
             break;
 
         /* Function parameters and stack variables */
 	case S_BPREL32_16t:
-            printf("BP-relative V1: '%s' @%d type:%x (%s)\n",
+            printf("BP-relative V1: '%s' @%d type:%x\n",
                    p_string(&sym->stack_v1.p_name),
-                   sym->stack_v1.offset, sym->stack_v1.symtype, curr_func);
+                   sym->stack_v1.offset, sym->stack_v1.symtype);
             break;
 
 	case S_BPREL32_ST:
-            printf("BP-relative V2: '%s' @%d type:%x (%s)\n",
+            printf("BP-relative V2: '%s' @%d type:%x\n",
                    p_string(&sym->stack_v2.p_name),
-                   sym->stack_v2.offset, sym->stack_v2.symtype, curr_func);
+                   sym->stack_v2.offset, sym->stack_v2.symtype);
             break;
 
         case S_BPREL32:
-            printf("BP-relative V3: '%s' @%d type:%x (in %s)\n",
+            printf("BP-relative V3: '%s' @%d type:%x\n",
                    sym->stack_v3.name, sym->stack_v3.offset,
-                   sym->stack_v3.symtype, curr_func);
+                   sym->stack_v3.symtype);
             break;
 
         case S_REGREL32:
-            printf("Reg-relative V3: '%s' @%d type:%x reg:%x (in %s)\n",
+            printf("Reg-relative V3: '%s' @%d type:%x reg:%x\n",
                    sym->regrel_v3.name, sym->regrel_v3.offset,
-                   sym->regrel_v3.symtype, sym->regrel_v3.reg, curr_func);
+                   sym->regrel_v3.symtype, sym->regrel_v3.reg);
             break;
 
         case S_REGISTER_16t:
-            printf("Register V1 '%s' in %s type:%x register:%x\n",
+            printf("Register V1 '%s' type:%x register:%x\n",
                    p_string(&sym->register_v1.p_name),
-                   curr_func, sym->register_v1.reg, sym->register_v1.type);
+                   sym->register_v1.reg, sym->register_v1.type);
             break;
 
         case S_REGISTER_ST:
-            printf("Register V2 '%s' in %s type:%x register:%x\n",
+            printf("Register V2 '%s' type:%x register:%x\n",
                    p_string(&sym->register_v2.p_name),
-                   curr_func, sym->register_v2.reg, sym->register_v2.type);
+                   sym->register_v2.reg, sym->register_v2.type);
             break;
 
         case S_REGISTER:
-            printf("Register V3 '%s' in %s type:%x register:%x\n",
-                   sym->register_v3.name,
-                   curr_func, sym->register_v3.reg, sym->register_v3.type);
+            printf("Register V3 '%s' type:%x register:%x\n",
+                   sym->register_v3.name, sym->register_v3.reg, sym->register_v3.type);
             break;
 
         case S_BLOCK32_ST:
-            printf("Block V1 '%s' in '%s' (%04x:%08x#%08x)\n",
+            printf("Block V1 '%s' (%04x:%08x#%08x)\n",
                    p_string(&sym->block_v1.p_name),
-                   curr_func,
                    sym->block_v1.segment, sym->block_v1.offset,
                    sym->block_v1.length);
-            nest_block++;
+            push_symbol_dumper(&sd, sym, sym->block_v1.end);
             break;
 
         case S_BLOCK32:
-            printf("Block V3 '%s' in '%s' (%04x:%08x#%08x) parent:%u end:%x\n",
-                   sym->block_v3.name, curr_func,
+            printf("Block V3 '%s' (%04x:%08x#%08x) parent:<%u> end:<%x>\n",
+                   sym->block_v3.name,
                    sym->block_v3.segment, sym->block_v3.offset, sym->block_v3.length,
                    sym->block_v3.parent, sym->block_v3.end);
-            nest_block++;
+            push_symbol_dumper(&sd, sym, sym->block_v3.end);
             break;
 
         /* Additional function information */
@@ -1504,16 +1580,16 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
             break;
 
         case S_END:
-            if (nest_block)
+            ref = sd.depth ? (const char*)sd.stack[sd.depth - 1].sym - (const char*)root : 0;
+            switch (pop_symbol_dumper(&sd, i))
             {
-                nest_block--;
-                printf("End-Of block (%u)\n", nest_block);
-            }
-            else
-            {
-                printf("End-Of %s\n", curr_func);
-                free(curr_func);
-                curr_func = NULL;
+            case S_BLOCK32_ST:
+            case S_BLOCK32:
+                printf("End-Of block <%x>\n", ref);
+                break;
+            default:
+                printf("End-Of <%x>\n", ref);
+                break;
             }
             break;
 
@@ -1537,7 +1613,7 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                 const char* ptr = sym->compile2_v2.p_name.name + sym->compile2_v2.p_name.namelen;
                 while (*ptr)
                 {
-                    printf("\t\t%s => ", ptr); ptr += strlen(ptr) + 1;
+                    printf("%*s| %s => ", indent, "", ptr); ptr += strlen(ptr) + 1;
                     printf("%s\n", ptr); ptr += strlen(ptr) + 1;
                 }
             }
@@ -1555,7 +1631,7 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                 const char* ptr = sym->compile2_v3.name + strlen(sym->compile2_v3.name) + 1;
                 while (*ptr)
                 {
-                    printf("\t\t%s => ", ptr); ptr += strlen(ptr) + 1;
+                    printf("%*s| %s => ", indent, "", ptr); ptr += strlen(ptr) + 1;
                     printf("%s\n", ptr); ptr += strlen(ptr) + 1;
                 }
             }
@@ -1576,12 +1652,12 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                 const char*             x1 = (const char*)sym + 4 + 1;
                 const char*             x2;
 
-                printf("\tTool conf V3\n");
+                printf("Tool conf V3\n");
                 while (*x1)
                 {
                     x2 = x1 + strlen(x1) + 1;
                     if (!*x2) break;
-                    printf("\t\t%s: %s\n", x1, x2);
+                    printf("%*s| %s: %s\n", indent, "", x1, x2);
                     x1 = x2 + strlen(x2) + 1;
                 }
             }
@@ -1608,14 +1684,14 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
             break;
 
         case S_LABEL32_ST:
-            printf("Label V1 '%s' in '%s' (%04x:%08x)\n",
+            printf("Label V1 '%s' (%04x:%08x)\n",
                    p_string(&sym->label_v1.p_name),
-                   curr_func, sym->label_v1.segment, sym->label_v1.offset);
+                   sym->label_v1.segment, sym->label_v1.offset);
             break;
 
         case S_LABEL32:
-            printf("Label V3 '%s' in '%s' (%04x:%08x) flag:%x\n",
-                   sym->label_v3.name, curr_func, sym->label_v3.segment,
+            printf("Label V3 '%s' (%04x:%08x) flag:%x\n",
+                   sym->label_v3.name, sym->label_v3.segment,
                    sym->label_v3.offset, sym->label_v3.flags);
             break;
 
@@ -1674,7 +1750,7 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
 
                 pname = PSTRING(sym, length);
                 length += (pname->namelen + 1 + 3) & ~3;
-                printf("\t%08x %08x %08x '%s'\n",
+                printf("%08x %08x %08x '%s'\n",
                        *(((const DWORD*)sym) + 1), *(((const DWORD*)sym) + 2), *(((const DWORD*)sym) + 3),
                        p_string(pname));
             }
@@ -1744,22 +1820,22 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
 
         case S_DEFRANGE:
             printf("DefRange dia:%x\n", sym->defrange_v3.program);
-            dump_defrange(&sym->defrange_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_v3.range, get_last(sym), indent);
             break;
         case S_DEFRANGE_SUBFIELD:
             printf("DefRange-subfield V3 dia:%x off-parent:%x\n",
                    sym->defrange_subfield_v3.program, sym->defrange_subfield_v3.offParent);
-            dump_defrange(&sym->defrange_subfield_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_subfield_v3.range, get_last(sym), indent);
             break;
         case S_DEFRANGE_REGISTER:
             printf("DefRange-register V3 reg:%x attr-unk:%x\n",
                    sym->defrange_register_v3.reg, sym->defrange_register_v3.attr);
-            dump_defrange(&sym->defrange_register_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_register_v3.range, get_last(sym), indent);
             break;
         case S_DEFRANGE_FRAMEPOINTER_REL:
             printf("DefRange-framepointer-rel V3 offFP:%x\n",
                    sym->defrange_frameptrrel_v3.offFramePointer);
-            dump_defrange(&sym->defrange_frameptrrel_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_frameptrrel_v3.range, get_last(sym), indent);
             break;
         case S_DEFRANGE_FRAMEPOINTER_REL_FULL_SCOPE:
             printf("DefRange-framepointer-rel-fullscope V3 offFP:%x\n",
@@ -1770,13 +1846,13 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                    sym->defrange_subfield_register_v3.reg,
                    sym->defrange_subfield_register_v3.attr,
                    sym->defrange_subfield_register_v3.offParent);
-            dump_defrange(&sym->defrange_subfield_register_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_subfield_register_v3.range, get_last(sym), indent);
             break;
         case S_DEFRANGE_REGISTER_REL:
             printf("DefRange-register-rel V3 reg:%x off-parent:%x off-BP:%x\n",
                    sym->defrange_registerrel_v3.baseReg, sym->defrange_registerrel_v3.offsetParent,
                    sym->defrange_registerrel_v3.offBasePointer);
-            dump_defrange(&sym->defrange_registerrel_v3.range, get_last(sym), "\t\t");
+            dump_defrange(&sym->defrange_registerrel_v3.range, get_last(sym), indent);
             break;
 
         case S_CALLSITEINFO:
@@ -1789,18 +1865,24 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
             break;
 
         case S_INLINESITE:
-            printf("Inline-site V3 parent:%x end:%x inlinee:%x\n",
+            printf("Inline-site V3 parent:<%x> end:<%x> inlinee:%x\n",
                    sym->inline_site_v3.pParent, sym->inline_site_v3.pEnd, sym->inline_site_v3.inlinee);
-            dump_binannot(sym->inline_site_v3.binaryAnnotations, get_last(sym), "\t\t");
+            dump_binannot(sym->inline_site_v3.binaryAnnotations, get_last(sym), indent);
+            push_symbol_dumper(&sd, sym, sym->inline_site_v3.pEnd);
             break;
+
         case S_INLINESITE2:
-            printf("Inline-site2 V3 parent:%x end:%x inlinee:%x #inv:%u\n",
+            printf("Inline-site2 V3 parent:<%x> end:<%x> inlinee:%x #inv:%u\n",
                    sym->inline_site2_v3.pParent, sym->inline_site2_v3.pEnd, sym->inline_site2_v3.inlinee,
                    sym->inline_site2_v3.invocations);
-            dump_binannot(sym->inline_site2_v3.binaryAnnotations, get_last(sym), "\t\t");
+            dump_binannot(sym->inline_site2_v3.binaryAnnotations, get_last(sym), indent);
+            push_symbol_dumper(&sd, sym, sym->inline_site2_v3.pEnd);
             break;
+
         case S_INLINESITE_END:
-            printf("Inline-site-end\n");
+            ref = sd.depth ? (const char*)sd.stack[sd.depth - 1].sym - (const char*)root : 0;
+            pop_symbol_dumper(&sd, i);
+            printf("Inline-site-end <%x>\n", ref);
             break;
 
         case S_CALLEES:
@@ -1819,7 +1901,8 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
                 ninvoc = (const unsigned*)get_last(sym) - invoc;
 
                 for (i = 0; i < sym->function_list_v3.count; ++i)
-                    printf("\t\tfunc:%x invoc:%u\n", sym->function_list_v3.funcs[i], i < ninvoc ? invoc[i] : 0);
+                    printf("%*s| func:%x invoc:%u\n",
+                           indent, "", sym->function_list_v3.funcs[i], i < ninvoc ? invoc[i] : 0);
             }
             break;
 
@@ -1847,11 +1930,37 @@ BOOL codeview_dump_symbols(const void* root, unsigned long size)
             printf("UNameSpace V3 '%s'\n", sym->unamespace_v3.name);
             break;
 
+        case S_SEPCODE:
+            printf("SepCode V3 parent:<%x> end:<%x> separated:%04x:%08x (#%u) from %04x:%08x\n",
+                   sym->sepcode_v3.pParent, sym->sepcode_v3.pEnd,
+                   sym->sepcode_v3.sect, sym->sepcode_v3.off, sym->sepcode_v3.length,
+                   sym->sepcode_v3.sectParent, sym->sepcode_v3.offParent);
+            push_symbol_dumper(&sd, sym, sym->sepcode_v3.pEnd);
+            break;
+
+        case S_ANNOTATION:
+            printf("Annotation V3 %04x:%08x\n",
+                   sym->annotation_v3.seg, sym->annotation_v3.off);
+            {
+                const char* ptr = sym->annotation_v3.rgsz;
+                const char* last = ptr + sym->annotation_v3.csz;
+                for (; ptr < last; ptr += strlen(ptr) + 1)
+                    printf("%*s| %s\n", indent, "", ptr);
+            }
+            break;
+
+        case S_POGODATA:
+            printf("PogoData V3 inv:%d dynCnt:%lld inst:%d staInst:%d\n",
+                   sym->pogoinfo_v3.invocations, (long long)sym->pogoinfo_v3.dynCount,
+                   sym->pogoinfo_v3.numInstrs, sym->pogoinfo_v3.staInstLive);
+            break;
+
         default:
             printf("\n\t\t>>> Unsupported symbol-id %x sz=%d\n", sym->generic.id, sym->generic.len + 2);
             dump_data((const void*)sym, sym->generic.len + 2, "  ");
         }
     }
+    dispose_symbol_dumper(&sd);
     return TRUE;
 }
 
@@ -1912,61 +2021,119 @@ void codeview_dump_linetab(const char* linetab, BOOL pascal_str, const char* pfx
 void codeview_dump_linetab2(const char* linetab, DWORD size, const char* strimage, DWORD strsize, const char* pfx)
 {
     unsigned    i;
-    const struct codeview_linetab2*     lt2;
-    const struct codeview_linetab2*     lt2_files = NULL;
-    const struct codeview_lt2blk_lines* lines_blk;
-    const struct codeview_linetab2_file*fd;
+    const struct CV_DebugSSubsectionHeader_t*     hdr;
+    const struct CV_DebugSSubsectionHeader_t*     next_hdr;
+    const struct CV_DebugSLinesHeader_t*          lines_hdr;
+    const struct CV_DebugSLinesFileBlockHeader_t* files_hdr;
+    const struct CV_Column_t*                     cols;
+    const struct CV_Checksum_t*                   chksms;
+    const struct CV_InlineeSourceLine_t*          inlsrc;
+    const struct CV_InlineeSourceLineEx_t*        inlsrcex;
 
-    /* locate LT2_FILES_BLOCK (if any) */
-    lt2 = (const struct codeview_linetab2*)linetab;
-    while ((const char*)(lt2 + 1) < linetab + size)
+    static const char* subsect_types[] = /* starting at 0xf1 */
     {
-        if (lt2->header == LT2_FILES_BLOCK)
+        "SYMBOLS",         "LINES",            "STRINGTABLE",       "FILECHKSMS",
+        "FRAMEDATA",       "INLINEELINES",     "CROSSSCOPEIMPORTS", "CROSSSCOPEEXPORTS",
+        "IL_LINES",        "FUNC_MDTOKEN_MAP", "TYPE_MDTOKEN_MAP",  "MERGED_ASSEMBLYINPUT",
+        "COFF_SYMBOL_RVA"
+        };
+    hdr = (const struct CV_DebugSSubsectionHeader_t*)linetab;
+    while (CV_IS_INSIDE(hdr, linetab + size))
+    {
+        next_hdr = CV_RECORD_GAP(hdr, hdr->cbLen);
+        if (hdr->type & DEBUG_S_IGNORE)
         {
-            lt2_files = lt2;
-            break;
-        }
-        lt2 = codeview_linetab2_next_block(lt2);
-    }
-    if (!lt2_files)
-    {
-        printf("%sNo LT2_FILES_BLOCK found\n", pfx);
-        return;
-    }
-
-    lt2 = (const struct codeview_linetab2*)linetab;
-    while ((const char*)(lt2 + 1) < linetab + size)
-    {
-        /* FIXME: should also check that whole lbh fits in linetab + size */
-        switch (lt2->header)
-        {
-        case LT2_LINES_BLOCK:
-            lines_blk = (const struct codeview_lt2blk_lines*)lt2;
-            printf("%sblock from %04x:%08x #%x (%x lines) fo=%x\n",
-                   pfx, lines_blk->seg, lines_blk->start, lines_blk->size,
-                   lines_blk->nlines, lines_blk->file_offset);
-            /* FIXME: should check that file_offset is within the LT2_FILES_BLOCK we've seen */
-            fd = (const struct codeview_linetab2_file*)((const char*)lt2_files + 8 + lines_blk->file_offset);
-            printf("%s  md5=%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\n",
-                   pfx, fd->md5[ 0], fd->md5[ 1], fd->md5[ 2], fd->md5[ 3],
-                   fd->md5[ 4], fd->md5[ 5], fd->md5[ 6], fd->md5[ 7],
-                   fd->md5[ 8], fd->md5[ 9], fd->md5[10], fd->md5[11],
-                   fd->md5[12], fd->md5[13], fd->md5[14], fd->md5[15]);
-            /* FIXME: should check that string is within strimage + strsize */
-            printf("%s  file=%s\n", pfx, strimage ? strimage + fd->offset : "--none--");
-            for (i = 0; i < lines_blk->nlines; i++)
-            {
-                printf("%s  offset=%08x line=%d\n",
-                       pfx, lines_blk->l[i].offset, lines_blk->l[i].lineno ^ 0x80000000);
-            }
-            break;
-        case LT2_FILES_BLOCK: /* skip */
-            break;
-        default:
-            printf("%sblock end %x\n", pfx, lt2->header);
-            lt2 = (const struct codeview_linetab2*)(linetab + size);
+            printf("%s------ debug subsection <ignored>\n", pfx);
+            hdr = next_hdr;
             continue;
         }
-        lt2 = codeview_linetab2_next_block(lt2);
+
+        printf("%s------ debug subsection %s\n", pfx,
+               (hdr->type >= 0xf1 && hdr->type - 0xf1 < ARRAY_SIZE(subsect_types)) ?
+               subsect_types[hdr->type - 0xf1] : "<<unknown>>");
+        switch (hdr->type)
+        {
+        case DEBUG_S_LINES:
+            lines_hdr = CV_RECORD_AFTER(hdr);
+            printf("%s  block from %04x:%08x flags:%x size=%u\n",
+                   pfx, lines_hdr->segCon, lines_hdr->offCon, lines_hdr->flags, lines_hdr->cbCon);
+            files_hdr = CV_RECORD_AFTER(lines_hdr);
+            /* FIXME: as of today partial coverage:
+             * - only one files_hdr has been seen in PDB files
+             *   OTOH, MS github presents two different structures
+             *   llvm ? one or two
+             * - next files_hdr (depending on previous question, can be a void question)
+             *   is code correct ; what do MS and llvm do?
+             */
+            while (CV_IS_INSIDE(files_hdr, next_hdr))
+            {
+                const struct CV_Line_t* lines;
+                printf("%s    file=%u lines=%d size=%x\n", pfx, files_hdr->offFile,
+                       files_hdr->nLines, files_hdr->cbBlock);
+                lines = CV_RECORD_AFTER(files_hdr);
+                cols = (const struct CV_Column_t*)&lines[files_hdr->nLines];
+                for (i = 0; i < files_hdr->nLines; i++)
+                {
+                    printf("%s      offset=%8x line=%d deltaLineEnd=%u %s",
+                           pfx, lines[i].offset, lines[i].linenumStart, lines[i].deltaLineEnd,
+                           lines[i].fStatement ? "statement" : "expression");
+                    if (lines_hdr->flags & CV_LINES_HAVE_COLUMNS)
+                        printf(" cols[start=%u end=%u]",
+                               cols[i].offColumnStart, cols[i].offColumnEnd);
+                    printf("\n");
+                }
+                if (lines_hdr->flags & CV_LINES_HAVE_COLUMNS)
+                    files_hdr = (const struct CV_DebugSLinesFileBlockHeader_t*)&cols[files_hdr->nLines];
+                else
+                    files_hdr = (const struct CV_DebugSLinesFileBlockHeader_t*)&lines[files_hdr->nLines];
+            }
+            break;
+        case DEBUG_S_FILECHKSMS:
+            chksms = CV_RECORD_AFTER(hdr);
+            while (CV_IS_INSIDE(chksms, next_hdr))
+            {
+                const char* meth[] = {"None", "MD5", "SHA1", "SHA256"};
+                printf("%s  %d] name=%s size=%u method=%s checksum=[",
+                       pfx, (unsigned)((const char*)chksms - (const char*)(hdr + 1)),
+                       strimage ? strimage + chksms->strOffset : "--none--",
+                       chksms->size, chksms->method < ARRAY_SIZE(meth) ? meth[chksms->method] : "<<unknown>>");
+                for (i = 0; i < chksms->size; ++i) printf("%02x", chksms->checksum[i]);
+                printf("]\n");
+                chksms = CV_ALIGN(CV_RECORD_GAP(chksms, chksms->size), 4);
+            }
+            break;
+        case DEBUG_S_INLINEELINES:
+            /* subsection starts with a DWORD signature */
+            switch (*(DWORD*)CV_RECORD_AFTER(hdr))
+            {
+            case CV_INLINEE_SOURCE_LINE_SIGNATURE:
+                inlsrc = CV_RECORD_GAP(hdr, sizeof(DWORD));
+                while (CV_IS_INSIDE(inlsrc, next_hdr))
+                {
+                    printf("%s  inlinee=%x fileref=%d srcstart=%d\n",
+                           pfx, inlsrc->inlinee, inlsrc->fileId, inlsrc->sourceLineNum);
+                    ++inlsrc;
+                }
+                break;
+            case CV_INLINEE_SOURCE_LINE_SIGNATURE_EX:
+                inlsrcex = CV_RECORD_GAP(hdr, sizeof(DWORD));
+                while (CV_IS_INSIDE(inlsrcex, next_hdr))
+                {
+                    printf("%s  inlinee=%x fileref=%d srcstart=%d numExtraFiles=%d\n",
+                           pfx, inlsrcex->inlinee, inlsrcex->fileId, inlsrcex->sourceLineNum,
+                           inlsrcex->countOfExtraFiles);
+                    inlsrcex = CV_RECORD_GAP(inlsrcex, inlsrcex->countOfExtraFiles * sizeof(inlsrcex->extraFileId[0]));
+                }
+                break;
+            default:
+                printf("%sUnknown signature %x in INLINEELINES subsection\n", pfx, *(DWORD*)CV_RECORD_AFTER(hdr));
+                break;
+            }
+            break;
+        default:
+            dump_data(CV_RECORD_AFTER(hdr), hdr->cbLen, pfx);
+            break;
+        }
+        hdr = next_hdr;
     }
 }
