@@ -96,6 +96,11 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <limits.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_SYSCTL_H
+# include <sys/sysctl.h>
+#endif
 
 #include "utils.h"
 
@@ -153,52 +158,11 @@ static const char *includedir;
 
 enum processor { proc_cc, proc_cxx, proc_cpp, proc_as };
 
-static const struct
-{
-    const char *name;
-    enum target_cpu cpu;
-} cpu_names[] =
-{
-    { "i386",    CPU_x86 },
-    { "i486",    CPU_x86 },
-    { "i586",    CPU_x86 },
-    { "i686",    CPU_x86 },
-    { "i786",    CPU_x86 },
-    { "amd64",   CPU_x86_64 },
-    { "x86_64",  CPU_x86_64 },
-    { "powerpc", CPU_POWERPC },
-    { "arm",     CPU_ARM },
-    { "armv5",   CPU_ARM },
-    { "armv6",   CPU_ARM },
-    { "armv7",   CPU_ARM },
-    { "armv7a",  CPU_ARM },
-    { "arm64",   CPU_ARM64 },
-    { "aarch64", CPU_ARM64 },
-};
-
-static const struct
-{
-    const char *name;
-    enum target_platform platform;
-} platform_names[] =
-{
-    { "macos",       PLATFORM_APPLE },
-    { "darwin",      PLATFORM_APPLE },
-    { "android",     PLATFORM_ANDROID },
-    { "solaris",     PLATFORM_SOLARIS },
-    { "cygwin",      PLATFORM_CYGWIN },
-    { "mingw32",     PLATFORM_MINGW },
-    { "windows-gnu", PLATFORM_MINGW },
-    { "windows",     PLATFORM_WINDOWS },
-    { "winnt",       PLATFORM_MINGW }
-};
-
 struct options
 {
     enum processor processor;
-    enum target_cpu target_cpu;
-    enum target_platform target_platform;
-    const char *target;
+    struct target target;
+    const char *target_alias;
     const char *version;
     int shared;
     int use_msvcrt;
@@ -215,6 +179,7 @@ struct options
     int force_pointer_size;
     int large_address_aware;
     int wine_builtin;
+    int fake_module;
     int unwind_tables;
     int strip;
     int pic;
@@ -242,34 +207,6 @@ struct options
     struct strarray delayimports;
 };
 
-#ifdef __i386__
-static const enum target_cpu build_cpu = CPU_x86;
-#elif defined(__x86_64__)
-static const enum target_cpu build_cpu = CPU_x86_64;
-#elif defined(__powerpc__)
-static const enum target_cpu build_cpu = CPU_POWERPC;
-#elif defined(__arm__)
-static const enum target_cpu build_cpu = CPU_ARM;
-#elif defined(__aarch64__)
-static const enum target_cpu build_cpu = CPU_ARM64;
-#else
-#error Unsupported CPU
-#endif
-
-#ifdef __APPLE__
-static enum target_platform build_platform = PLATFORM_APPLE;
-#elif defined(__ANDROID__)
-static enum target_platform build_platform = PLATFORM_ANDROID;
-#elif defined(__sun)
-static enum target_platform build_platform = PLATFORM_SOLARIS;
-#elif defined(__CYGWIN__)
-static enum target_platform build_platform = PLATFORM_CYGWIN;
-#elif defined(_WIN32)
-static enum target_platform build_platform = PLATFORM_MINGW;
-#else
-static enum target_platform build_platform = PLATFORM_UNSPECIFIED;
-#endif
-
 static void cleanup_output_files(void)
 {
     if (output_file_name) unlink( output_file_name );
@@ -314,7 +251,7 @@ static char* get_temp_file(const char* prefix, const char* suffix)
 
 static int is_pe_target( const struct options *opts )
 {
-    switch(opts->target_platform)
+    switch (opts->target.platform)
     {
     case PLATFORM_MINGW:
     case PLATFORM_CYGWIN:
@@ -357,13 +294,13 @@ static struct strarray build_tool_name( struct options *opts, enum tool tool )
     struct strarray ret = empty_strarray;
     char* str;
 
-    if (opts->target && opts->version)
+    if (opts->target_alias && opts->version)
     {
-        str = strmake("%s-%s-%s", opts->target, base, opts->version);
+        str = strmake("%s-%s-%s", opts->target_alias, base, opts->version);
     }
-    else if (opts->target)
+    else if (opts->target_alias)
     {
-        str = strmake("%s-%s", opts->target, base);
+        str = strmake("%s-%s", opts->target_alias, base);
     }
     else if (opts->version)
     {
@@ -386,10 +323,10 @@ static struct strarray build_tool_name( struct options *opts, enum tool tool )
     ret = strarray_fromstring( path, " " );
     if (!strncmp( llvm_base, "clang", 5 ))
     {
-        if (opts->target)
+        if (opts->target_alias)
         {
             strarray_add( &ret, "-target" );
-            strarray_add( &ret, opts->target );
+            strarray_add( &ret, opts->target_alias );
         }
         strarray_add( &ret, "-Wno-unused-command-line-argument" );
         strarray_add( &ret, "-fuse-ld=lld" );
@@ -463,17 +400,12 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
 
     if (verbose > 1) strarray_add( &flags, "-v" );
 
-    switch (opts->target_platform)
+    switch (opts->target.platform)
     {
     case PLATFORM_APPLE:
         strarray_add( &flags, opts->unix_lib ? "-dynamiclib" : "-bundle" );
         strarray_add( &flags, "-multiply_defined" );
         strarray_add( &flags, "suppress" );
-        if (opts->target_cpu == CPU_POWERPC)
-        {
-            strarray_add( &flags, "-read_only_relocs" );
-            strarray_add( &flags, "warning" );
-        }
         if (opts->image_base)
         {
             strarray_add( &flags, "-image_base" );
@@ -522,7 +454,7 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
 
         if (opts->image_base) strarray_add( &flags, strmake("-Wl,--image-base,%s", opts->image_base ));
 
-        if (opts->large_address_aware && opts->target_cpu == CPU_x86)
+        if (opts->large_address_aware && opts->target.cpu == CPU_i386)
             strarray_add( &flags, "-Wl,--large-address-aware" );
 
         /* make sure we don't need a libgcc_s dll on Windows */
@@ -596,7 +528,7 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
 
     strarray_add( &link_args, "-shared" );
     strarray_add( &link_args, "-Wl,-Bsymbolic" );
-    if (!opts->noshortwchar && opts->target_cpu == CPU_ARM)
+    if (!opts->noshortwchar && opts->target.cpu == CPU_ARM)
         strarray_add( &flags, "-Wl,--no-wchar-size-warning" );
     if (!try_link( opts->prefix, link_args, "-Wl,-z,defs" ))
         strarray_add( &flags, "-Wl,-z,defs" );
@@ -605,42 +537,17 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
     return link_args;
 }
 
-static const char *get_multiarch_dir( enum target_cpu cpu )
+static const char *get_multiarch_dir( struct target target )
 {
-   switch(cpu)
+   switch (target.cpu)
    {
-   case CPU_x86:     return "/i386-linux-gnu";
+   case CPU_i386:    return "/i386-linux-gnu";
    case CPU_x86_64:  return "/x86_64-linux-gnu";
    case CPU_ARM:     return "/arm-linux-gnueabi";
    case CPU_ARM64:   return "/aarch64-linux-gnu";
-   case CPU_POWERPC: return "/powerpc-linux-gnu";
-   default:
-       assert(0);
-       return NULL;
    }
-}
-
-static const char *get_wine_arch_dir( enum target_cpu target_cpu, enum target_platform target_platform )
-{
-    const char *cpu;
-
-    switch (target_cpu)
-    {
-    case CPU_x86:     cpu = "i386";     break;
-    case CPU_x86_64:  cpu = "x86_64";   break;
-    case CPU_ARM:     cpu = "arm";      break;
-    case CPU_ARM64:   cpu = "aarch64";  break;
-    default: return "/wine";
-    }
-    switch (target_platform)
-    {
-    case PLATFORM_WINDOWS:
-    case PLATFORM_CYGWIN:
-    case PLATFORM_MINGW:
-        return strmake( "/wine/%s-windows", cpu );
-    default:
-        return strmake( "/wine/%s-unix", cpu );
-    }
+   assert(0);
+   return NULL;
 }
 
 static char *get_lib_dir( struct options *opts )
@@ -652,11 +559,11 @@ static char *get_lib_dir( struct options *opts )
     struct stat st;
     size_t build_len, target_len;
 
-    bit_suffix = opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64 ? "64" : "32";
-    other_bit_suffix = opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64 ? "32" : "64";
-    winecrt0 = strmake( "%s/libwinecrt0.a", get_wine_arch_dir( opts->target_cpu, opts->target_platform ));
-    build_multiarch = get_multiarch_dir( build_cpu );
-    target_multiarch = get_multiarch_dir( opts->target_cpu );
+    bit_suffix = get_target_ptr_size( opts->target ) == 8 ? "64" : "32";
+    other_bit_suffix = get_target_ptr_size( opts->target ) == 8 ? "32" : "64";
+    winecrt0 = strmake( "/wine%s/libwinecrt0.a", get_arch_dir( opts->target ));
+    build_multiarch = get_multiarch_dir( get_default_target() );
+    target_multiarch = get_multiarch_dir( opts->target );
     build_len = strlen( build_multiarch );
     target_len = strlen( target_multiarch );
 
@@ -701,7 +608,8 @@ static char *get_lib_dir( struct options *opts )
             if (*p != '/') break;
 
             /* try s/$build_cpu/$target_cpu/ on multiarch */
-            if (build_cpu != opts->target_cpu && !memcmp( p, build_multiarch, build_len ) && p[build_len] == '/')
+            if (get_default_target().cpu != opts->target.cpu &&
+                !memcmp( p, build_multiarch, build_len ) && p[build_len] == '/')
             {
                 memmove( p + target_len, p + build_len, strlen( p + build_len ) + 1 );
                 memcpy( p, target_multiarch, target_len );
@@ -747,7 +655,12 @@ static void init_argv0_dir( const char *argv0 )
 #if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
     dir = realpath( "/proc/self/exe", NULL );
 #elif defined (__FreeBSD__) || defined(__DragonFly__)
-    dir = realpath( "/proc/curproc/file", NULL );
+    static int pathname[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
+    size_t path_size = PATH_MAX;
+    char *path = malloc( path_size );
+    if (path && !sysctl( pathname, sizeof(pathname)/sizeof(pathname[0]), path, &path_size, NULL, 0 ))
+        dir = realpath( path, NULL );
+    free( path );
 #else
     dir = realpath( argv0, NULL );
 #endif
@@ -790,7 +703,9 @@ static void compile(struct options* opts, const char* lang)
             break;
     }
 
-    if (opts->target_platform == PLATFORM_WINDOWS || opts->target_platform == PLATFORM_CYGWIN || opts->target_platform == PLATFORM_MINGW)
+    if (opts->target.platform == PLATFORM_WINDOWS ||
+        opts->target.platform == PLATFORM_CYGWIN ||
+        opts->target.platform == PLATFORM_MINGW)
         goto no_compat_defines;
 
     if (opts->processor != proc_cpp)
@@ -807,7 +722,7 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(&comp_args, "-fno-PIC");
     }
 
-    if (opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64)
+    if (get_target_ptr_size( opts->target ) == 8)
     {
         strarray_add(&comp_args, "-DWIN64");
         strarray_add(&comp_args, "-D_WIN64");
@@ -824,7 +739,7 @@ static void compile(struct options* opts, const char* lang)
 
     if (gcc_defs)
     {
-        switch (opts->target_cpu)
+        switch (opts->target.cpu)
         {
         case CPU_x86_64:
         case CPU_ARM64:
@@ -832,7 +747,7 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(&comp_args, "-D__cdecl=__stdcall");
             strarray_add(&comp_args, "-D__fastcall=__stdcall");
             break;
-        case CPU_x86:
+        case CPU_i386:
             strarray_add(&comp_args, "-D__stdcall=__attribute__((__stdcall__)) __attribute__((__force_align_arg_pointer__))");
             strarray_add(&comp_args, "-D__cdecl=__attribute__((__cdecl__)) __attribute__((__force_align_arg_pointer__))");
             strarray_add(&comp_args, "-D__fastcall=__attribute__((__fastcall__))");
@@ -842,11 +757,6 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(&comp_args, "-D__cdecl=__stdcall");
             strarray_add(&comp_args, "-D__fastcall=__stdcall");
             break;
-        case CPU_POWERPC:
-            strarray_add(&comp_args, "-D__stdcall=");
-            strarray_add(&comp_args, "-D__cdecl=");
-            strarray_add(&comp_args, "-D__fastcall=");
-            break;
         }
         strarray_add(&comp_args, "-D_stdcall=__stdcall");
         strarray_add(&comp_args, "-D_cdecl=__cdecl");
@@ -869,7 +779,7 @@ static void compile(struct options* opts, const char* lang)
     strarray_add(&comp_args, "-D__int8=char");
     strarray_add(&comp_args, "-D__int16=short");
     strarray_add(&comp_args, "-D__int32=int");
-    if (opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64)
+    if (get_target_ptr_size( opts->target ) == 8)
         strarray_add(&comp_args, "-D__int64=long");
     else
         strarray_add(&comp_args, "-D__int64=long long");
@@ -973,10 +883,10 @@ static struct strarray get_winebuild_args(struct options *opts)
     strarray_add( &spec_args, binary );
     if (verbose) strarray_add( &spec_args, "-v" );
     if (keep_generated) strarray_add( &spec_args, "--save-temps" );
-    if (opts->target)
+    if (opts->target_alias)
     {
         strarray_add( &spec_args, "--target" );
-        strarray_add( &spec_args, opts->target );
+        strarray_add( &spec_args, opts->target_alias );
     }
     for (i = 0; i < opts->prefix.count; i++)
         strarray_add( &spec_args, strmake( "-B%s", opts->prefix.str[i] ));
@@ -1062,7 +972,7 @@ static void add_library( struct options *opts, struct strarray lib_dirs,
 {
     char *static_lib, *fullname = 0;
 
-    switch(get_lib_type(opts->target_platform, lib_dirs, library, "lib", opts->lib_suffix, &fullname))
+    switch(get_lib_type(opts->target, lib_dirs, library, "lib", opts->lib_suffix, &fullname))
     {
     case file_arh:
         strarray_add(files, strmake("-a%s", fullname));
@@ -1093,7 +1003,6 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
     struct strarray spec_args = get_winebuild_args( opts );
     struct strarray tool;
     const char *spec_o_name, *output_name;
-    int fake_module = strendswith(output_file, ".fake");
 
     /* get the filename from the path */
     output_name = get_basename( output_file );
@@ -1111,7 +1020,7 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
         strarray_add(&spec_args, strmake("-m%u", 8 * opts->force_pointer_size ));
     if (opts->pic && !is_pe) strarray_add(&spec_args, "-fPIC");
     strarray_add(&spec_args, opts->shared ? "--dll" : "--exe");
-    if (fake_module)
+    if (opts->fake_module)
     {
         strarray_add(&spec_args, "--fake-module");
         strarray_add(&spec_args, "-o");
@@ -1137,7 +1046,7 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
         if (opts->large_address_aware) strarray_add( &spec_args, "--large-address-aware" );
     }
 
-    if (opts->target_platform == PLATFORM_WINDOWS) strarray_add(&spec_args, "--safeseh");
+    if (opts->target.platform == PLATFORM_WINDOWS) strarray_add(&spec_args, "--safeseh");
 
     if (entry_point)
     {
@@ -1192,7 +1101,6 @@ static void build(struct options* opts)
     const char *output_name, *spec_file, *lang;
     int generate_app_loader = 1;
     const char *crt_lib = NULL, *entry_point = NULL;
-    int fake_module = 0;
     int is_pe = is_pe_target( opts );
     unsigned int j;
 
@@ -1219,7 +1127,7 @@ static void build(struct options* opts)
     if (opts->shared || is_pe || strendswith(output_file, ".so"))
 	generate_app_loader = 0;
 
-    if (strendswith(output_file, ".fake")) fake_module = 1;
+    if (strendswith(output_file, ".fake")) opts->fake_module = 1;
 
     /* normalize the filename a bit: strip .so, ensure it has proper ext */
     if (!strchr(get_basename( output_file ), '.'))
@@ -1236,8 +1144,7 @@ static void build(struct options* opts)
     {
         char *lib_dir = get_lib_dir( opts );
         strarray_addall( &lib_dirs, opts->lib_dirs );
-        strarray_add( &lib_dirs, strmake( "%s%s", lib_dir,
-                                         get_wine_arch_dir( opts->target_cpu, opts->target_platform )));
+        strarray_add( &lib_dirs, strmake( "%s/wine%s", lib_dir, get_arch_dir( opts->target )));
         strarray_add( &lib_dirs, lib_dir );
     }
     else
@@ -1311,10 +1218,6 @@ static void build(struct options* opts)
 	}
         add_library(opts, lib_dirs, &files, "advapi32");
         add_library(opts, lib_dirs, &files, "user32");
-    }
-
-    if (!opts->nodefaultlibs && !opts->unix_lib)
-    {
         add_library(opts, lib_dirs, &files, "winecrt0");
         if (opts->use_msvcrt)
         {
@@ -1336,7 +1239,7 @@ static void build(struct options* opts)
     if (!opts->entry_point)
     {
         if (opts->subsystem && !opts->unix_lib && !strcmp( opts->subsystem, "native" ))
-            entry_point = (is_pe && opts->target_cpu == CPU_x86) ? "DriverEntry@8" : "DriverEntry";
+            entry_point = (is_pe && opts->target.cpu == CPU_i386) ? "DriverEntry@8" : "DriverEntry";
         else if (opts->use_msvcrt && !opts->shared && !opts->win16_app)
             entry_point = opts->unicode_app ? "wmainCRTStartup" : "mainCRTStartup";
     }
@@ -1346,12 +1249,12 @@ static void build(struct options* opts)
     if (spec_file || !opts->unix_lib)
         spec_o_name = build_spec_obj( opts, spec_file, output_file, files, lib_dirs, entry_point );
 
-    if (fake_module) return;  /* nothing else to do */
+    if (opts->fake_module) return;  /* nothing else to do */
 
     /* link everything together now */
     link_args = get_link_args( opts, output_name );
 
-    if ((opts->nodefaultlibs || opts->use_msvcrt) && opts->target_platform == PLATFORM_MINGW)
+    if ((opts->nodefaultlibs || opts->use_msvcrt) && opts->target.platform == PLATFORM_MINGW)
     {
         libgcc = find_libgcc(opts->prefix, link_args);
         if (!libgcc) libgcc = "-lgcc";
@@ -1364,15 +1267,15 @@ static void build(struct options* opts)
 	strarray_add(&link_args, strmake("-L%s", lib_dirs.str[j]));
 
     if (is_pe && opts->use_msvcrt && !entry_point && (opts->shared || opts->win16_app))
-        entry_point = opts->target_cpu == CPU_x86 ? "DllMainCRTStartup@12" : "DllMainCRTStartup";
+        entry_point = opts->target.cpu == CPU_i386 ? "DllMainCRTStartup@12" : "DllMainCRTStartup";
 
     if (is_pe && entry_point)
     {
-        if (opts->target_platform == PLATFORM_WINDOWS)
+        if (opts->target.platform == PLATFORM_WINDOWS)
             strarray_add(&link_args, strmake("-Wl,-entry:%s", entry_point));
         else
             strarray_add(&link_args, strmake("-Wl,--entry,%s%s",
-                                            is_pe && opts->target_cpu == CPU_x86 ? "_" : "",
+                                            is_pe && opts->target.cpu == CPU_i386 ? "_" : "",
                                             entry_point));
     }
 
@@ -1382,7 +1285,7 @@ static void build(struct options* opts)
     {
         for (j = 0; j < opts->delayimports.count; j++)
         {
-            if (opts->target_platform == PLATFORM_WINDOWS)
+            if (opts->target.platform == PLATFORM_WINDOWS)
                 strarray_add(&link_args, strmake("-Wl,-delayload:%s", opts->delayimports.str[j]));
             else
                 strarray_add(&link_args, strmake("-Wl,-delayload,%s",opts->delayimports.str[j]));
@@ -1402,18 +1305,28 @@ static void build(struct options* opts)
 		strarray_add(&link_args, name);
 		break;
 	    case 'a':
-                if (is_pe && !opts->use_msvcrt && !opts->lib_suffix && strchr(name, '/'))
+                if (!opts->use_msvcrt && !opts->lib_suffix && strchr(name, '/'))
                 {
                     /* turn the path back into -Ldir -lfoo options
                      * this makes sure that we use the specified libs even
                      * when mingw adds its own import libs to the link */
                     const char *p = get_basename( name );
 
-                    if (!strncmp( p, "lib", 3 ) && strcmp( p, "libmsvcrt.a" ))
+                    if (is_pe)
                     {
-                        strarray_add(&link_args, strmake("-L%s", get_dirname(name) ));
-                        strarray_add(&link_args, strmake("-l%s", get_basename_noext( p + 3 )));
-                        break;
+                        if (!strncmp( p, "lib", 3 ) && strcmp( p, "libmsvcrt.a" ))
+                        {
+                            strarray_add(&link_args, strmake("-L%s", get_dirname(name) ));
+                            strarray_add(&link_args, strmake("-l%s", get_basename_noext( p + 3 )));
+                            break;
+                        }
+                    }
+                    else
+                    {
+                        /* don't link to ntdll or ntoskrnl in non-msvcrt mode
+                         * since they export CRT functions */
+                        if (!strcmp( p, "libntdll.a" )) break;
+                        if (!strcmp( p, "libntoskrnl.a" )) break;
                     }
                 }
 		strarray_add(&link_args, name);
@@ -1489,7 +1402,7 @@ static void build(struct options* opts)
     }
 
     /* set the base address with prelink if linker support is not present */
-    if (opts->prelink && !opts->target)
+    if (opts->prelink && !opts->target_alias)
     {
         if (opts->prelink[0] && strcmp(opts->prelink,"false"))
         {
@@ -1561,54 +1474,8 @@ static int is_linker_arg(const char* arg)
 
 static void parse_target_option( struct options *opts, const char *target )
 {
-    char *p, *spec = xstrdup( target );
-    unsigned int i;
-
-    /* target specification is in the form CPU-MANUFACTURER-OS or CPU-MANUFACTURER-KERNEL-OS */
-
-    /* get the CPU part */
-
-    if ((p = strchr( spec, '-' )))
-    {
-        *p++ = 0;
-        for (i = 0; i < ARRAY_SIZE(cpu_names); i++)
-        {
-            if (!strcmp( cpu_names[i].name, spec ))
-            {
-                opts->target_cpu = cpu_names[i].cpu;
-                break;
-            }
-        }
-        if (i == ARRAY_SIZE(cpu_names))
-            error( "Unrecognized CPU '%s'\n", spec );
-    }
-    else if (!strcmp( spec, "mingw32" ))
-    {
-        opts->target_cpu = CPU_x86;
-        p = spec;
-    }
-    else
-        error( "Invalid target specification '%s'\n", target );
-
-    /* get the OS part */
-
-    opts->target_platform = PLATFORM_UNSPECIFIED;  /* default value */
-    for (;;)
-    {
-        for (i = 0; i < ARRAY_SIZE(platform_names); i++)
-        {
-            if (!strncmp( platform_names[i].name, p, strlen(platform_names[i].name) ))
-            {
-                opts->target_platform = platform_names[i].platform;
-                break;
-            }
-        }
-        if (opts->target_platform != PLATFORM_UNSPECIFIED || !(p = strchr( p, '-' ))) break;
-        p++;
-    }
-
-    free( spec );
-    opts->target = xstrdup( target );
+    opts->target_alias = xstrdup( target );
+    if (!parse_target( target, &opts->target )) error( "Invalid target specification '%s'\n", target );
 }
 
 static int is_option( struct options *opts, int i, const char *option, const char **option_arg )
@@ -1654,8 +1521,7 @@ int main(int argc, char **argv)
 
     /* initialize options */
     memset(&opts, 0, sizeof(opts));
-    opts.target_cpu = build_cpu;
-    opts.target_platform = build_platform;
+    opts.target = init_argv0_target( argv[0] );
     opts.pic = 1;
 
     /* determine the processor type */
@@ -1867,19 +1733,13 @@ int main(int argc, char **argv)
                     }
 		    else if (strcmp("-m32", opts.args.str[i]) == 0)
                     {
-                        if (opts.target_cpu == CPU_x86_64)
-                            opts.target_cpu = CPU_x86;
-                        else if (opts.target_cpu == CPU_ARM64)
-                            opts.target_cpu = CPU_ARM;
+                        set_target_ptr_size( &opts.target, 4 );
                         opts.force_pointer_size = 4;
 			raw_linker_arg = 1;
                     }
 		    else if (strcmp("-m64", opts.args.str[i]) == 0)
                     {
-                        if (opts.target_cpu == CPU_x86)
-                            opts.target_cpu = CPU_x86_64;
-                        else if (opts.target_cpu == CPU_ARM)
-                            opts.target_cpu = CPU_ARM64;
+                        set_target_ptr_size( &opts.target, 8 );
                         opts.force_pointer_size = 8;
 			raw_linker_arg = 1;
                     }
@@ -1927,7 +1787,7 @@ int main(int argc, char **argv)
 			opts.shared = 1;
                         raw_compiler_arg = raw_linker_arg = 0;
 		    }
-                    else if (strcmp("-s", opts.args.str[i]) == 0 && opts.target_platform == PLATFORM_APPLE)
+                    else if (strcmp("-s", opts.args.str[i]) == 0 && opts.target.platform == PLATFORM_APPLE)
                     {
                         /* On Mac, change -s into -Wl,-x. ld's -s switch
                          * is deprecated, and it doesn't work on Tiger with
@@ -2019,9 +1879,13 @@ int main(int argc, char **argv)
                     }
 		    else if (strncmp("-Wb,", opts.args.str[i], 4) == 0)
 		    {
-                        strarray_addall( &opts.winebuild_args,
-                                         strarray_fromstring( opts.args.str[i] + 4, ",") );
-                        /* don't pass it to the compiler, it generates errors */
+                        unsigned int j;
+                        struct strarray Wb = strarray_fromstring(opts.args.str[i] + 4, ",");
+                        for (j = 0; j < Wb.count; j++)
+                        {
+                            if (!strcmp(Wb.str[j], "--fake-module")) opts.fake_module = 1;
+                            else strarray_add( &opts.winebuild_args, Wb.str[j] );
+                        }
                         raw_compiler_arg = raw_linker_arg = 0;
 		    }
                     break;
@@ -2091,12 +1955,13 @@ int main(int argc, char **argv)
 	}
     }
 
+    if (opts.force_pointer_size) set_target_ptr_size( &opts.target, opts.force_pointer_size );
     if (opts.processor == proc_cpp) linking = 0;
     if (linking == -1) error("Static linking is not supported\n");
 
     if (!opts.wine_objdir && is_pe_target( &opts )) opts.use_msvcrt = 1;
 
-    if (opts.files.count == 0) forward(&opts);
+    if (opts.files.count == 0 && !opts.fake_module) forward(&opts);
     else if (linking) build(&opts);
     else compile(&opts, lang);
 
